package viptool.base.editor;

import viptool.base.io.IFileInterpreter;

public interface IEditorManager
{
    // <><><><><><><><><><><><><><><><><><><><><>
    // adding and removing file interpreter providers
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Adds a new editor to the editor manager. Editor providers are used to
     * determine the correct editor for a given file interpreter.
     * 
     * @param provider
     *            the new editor provider
     */
    public void addEditorProvider(IEditorProvider provider);

    /**
     * Removes the given editor provider from the editor manager. The editor can
     * no longer be used in viptool now.
     * 
     * @param provider
     *            the editor provider that should be removed from the manager
     */
    public void removeEditorProvider(IEditorProvider provider);

    // <><><><><><><><><><><><><><><><><><><><><>
    // get editor providers
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns the default editor for the given file interpreter. With the help
     * of the editor provider an editor can be created that can display the
     * given file interpreter.
     * 
     * @param fileInterpreter
     *            the file interpreter for which the editor provider is needed
     * @return the default editor provider for the given file interpreter
     */
    public IEditorProvider getDefaultEditorProvider(
            IFileInterpreter fileInterpreter);

    /**
     * Returns all editor providers that would accept the given file
     * interpreter. If the given file interpreter isn't accepted by any
     * provider, an empty array will be returned. The array is sorted by the
     * priority of each editor provider. This means that the default editor will
     * be at the first index zero.
     * 
     * @param fileInterpreter
     *            the file interpreter for which editor providers are searched
     * @return an array containing all editor providers that would accept the
     *         given file interpreter
     */
    public IEditorProvider[] getMatchingEditorProviders(
            IFileInterpreter fileInterpreter);

    /**
     * Returns an array that contains all editor providers.
     * 
     * @return an array of all editor providers
     */
    public IEditorProvider[] getEditorProviders();

    /**
     * Returns the editor provider instance with the given class.
     * 
     * @param clazz
     *            the class of the editor provider to return
     * @return the editor provider matching the given class or null
     */
    public IEditorProvider getEditorProvider(Class clazz);
}
