package viptool.base.event;

import viptool.base.IWorkbench;
import viptool.project.IWorkspace;

/**
 * This is the event object for workspace change events. It holds the previous
 * workspace (this is the workspace before the change was made), and the current
 * workspace. Note, that the previous workspace as well as the current workspace
 * may be null.
 */
public class WorkspaceChangeEvent extends WorkbenchEvent
{
    // the previous (old) workspace
    private IWorkspace oldWorkspace_;

    // the current (new) workspace
    private IWorkspace newWorkspace_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Creates a new workspace change event that holds several information about
     * the workspace change.
     * 
     * @param workbench
     *            the workbench that fired the event
     * @param oldWorkspace
     *            the previous workspace
     * @param newWorkspace
     *            the current workspace
     */
    public WorkspaceChangeEvent(IWorkbench workbench, IWorkspace oldWorkspace,
            IWorkspace newWorkspace)
    {
        super(workbench);

        oldWorkspace_ = oldWorkspace;
        newWorkspace_ = newWorkspace;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // getter methods
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns the previous workspace. This is the workspace that was loaded
     * before the new one.
     * 
     * @return the previously loaded workspace
     */
    public IWorkspace getPreviousWorkspace()
    {
        return oldWorkspace_;
    }

    /**
     * Returns the current workspace. This workspace has 'replaced' the old one.
     * 
     * @return the currently loaded workspace
     */
    public IWorkspace getCurrentWorkspace()
    {
        return newWorkspace_;
    }
}
