package viptool.base.gui;

import javax.swing.JMenuBar;
import javax.swing.JToolBar;

/**
 * All child windows that should be managed by the extended pane must implement
 * this interface. The interface provides some methods to get information about
 * the child windows properties.
 */
public interface IChildWindow extends IWindow
{

    /**
     * Gets the menu for this window. The menu is displayed in the mainframe
     * whenever the window is activated.
     * 
     * @return a reference to the menu
     */
    public JMenuBar getMenuBar();

    /**
     * Gets the toolbar for this window. The toolbar is displayed in the
     * mainframe whenever the window is activated. It can be a per-user setting
     * to display the toolbar in the parent tab as well.
     * 
     * @return a reference to the toolbar
     */
    public JToolBar getToolBar();

    /**
     * Gets the tooltip text of this window. The tooltip text is displayed if
     * the user is over the window.
     * 
     * @return a string representing the windows tooltip text
     */
    public String getToolTipText();

    /**
     * Returns whether the window is ready to close or not. In this method a
     * window can perform resource releases, save operations or other to get
     * ready for closing. The force argument tells will tell the window if the
     * answer can cancel the closing operation.<br>
     * This method should only be called if the window is really closed after
     * return of the method.
     * 
     * @params force is set to false if the window can cancel the closing
     *         operation. If force is true, the window will be closed after
     *         returning regardless of the return code.
     */
    public boolean canClose(boolean force);

    public void windowActivated();
}
