package viptool.base.gui.component;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.HashSet;
import java.util.Iterator;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import viptool.base.IWorkbench;
import viptool.base.gui.component.FolderTreePanel.FolderTreeSelection;
import viptool.base.gui.component.event.FolderTreePanelSelectionEvent;
import viptool.base.gui.component.event.IFileNamePanelListener;
import viptool.base.gui.component.event.IFolderTreePanelListener;
import viptool.util.Locale;
import viptool.util.resources.ResourceConstants;

public class FileNamePanel extends JPanel implements IFolderTreePanelListener
{
    // the label (Name)
    private JLabel              nameLabel_    = null;

    // the textfield where to enter the name
    private JTextField          name_         = null;

    // the folder panel
    private FolderTreePanel     folderPanel_  = null;

    // the button for creating a new folder
    private JButton             newFolder_    = null;

    // the label for the folder box
    private JLabel              folderLabel_  = null;

    // the current workbench
    private IWorkbench          workbench_    = null;

    // the set of listeners
    private HashSet             listeners_    = null;

    // the current selected path
    private FolderTreeSelection selectedPath_ = null;

    // <><><><><><><><><><><><><><><><><><><><><>
    // inner classes
    // <><><><><><><><><><><><><><><><><><><><><>

    private class NewFolderAction extends AbstractAction
    {
        public NewFolderAction()
        {
            if (ResourceConstants.ICON_NEW_FOLDER != null)
            {
                putValue(AbstractAction.SMALL_ICON,
                        workbench_.getResourceLoader().getIcon(
                                ResourceConstants.ICON_NEW_FOLDER));
            }
            if (Locale.ACCELERATOR_KEY_NEW_FOLDER != null)
            {
                putValue(AbstractAction.ACCELERATOR_KEY,
                        Locale.ACCELERATOR_KEY_NEW_FOLDER);
            }
            if (Locale.MENUITEMNAME_NEW_FOLDER != null)
            {
                putValue(AbstractAction.MNEMONIC_KEY,
                        Locale.MNEMONIC_KEY_NEW_FOLDER);
            }

            putValue(AbstractAction.SHORT_DESCRIPTION,
                    Locale.SHORT_DESCRIPTION_NEW_FOLDER);
            putValue(AbstractAction.LONG_DESCRIPTION,
                    Locale.LONG_DESCRIPTION_NEW_FOLDER);
            putValue(AbstractAction.NAME, Locale.MENUITEMNAME_NEW_FOLDER);
        }

        public void actionPerformed(ActionEvent e)
        {
            String name = JOptionPane.showInputDialog(
                    workbench_.getMainFrame(), "Enter name for new Folder:",
                    "new Folder:", JOptionPane.QUESTION_MESSAGE);

            if (name != null && name.length() > 0)
            {
                folderPanel_.createFolder(name);
            }
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * This is the default constructor
     */
    public FileNamePanel(IWorkbench workbench)
    {
        this(workbench, null);
    }

    public FileNamePanel(IWorkbench workbench, Object select)
    {
        this(workbench, null, "");
    }

    public FileNamePanel(IWorkbench workbench, Object select,
            String defaultFolder)
    {
        super();

        workbench_ = workbench;
        listeners_ = new HashSet();

        GridBagConstraints newFolderConstraints = new GridBagConstraints();
        newFolderConstraints.gridx = 1;
        newFolderConstraints.anchor = GridBagConstraints.EAST;
        newFolderConstraints.gridy = 3;
        newFolderConstraints.insets = new Insets(0, 0, 5, 5);

        GridBagConstraints folderLabelConstraints = new GridBagConstraints();
        folderLabelConstraints.gridx = 0;
        folderLabelConstraints.gridy = 1;
        folderLabelConstraints.gridwidth = 2;
        folderLabelConstraints.fill = GridBagConstraints.HORIZONTAL;
        folderLabelConstraints.weightx = 1.0;
        folderLabelConstraints.insets = new Insets(5, 5, 5, 5);

        GridBagConstraints folderConstraints = new GridBagConstraints();
        folderConstraints.gridx = 0;
        folderConstraints.fill = GridBagConstraints.BOTH;
        folderConstraints.gridwidth = 2;
        folderConstraints.gridy = 2;
        folderConstraints.weightx = 1.0;
        folderConstraints.weighty = 1.0;
        folderConstraints.insets = new Insets(0, 5, 5, 5);

        GridBagConstraints nameConstraints = new GridBagConstraints();
        nameConstraints.fill = GridBagConstraints.BOTH;
        nameConstraints.gridy = 0;
        nameConstraints.weightx = 1.0;
        nameConstraints.gridx = 1;
        nameConstraints.insets = new Insets(5, 5, 3, 5);
        nameConstraints.anchor = GridBagConstraints.CENTER;

        GridBagConstraints labelConstraints = new GridBagConstraints();
        labelConstraints.gridx = 0;
        labelConstraints.gridy = 0;
        labelConstraints.insets = new Insets(5, 5, 3, 0);

        nameLabel_ = new JLabel("Name");

        name_ = new JTextField();
        name_.getDocument().addDocumentListener(new DocumentListener()
        {
            public void changedUpdate(DocumentEvent e)
            {
                validateFields();
            }

            public void insertUpdate(DocumentEvent e)
            {
                validateFields();
            }

            public void removeUpdate(DocumentEvent e)
            {
                validateFields();
            }
        });
        name_.addFocusListener(new FocusListener()
        {
            public void focusGained(FocusEvent e)
            {
                fireFocusGainedOnName();
            }

            public void focusLost(FocusEvent e)
            {
                fireFocusLostEvent();
            }
        });

        newFolder_ = new JButton(new NewFolderAction());

        folderLabel_ = new JLabel("Create new file in the following folder");
        folderPanel_ = new FolderTreePanel(workbench);
        folderPanel_.addFolderTreePanelListener(this);

        setLayout(new GridBagLayout());

        add(nameLabel_, labelConstraints);
        add(name_, nameConstraints);
        add(folderLabel_, folderLabelConstraints);
        add(folderPanel_, folderConstraints);
        add(newFolder_, newFolderConstraints);

        if (select != null)
        {
            folderPanel_.selectItem(select);
            if (defaultFolder != null && defaultFolder.length() > 0)
            {
                folderPanel_.createFolder(defaultFolder);
            }
        }
    }

    /**
     * This method initializes this
     * 
     * @return void
     */
    public void initialize(Object select, String subfolder)
    {
        folderPanel_.selectItem(select);
        if (subfolder != null && subfolder.length() > 0)
        {
            folderPanel_.createFolder(subfolder);
        }

        initialize();
    }

    public void initialize()
    {
        if (name_.getParent() != null)
        {
            name_.requestFocus();
        }
        else
        {
            folderPanel_.requestFocus();
        }
    }

    public String getEnteredName()
    {
        return name_.getText();
    }

    public FolderTreeSelection getSelectedPath()
    {
        return selectedPath_;
    }

    public void disableNewFolder()
    {
        remove(newFolder_);

        invalidate();
        validate();
    }

    public void disableEnterName()
    {
        remove(nameLabel_);
        remove(name_);

        invalidate();
        validate();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Supporting methods
    // <><><><><><><><><><><><><><><><><><><><><>

    private void validateFields()
    {
        if (name_.getParent() != null)
        {
            if (name_.getText().length() > 0 && selectedPath_ != null)
            {
                fireValidated();
            }
            else
            {
                fireNameNotValid();
            }
        }
        else
        {
            if (selectedPath_ != null)
            {
                fireValidated();
            }
            else
            {
                fireNameNotValid();
            }
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // listener support
    // <><><><><><><><><><><><><><><><><><><><><>

    public void addFileNamePanelListener(IFileNamePanelListener listener)
    {
        if (listener != null)
        {
            listeners_.add(listener);
        }
    }

    public void removeFileNamePanelListener(IFileNamePanelListener listener)
    {
        listeners_.remove(listener);
    }

    protected void fireFocusLostEvent()
    {
        Iterator it;

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IFileNamePanelListener) it.next()).focusLost();
        }
    }

    protected void fireFocusGainedOnName()
    {
        Iterator it;

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IFileNamePanelListener) it.next()).focusGainedOnName();
        }
    }

    protected void fireNameNotValid()
    {
        Iterator it;

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IFileNamePanelListener) it.next()).nameNotValid();
        }
    }

    protected void fireValidated()
    {
        Iterator it;

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IFileNamePanelListener) it.next()).validated();
        }
    }

    protected void fireFocusGainedOnPath()
    {
        Iterator it;

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IFileNamePanelListener) it.next()).focusGainedOnPathTree();
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IFolderTreePanelListener interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.base.gui.component.event.IFolderTreePanelListener */
    public void focusGained(FocusEvent event)
    {
        fireFocusGainedOnPath();
    }

    /* @see viptool.base.gui.component.event.IFolderTreePanelListener */
    public void focusLost(FocusEvent event)
    {
        fireFocusLostEvent();
    }

    /* @see viptool.base.gui.component.event.IFolderTreePanelListener */
    public void itemSelected(FolderTreePanelSelectionEvent event)
    {
        selectedPath_ = event.getSelection();
        validateFields();

        if (selectedPath_ != null)
        {
            newFolder_.setEnabled(!selectedPath_.isVirtualFolder());
        }
    }
}
