package viptool.base.gui.component;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import viptool.base.gui.component.event.IPathBrowserPanelListener;
import viptool.util.resources.ResourceConstants;

/**
 * This panel displays a name field and a path browser and can be embedded into
 * dialogs to give the user an ability to set name and system wide path of a new
 * file.
 */
public final class PathBrowserPanel extends JPanel implements DocumentListener,
        ItemListener, FocusListener, PropertyChangeListener
{
    // the label for the name input field
    private JLabel        nameLabel_         = null;

    // the input field for the name
    private JTextField    nameField_         = null;

    // checkbox for default path
    private JCheckBox     defaultPathBox_    = null;

    // the label for the path input
    private JLabel        pathLabel_         = null;

    // the input field for the path
    private JTextField    pathField_         = null;

    // button to browse a path
    private JToggleButton browseButton_      = null;

    // the file chooser for browsing
    private JFileChooser  pathBrowser_       = null;

    // panel for filling up extra space so components are layoutet on
    // top of the dialog
    private JPanel        fillPanel_         = null;

    // flag indication the path browser changes the path field
    private boolean       changePathField_   = false;

    // flag indication the path field changes the path browser
    private boolean       changePathBrowser_ = false;

    // a flag indicating that the validation is in progress
    private boolean       validating_        = false;

    // a set for the path browser panel listeners
    private Set           listeners_         = new HashSet();

    // the default path
    private String        defaultPath_       = null;

    // flag if the path should be extended by the name
    private boolean       extendPath_        = false;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Creates a new path browser panel. The path browser panel can be embedded
     * into dialogs and gives the user the ability to select a name and a path
     * for new files.
     */
    public PathBrowserPanel()
    {
        super();

        GridBagConstraints nameLabelConstraints;
        GridBagConstraints nameFieldConstraints;
        GridBagConstraints defaultPathConstraints;
        GridBagConstraints pathLabelConstraints;
        GridBagConstraints pathFieldConstraints;
        GridBagConstraints browseButtonConstraints;
        GridBagConstraints pathBrowserConstraints;
        GridBagConstraints fillPanelConstraints;
        GridBagLayout gridBagLayout;

        // Create the components

        // name label
        nameLabel_ = new JLabel();
        nameLabel_.setText("Name");

        // name field
        nameField_ = new JTextField();
        nameField_.setPreferredSize(new Dimension(30, 25));
        nameField_.setMinimumSize(new Dimension(10, 25));

        // default path
        defaultPathBox_ = new JCheckBox("Use default path", true);

        // path label
        pathLabel_ = new JLabel();
        pathLabel_.setText("Path");

        // path field
        pathField_ = new JTextField();
        pathField_.setPreferredSize(new Dimension(20, 25));
        pathField_.setEditable(false);

        // browse button
        browseButton_ = new JToggleButton("Browse...");
        browseButton_.setEnabled(false);

        // path browser
        pathBrowser_ = new JFileChooser();
        pathBrowser_.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        pathBrowser_.setVisible(false);
        pathBrowser_.setControlButtonsAreShown(false);

        // fill panel
        fillPanel_ = new JPanel();

        // Create layout constraints
        // name label
        nameLabelConstraints = new GridBagConstraints();
        nameLabelConstraints.gridx = 0;
        nameLabelConstraints.anchor = GridBagConstraints.WEST;
        nameLabelConstraints.insets = new Insets(5, 5, 3, 0);
        nameLabelConstraints.gridy = 0;

        // name field
        nameFieldConstraints = new GridBagConstraints();
        nameFieldConstraints.fill = GridBagConstraints.BOTH;
        nameFieldConstraints.gridy = 0;
        nameFieldConstraints.gridwidth = 2;
        nameFieldConstraints.insets = new Insets(5, 5, 3, 5);
        nameFieldConstraints.gridx = 1;
        nameFieldConstraints.weightx = 1;

        // default path
        defaultPathConstraints = new GridBagConstraints();
        defaultPathConstraints.gridx = 0;
        defaultPathConstraints.gridy = 1;
        defaultPathConstraints.gridwidth = 3;
        defaultPathConstraints.anchor = GridBagConstraints.WEST;
        defaultPathConstraints.fill = GridBagConstraints.BOTH;
        defaultPathConstraints.insets = new Insets(12, 2, 3, 5);

        // path label
        pathLabelConstraints = new GridBagConstraints();
        pathLabelConstraints.gridx = 0;
        pathLabelConstraints.gridy = 2;
        pathLabelConstraints.anchor = GridBagConstraints.WEST;
        pathLabelConstraints.insets = new Insets(0, 15, 3, 0);

        // path field
        pathFieldConstraints = new GridBagConstraints();
        pathFieldConstraints.fill = GridBagConstraints.BOTH;
        pathFieldConstraints.gridy = 2;
        pathFieldConstraints.insets = new Insets(0, 5, 0, 5);
        pathFieldConstraints.gridx = 1;
        pathFieldConstraints.weightx = 1;

        // browse button
        browseButtonConstraints = new GridBagConstraints();
        browseButtonConstraints.gridx = 2;
        browseButtonConstraints.gridy = 2;
        browseButtonConstraints.insets = new Insets(0, 0, 0, 5);

        // path browser
        pathBrowserConstraints = new GridBagConstraints();
        pathBrowserConstraints.gridx = 0;
        pathBrowserConstraints.gridy = 3;
        pathBrowserConstraints.gridwidth = 3;
        pathBrowserConstraints.insets = new Insets(0, 5, 0, 5);
        pathBrowserConstraints.fill = GridBagConstraints.BOTH;
        pathBrowserConstraints.weighty = 1.0;

        // fill panel
        fillPanelConstraints = new GridBagConstraints();
        fillPanelConstraints.gridx = 0;
        fillPanelConstraints.gridy = 4;
        fillPanelConstraints.gridwidth = 3;
        fillPanelConstraints.fill = GridBagConstraints.BOTH;
        fillPanelConstraints.weighty = 1.0;

        // layout
        gridBagLayout = new GridBagLayout();

        // initialize the content
        setLayout(gridBagLayout);
        add(nameLabel_, nameLabelConstraints);
        add(nameField_, nameFieldConstraints);
        add(defaultPathBox_, defaultPathConstraints);
        add(pathLabel_, pathLabelConstraints);
        add(pathField_, pathFieldConstraints);
        add(browseButton_, browseButtonConstraints);
        add(pathBrowser_, pathBrowserConstraints);
        add(fillPanel_, fillPanelConstraints);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // getter methods
    // <><><><><><><><><><><><><><><><><><><><><>
    public String getSelectedPath()
    {
        String path = pathField_.getText();

        // remove the / or \ at the end if there is one
        if(path.endsWith(File.separator))
        {
            path = path.substring(0, path.length() - 1);
        }

        return path;
    }

    public String getSelectedName()
    {
        return nameField_.getText();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // supporting methods
    // <><><><><><><><><><><><><><><><><><><><><>
    public void initialize(String defaultPath, boolean extendPath)
    {
        extendPath_ = extendPath;
        defaultPath_ = defaultPath;

        // initialize
        validateValues();
        setNewPath(defaultPath_);
        updatePathBrowser();

        // add listeners
        nameField_.getDocument().addDocumentListener(this);
        nameField_.addFocusListener(this);

        pathField_.addFocusListener(this);
        pathField_.getDocument().addDocumentListener(this);

        defaultPathBox_.addItemListener(this);
        browseButton_.addItemListener(this);

        pathBrowser_.addPropertyChangeListener(
                JFileChooser.DIRECTORY_CHANGED_PROPERTY, this);

        pathBrowser_.addPropertyChangeListener(
                JFileChooser.SELECTED_FILE_CHANGED_PROPERTY, this);

        nameField_.requestFocus();
    }

    private void validateValues()
    {
        if(validating_)
        {
            return;
        }

        validating_ = true;

        // file checking
        if(!validateName())
        {
            fireNameNotValidEvent();
            validating_ = false;
            return;
        }

        // path checking
        if(!validatePath())
        {
            firePathNotValidEvent();
            validating_ = false;
            return;
        }

        // inform the listeners that everything is ok now
        fireValidatedEvent();

        validating_ = false;
    }

    /**
     * Sets the path browser to the path that is given in the text path field.
     * This method is ignored if the field changePathField_ is set to true. This
     * might be in the case that the program changes the field.
     */
    private void updatePathBrowser()
    {
        String path;
        File folder;

        if(changePathField_)
        {
            return;
        }

        if(extendPath_)
        {
            path = pathField_.getText().substring(0,
                    pathField_.getText().lastIndexOf(File.separatorChar));
        }
        else
        {
            path = pathField_.getText();
        }

        folder = new File(path);

        changePathBrowser_ = true;
        pathBrowser_.setCurrentDirectory(folder);
        pathBrowser_.setSelectedFile(folder);
        changePathBrowser_ = false;
    }

    /**
     * Sets the path field according to the selected file in the path browser.
     * If no directory is selected the current path of the browser is taken.
     */
    private void updatePath()
    {
        setNewPath(getPathBrowserPath());
    }

    private String getPathBrowserPath()
    {
        File folder = pathBrowser_.getSelectedFile();

        if(folder == null)
        {
            folder = pathBrowser_.getCurrentDirectory();
        }

        if(folder != null)
        {
            return folder.getAbsolutePath() + File.separator;
        }

        return "";
    }

    /**
     * Validates the name that was entered by the user. If the name is valid,
     * the method will return true, otherwise false is returned. <br>
     * The name is valid if the a file with this name can be created on the
     * underlying file system.
     * 
     * @return true if the name is valid, false otherwise
     */
    private boolean validateName()
    {
        // TODO: name synax check via regular expressions
        return (nameField_.getText().length() > 0);
    }

    /**
     * Validates the path that was entered by the user. If the path is valid,
     * the method will return true, otherwise false is returned. <br>
     * The path is valid if the a path with this name can be created on the
     * underlying file system.
     * 
     * @return true if the path is valid, false otherwise
     */
    private boolean validatePath()
    {
        // TODO: name synax check via regular expressions
        return (pathField_.getText().length() > 0);
    }

    /**
     * Displays or hides the path browser in the panel. The parent dialog will
     * be resized so that the preferred size of the path browser can be
     * displayed in the dialog. The path browsers directory will be set to the
     * one entered in the path field.
     * 
     * @param show
     *            set to true to show the browser and false to hide the browser
     */
    private void showPathBrowser(boolean show)
    {
        Container parent = getTopLevelAncestor();

        pathBrowser_.setVisible(show);
        fillPanel_.setVisible(!show);
        pathField_.setEditable(!show);

        if(show)
        {
            parent.setSize(parent.getPreferredSize());

            updatePathBrowser();
        }
    }

    /**
     * This will set the new path to the path field. The path browser is not
     * updated. This is not necessary because the path field is write proteced
     * while the path browser is shown, and the path browser is updated when he
     * is displayed.
     * 
     * @param path
     *            the path that should set into the path field
     */
    private void setNewPath(String path)
    {
        // lock the path browser for updates to avoid an
        // IllegalStateException
        changePathField_ = true;

        if(!path.endsWith(File.separator))
        {
            path += File.separator;
        }

        if(extendPath_)
        {
            path += nameField_.getText();
        }

        pathField_.setText(path);
        changePathField_ = false;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Supporting methods for the listener pattern
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Adds a new path browser panel listener to the path browser panel. These
     * listeners will be notified about several events happening in the path
     * browser panel. The following events will be fired:
     * <ul>
     * <li>nameNotValid
     * <li>pathNotValid
     * <li>validated
     * <li>focusGainedOnName
     * <li>focusGainedOnPath
     * <li>focusLost
     * </ul>
     * See the IPathBrowserPanelListener interface for detailed descriptions of
     * the events.
     * 
     * @param listener
     *            the listener that should be added to the path browser panel
     * @see viptool.base.gui.component.event.IPathBrowserPanelListener
     */
    public void addPathBrowserPanelListener(IPathBrowserPanelListener listener)
    {
        if(listener != null)
        {
            listeners_.add(listener);
        }
    }

    /**
     * Removes the given listener from the path browser panel. This listener
     * will no longer be notified about events in the path browser panel.
     * 
     * @param listener
     *            the listener that should be removed from the path browser
     *            panel
     */
    public void removePathBrowserPanelListener(
            IPathBrowserPanelListener listener)
    {
        listeners_.remove(listener);
    }

    /**
     * Notifies the listeners that the user not entered a name yet or that the
     * name is not valid. A name is not valid, if a file with this name cannot
     * be created on the underlying file system.
     */
    private void fireNameNotValidEvent()
    {
        Iterator it;

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IPathBrowserPanelListener) it.next()).nameNotValid();
        }
    }

    /**
     * Notifies the listeners that the user not entered a path yet or that the
     * path is not valid. A path is not valid, if a path with this name cannot
     * be created on the underlying file system.
     */
    private void firePathNotValidEvent()
    {
        Iterator it;

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IPathBrowserPanelListener) it.next()).pathNotValid();
        }
    }

    /**
     * Notifies the listeners that name and path are now valid.
     */
    private void fireValidatedEvent()
    {
        Iterator it;

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IPathBrowserPanelListener) it.next()).validated();
        }
    }

    /**
     * Informs the listeners that the name field has the focus now.
     */
    private void fireNameFocusGainedEvent()
    {
        Iterator it;

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IPathBrowserPanelListener) it.next()).focusGainedOnName();
        }
    }

    /**
     * Informs the listeners that the path field has the focus now.
     */
    private void firePathFocusGainedEvent()
    {
        Iterator it;

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IPathBrowserPanelListener) it.next()).focusGainedOnPath();
        }
    }

    /**
     * Informs the listeners that neither the name field nor the path field has
     * the focus now.
     */
    private void fireFocusLostEvent()
    {
        Iterator it;

        it = listeners_.iterator();
        while (it.hasNext())
        {
            ((IPathBrowserPanelListener) it.next()).focusLost();
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the PropertyChangeListener interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see java.beans.PropertyChangeListener */
    public void propertyChange(PropertyChangeEvent evt)
    {
        if(!changePathBrowser_)
        {
            updatePath();
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the Focuslistener interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see java.awt.event.FocusListener */
    public void focusGained(FocusEvent event)
    {
        if(event.getSource() == nameField_)
        {
            fireNameFocusGainedEvent();
        }
        else if(event.getSource() == pathField_)
        {
            firePathFocusGainedEvent();
        }
    }

    /* @see java.awt.event.FocusListener */
    public void focusLost(FocusEvent event)
    {
        fireFocusLostEvent();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the ChangeListener interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see javax.swing.event.ChangeListener */
    public void itemStateChanged(ItemEvent event)
    {
        boolean selected = event.getStateChange() == ItemEvent.SELECTED;
        if(event.getItemSelectable() == defaultPathBox_)
        {
            if(selected)
            {
                // restore the path field to the default path
                setNewPath(ResourceConstants.PATH_DEFAULT_WORKSPACE);

                // hide the path browser
                showPathBrowser(false);
                pathField_.setEditable(false);
            }
            else
            {
                // show the path browser if necessary
                showPathBrowser(browseButton_.isSelected());
            }

            // lock / unlock browse button
            browseButton_.setEnabled(!selected);
        }

        if(event.getItemSelectable() == browseButton_)
        {
            showPathBrowser(selected);
        }

        validateValues();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the DocumentListener interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see javax.swing.event.DocumentListener */
    public void changedUpdate(DocumentEvent event)
    {
        validateValues();

        if(event.getDocument() == pathField_.getDocument())
        {
            updatePathBrowser();
        }
        if(event.getDocument() == nameField_.getDocument())
        {
            setNewPath(getPathBrowserPath());
        }
    }

    /* @see javax.swing.event.DocumentListener */
    public void insertUpdate(DocumentEvent event)
    {
        validateValues();

        if(event.getDocument() == pathField_.getDocument())
        {
            updatePathBrowser();
        }
        if(event.getDocument() == nameField_.getDocument())
        {
            setNewPath(getPathBrowserPath());
        }
    }

    /* @see javax.swing.event.DocumentListener */
    public void removeUpdate(DocumentEvent event)
    {
        validateValues();

        if(event.getDocument() == pathField_.getDocument())
        {
            updatePathBrowser();
        }
        if(event.getDocument() == nameField_.getDocument())
        {
            setNewPath(getPathBrowserPath());
        }
    }

}