package viptool.base.gui.component.event;

/**
 * Interface for listeners of the path browser panel. The path browser panel is
 * a panel in which the user can enter a file name and a file system related
 * path. This listener interface contains method so the caller of the panel is
 * informed about the important steps in the panel.
 */
public interface IPathBrowserPanelListener
{
    /**
     * Notifies the listener that no name is given or that the given name is
     * invalid. This event can be used to inform the user, that he has not
     * correctly entered the needed information.<br>
     * A name is not valid, if a file with this name cannot be created on the
     * underlying file system.
     */
    public void nameNotValid();

    /**
     * Notifies the listener that no path is given or that the given path is
     * invalid. This event can be used to inform the user, that he has not
     * correctly entered the needed information.<br>
     * A path is not valid, if a path with this name cannot be created on the
     * underlying file system.<br>
     * This method is not called, until the name is valid!
     */
    public void pathNotValid();

    /**
     * Notifies the listener that the user has now entered a name and a path and
     * that both information is valid. The caller of the panel can allow the
     * next operation now.
     */
    public void validated();

    /**
     * Notifies the listener that the name field has the focus now. This method
     * can be used to display additional information about the name field. This
     * can help the user to enter his information correctly.
     */
    public void focusGainedOnName();

    /**
     * Notifies the listener that the path field has the focus now. This method
     * can be used to display additional information about the path field. This
     * can help the user to enter his information correctly.
     */
    public void focusGainedOnPath();

    /**
     * Notifies the listener that neither the name nor the path field has the
     * focus now. If any help message has been displayed to help the user, these
     * messages should be hided now.
     */
    public void focusLost();
}
