package viptool.base.gui.menu;

import java.net.URI;

import javax.swing.JMenuBar;

import viptool.base.error.XMLException;
import viptool.base.gui.IWindow;

/**
 * The menu bar manager manages the menu bars in viptool. It works by handling
 * so called menu bar templates that are not displayed. Instead if a menu is
 * requested by a window the template is duplicated and the clone is registered
 * with an instance number. The actions that are in the menu are also cloned
 * with the instance number as reference.
 */
public interface IMenuBarManager
{
    // <><><><><><><><><><><><><><><><><><><><><>
    // template manipulation
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Adds the given template to the menu template manager. If a template
     * already exists to the given id, the old template will be overwritten. The
     * template must not be null.
     * 
     * @param template
     *            the template that should be added to the menu template manager
     */
    public abstract void addTemplate(IMenuTemplate template);

    
    public void addEmptyTemplate(String id, String inheritFrom);

    // <><><><><><><><><><><><><><><><><><><><><>
    // menu manipulation
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Creates a concrete instance of the template that was specified by the id
     * parameter. The returned menu bar is a clone of the template with fully
     * cloned actions, that are registered in the action handler with the same
     * instance id. This enables access to the same actions as they are in the
     * menu (good for using the actions somewhere else, e.g. in toolbars).
     * 
     * @param id
     *            the id of the template from which the menu should be created
     * @param instance
     *            an unique instance id for further access to the same menu bar
     * @return a menu bar based on the template given by the id
     */
    public JMenuBar createMenuBar(String id,  IWindow window);
    
    public IMenuTemplate getTemplate(String id);

	

}
