package viptool.base.gui.menu.event;

import java.util.EventObject;

import viptool.base.gui.menu.IExtendedMenu;
import viptool.base.gui.menu.IExtendedMenuBar;

/**
 * This event object is sent with events to the template menu listeners. It
 * contains additional informations about the changed element, such as the id of
 * the element and the containing template.
 */
public class ExtendedMenuEvent extends EventObject
{
    // the id of the changed element
    private String id_;

    // the enclosing group
    private String container_;

    // the positional string
    private String insertAt_;

    // the exact position relative to insertAt
    private int    seek_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs a new template menu event.
     * 
     * @param menu
     *            the menu in which the change has occured
     * @param id
     *            the id of the changed element
     */
    public ExtendedMenuEvent(IExtendedMenu menu, String id)
    {
        super(menu);

        id_ = id;
        container_ = null;
        insertAt_ = null;
        seek_ = 0;
    }
    
    /**
     * Constructs a new template menu event.
     * 
     * @param menu
     *            the menu in which the change has occured
     * @param id
     *            the id of the changed element
     * @param container
     *            the container of the element
     * @param insertAt
     *            a positional string of the element
     * @param seek
     *            a value that refers to insertAt and describing the exact
     *            position
     */
    public ExtendedMenuEvent(IExtendedMenu menu, String id, String container,
            String insertAt, int seek)
    {
        super(menu);

        id_ = id;
        container_ = container;
        insertAt_ = insertAt;
        seek_ = seek;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // state information
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns the id of the changed element.
     * 
     * @return the id of the element
     */
    public String getId()
    {
        return id_;
    }

    /**
     * Returns the container id of the element.
     * 
     * @return the container id of the element
     */
    public String getContainer()
    {
        return container_;
    }

    /**
     * Returns a positional string parameter. The elements position can be fully
     * described within the group through the insertAt and the seek parameter.
     * 
     * @return the id of the element to which this elements position refers
     */
    public String getInsertAt()
    {
        return insertAt_;
    }

    /**
     * Returns the exact position of the element relative to the insertAt value.
     * 
     * @return a positional parameter value
     */
    public int getSeek()
    {
        return seek_;
    }

    /**
     * Returns the menu template that has fired the event.
     * 
     * @return the firing menu template
     */
    public IExtendedMenuBar getMenuTemplate()
    {
        return (IExtendedMenuBar) getSource();
    }
}
