package viptool.base.gui.toolbar;

import java.net.URI;

import javax.swing.JToolBar;

import viptool.base.error.XMLException;
import viptool.base.gui.IWindow;

/**
 * The toolbar manager manages the toolbar templates in viptool.
 * Toolbar templates are toolbars that are not displayed. Instead if a toolbar
 * is requested by a window the template is duplicated and the clone is
 * registered with an instance number. The actions that are in the toolbar are
 * also cloned with the instance number as reference.
 */
public interface IToolBarManager
  {
    // <><><><><><><><><><><><><><><><><><><><><>
    // template manipulation
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Adds the given template to the toolbar template manager. If a template
     * already exists to the given id, the old template will be overwritten. The
     * template must not be null.
     * 
     * @param template
     *          the template that should be added to the toolbar template
     *          manager
     */
    public void addTemplate(IToolBarTemplate template);
    
    public void addEmptyTemplate(String id, String inheritFrom);

    /**
     * Returns the template with the given id. If no such template exists, null
     * is returned.
     * 
     * @param id
     *          the id of the template to return
     * @return the template instance belonging to the given id
     */
    public IToolBarTemplate getTemplate(String id);

    // <><><><><><><><><><><><><><><><><><><><><>
    // toolbar manipulation
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Creates a concrete instance of the template that was specified by the id
     * parameter. The returned toolbar is a clone of the template with fully
     * cloned actions, that are registered in the action handler with the same
     * instance id. This enables access to the same actions as they are in the
     * toolbar (good for using the actions somewhere else, e.g. in menus).
     * 
     * @param id
     *          the id of the template from which the toolbar should be created
     * @param instance
     *          an unique instance id for further access to the same toolbar
     * @return a toolbar based on the template given by the id
     */
    public JToolBar createToolBar(String id, IWindow window);

  }
