package viptool.base.wizard;

import java.awt.Image;

import javax.swing.JComponent;

/**
 * This interface is a base interface for all wizard pages used in the standard
 * dialog. It supports image, description and can-go-back info. Wizard pages are
 * returned by the call of methods in the Wizard interface.
 */
public interface IWizardPage
{
    /**
     * Is called to determine whether the user can go a step back. Should return
     * true if the previous page is accessible, false otherwise.
     * 
     * @return flag that indicates whether the user can move back
     */
    public boolean canGoToPrevoius();

    /**
     * Should return the image that is displayed in the right top corner of the
     * dialog. If null is returned, no image is displayed.
     * 
     * @return the image that is displayed in the dialog or null if no image
     *         should be displayed
     */
    public Image getImage();

    /**
     * Is called to get the main description for the dialog page. This
     * description should describe the page content in a VERY SHORT way. Don't
     * give more than two or three words.
     * 
     * @return the main description for the dialog content
     */
    public String getMainDescription();

    /**
     * Must return the component that should be displayed in the client area of
     * the dialog. The wizard page is responsible for validating the entered
     * data. MUST NOT return null.
     * 
     * @return a reference to a JComponent that is displayed in the dialog
     */
    public JComponent getContent();
}
