package viptool.base.wizard;

import java.util.ArrayList;

import viptool.base.gui.dialog.DialogConstants;
import viptool.base.gui.dialog.IStandardModalDialog;
import viptool.base.wizard.event.ISimpleWizardPageListener;

public abstract class SimpleWizard extends WizardAdapter implements
        ISimpleWizardPageListener
{
    private int                               currentPage_;
    private boolean                           initialized_;
    private IStandardModalDialog              dialog_;
    private final ArrayList<SimpleWizardPage> pages_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructor
    // <><><><><><><><><><><><><><><><><><><><><>

    public SimpleWizard()
    {
        dialog_ = null;
        currentPage_ = -1;
        initialized_ = false;
        pages_ = new ArrayList<SimpleWizardPage>();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Page handling
    // <><><><><><><><><><><><><><><><><><><><><>

    public void addPage(SimpleWizardPage page)
    {
        if (!initialized_ && page != null)
        {
            pages_.add(page);
            page.addSimpleWizardPageListener(this);
        }
    }

    public void removePage(SimpleWizardPage page)
    {
        if (!initialized_)
        {
            pages_.remove(page);
        }
    }

    public SimpleWizardPage getCurrentPage()
    {
        return pages_.get(currentPage_);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // supporting methods
    // <><><><><><><><><><><><><><><><><><><><><>
    public IStandardModalDialog getWizardDialog()
    {
        return dialog_;
    }

    private void updateButtons()
    {
        dialog_.setButtonsEnabled(DialogConstants.BUTTON_FINISH,
                getCurrentPage().canFinish() && getCurrentPage().isPageValid());
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implemented methods from IWizard
    // <><><><><><><><><><><><><><><><><><><><><>

    public IWizardPage getFirstPage(IStandardModalDialog dialog)
    {
        dialog_ = dialog;

        if (pages_.size() == 0)
        {
            return null;
        }

        initialized_ = true;
        currentPage_ = 0;

        updateButtons();
        return pages_.get(currentPage_);
    }

    public IWizardPage getNextPage()
    {
        if (currentPage_ < pages_.size() - 1)
        {
            currentPage_++;
            updateButtons();
            return pages_.get(currentPage_);
        }

        return null;
    }

    public IWizardPage getPreviousPage()
    {
        if (currentPage_ > 0)
        {
            currentPage_--;
            updateButtons();
            return pages_.get(currentPage_);
        }

        return null;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implemented methods from ISimpleWizardPageListener
    // <><><><><><><><><><><><><><><><><><><><><>

    public void pageIsInvalid()
    {
        updateButtons();
    }

    public void pageIsValid()
    {
        updateButtons();
    }

}
