package viptool.base.wizard;

import java.util.HashSet;

import viptool.base.wizard.event.ISimpleWizardPageListener;

public abstract class SimpleWizardPage implements IWizardPage
{
    private final HashSet<ISimpleWizardPageListener> listeners_;

    public SimpleWizardPage()
    {
        listeners_ = new HashSet<ISimpleWizardPageListener>();
    }

    /**
     * Should return true, if the wizard can be finished after that page
     */
    public abstract boolean canFinish();

    /**
     * Should return true, if the page is valid. False otherwise.
     */
    public abstract boolean isPageValid();

    // <><><><><><><><><><><><><><><><><><><><><>
    // listener support
    // <><><><><><><><><><><><><><><><><><><><><>
    public void addSimpleWizardPageListener(ISimpleWizardPageListener listener)
    {
        if (listener != null)
        {
            listeners_.add(listener);
        }
    }

    public void removeSimpleWizardPageListener(
            ISimpleWizardPageListener listener)
    {
        listeners_.remove(listener);
    }

    protected void firePageValidEvent()
    {
        for (ISimpleWizardPageListener listener : listeners_)
        {
            listener.pageIsValid();
        }
    }

    protected void firePageInvalidEvent()
    {
        for (ISimpleWizardPageListener listener : listeners_)
        {
            listener.pageIsInvalid();
        }
    }
}
