package viptool.extension;

import java.awt.Image;
import java.awt.MediaTracker;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;

import viptool.base.error.ResourceLocateException;
import viptool.base.error.VipToolException;
import viptool.util.resources.AbstractResourceLoader;

/**
 * The ExtensionResourceLoader can be used to create URIs to the resources 
 * form the extension with which is was initiated.
 */
public class ExtensionResourceLoader extends AbstractResourceLoader
{
    private int imageIds_;
    
    private String    resourceBase_;

    private Extension extension_;
    
    private MediaTracker mediaTracker_;

    /**
     * This constructor creates a new ExtensionResourceLoader for the given 
     * extension. The absolute path for result files is build up from the path
     * of the extension, the given base and the relative path of the resources.
     * 
     * @param extension
     * 		extension as Extension
     * @param base
     * 		relative path from the extension to the resource directory
     */
    public ExtensionResourceLoader(Extension extension, String base)
    {
        int index;
        imageIds_ = 0;

        extension_ = extension;
        resourceBase_ = extension.getClass().getName().replace('.', '/');
        index = resourceBase_.lastIndexOf('/');
        resourceBase_ = resourceBase_.substring(0, index + 1) + base + "/";
        
        mediaTracker_ = new MediaTracker(extension.getWorkbench().getMainFrame());
    }

    /**
     * This method returns a URI to the resource with the given relative path.
     */
    public URI getResource(String resource)
    {
        URI resourceURI;

        resourceURI = getResource(resourceBase_, resource);
        if(resourceURI == null)
        {
            resourceURI = extension_.getWorkbench().getResourceLoader()
                    .getResource(resource);
        }

        return resourceURI;
    }

    /**
     * This method returns an URI to the resource with the given relative
     * path and the given base.
     */
    public URI getResource(String base, String resource)
    {
        URL classURL;
        
        
        if(!base.endsWith("/"))
        {
            base += "/";
        }
        
        classURL = extension_.getClass().getClassLoader().getResource(
                base + resource);

        try
        {
            if(classURL != null)
            {
                return classURL.toURI();
            }

            return null;
        }
        catch (URISyntaxException exception)
        {
            extension_.getWorkbench().showExceptionInDialog(
                    new VipToolException(exception));

            throw new ResourceLocateException(exception);
        }
    }
    
    public Image getImage(String name)
    {
        Image image = super.getImage(name);
        
        mediaTracker_.addImage(image, imageIds_);
        try
        {
            mediaTracker_.waitForID(imageIds_);
        }
        catch (InterruptedException exception) 
        {
            exception.printStackTrace();
        }
            
        imageIds_++;
        return image;
    }
}
