package viptool.extension;


/**
 * This interface has to implemented by all classes which want to be called if a
 * change on the state of running or known extensions was made.
 */
public interface IExtensionListener {
	/**
	 * This method will be called if a new extension was added to the list of known 
	 * extensions.
	 * 
	 * @param extInfo
	 * 		extension which was added
	 */
	public void extensionAdded(IExtensionInfo extInfo);
	
	/**
	 * This method will be called if an extension was removed from the list of known
	 * extensions.
	 * 
	 * @param extInfo
	 * 		extension which was removed
	 */
	public void extensionRemoved(IExtensionInfo extInfo);
	
	/**
	 * This method will be called if an extension was loaded.
	 * 
	 * @param extInfo
	 * 		extension which was loaded
	 */
	public void extensionStarted(IExtensionInfo extInfo);
	
	/**
	 * This method will be called if an extension was stoppen
	 * 
	 * @param extInfo
	 * 		extension which was stopped
	 */
	public void extensionStopped(IExtensionInfo extInfo);
}
