package viptool.extension;

import java.net.URL;
import java.util.jar.JarFile;

import viptool.util.ClassFile;

/**
 * The interface IExtensionLoader describes all methods of the ExtensionLoader.
 */
public interface IExtensionLoader
{

    /**
     * This method adds the given JarFile to the URL-List of the
     * ExtensionLoader. This is necessary to create instances of the contained
     * classes later. This method should be used to add JarFile which includes
     * extensions.
     * 
     * @param jarfile
     *            JarFile to add
     */
    public void addJarFile(JarFile jarfile);

    /**
     * This method adds the given ClassFild to the URL-List of the
     * ExtensionLoader. This is necessary to create instances of the class
     * later. This method should be used to add class files from extension which
     * are not packed as JarFiles.
     * 
     * @param classfile
     *            ClassFile to add
     */
    public void addClassFile(ClassFile classfile);

    /**
     * This method adds the given URL to the URL-List of this ExtensionLoader.
     * This means all classes which are available over this URL can be loaded.
     * 
     * @param url
     *            url to add
     */
    public void addClassURL(URL url);

    /**
     * This method adds the given URL to the URL-List for libraries of this
     * ExtensionLoader. This means that libraries are searched also on this
     * paths.
     * 
     * @param url
     *            url to add
     */
    public void addLibraryURL(URL url);

    /**
     * This method returns an Class-Object to the given class name.
     * 
     * @param name
     *            full qualified class name
     * @return Class-Object of the class
     */
    public Class getClass(String name);
}
