package viptool.extension;

import viptool.base.error.VipToolException;

public interface IExtensionManager {

    /**
	 * This method returns an ExtensionInfo for the given full qualified class name if
	 * the extension is known (was registered).
	 * 
	 * @param fqcn
	 * 		full qualified name of the extension
	 * @return
	 * 		extension with the given name or null
	 */
	public IExtensionInfo findExtenison(String fqcn);
	
	/**
	 * add ExtensionListener
	 * 
	 * @param listener
	 * 		listener to add
	 */
	public void addExtensionListener(IExtensionListener listener);
	
	/**
	 * remove ExtensionListener
	 * 
	 * @param listener
	 * 		listener to remove
	 */
	public void removeExtensionListener(IExtensionListener listener);
	
	/**
	 * This method loads the extension by creating an instance of the extensions main class.
	 *
	 * @return
	 * 		instance of the extension
	 */	
	public Extension loadExtension(IExtensionInfo extInfo) throws VipToolException;
	
	/**
	 * This method stops the extension.
	 */
	public void stopExtension(IExtensionInfo extInfo);
	
	/**
	 * This method return true if the given extension is activated.
	 * 
	 * @return
	 * 		true if activated
	 */
	public boolean isActivated(IExtensionInfo extInfo);
	
	/**
     * This method returns the index of the given extension into the list of known extensions.
     */
    public int getIndexOfExtension(IExtensionInfo extInfo);
    
    /**
     * This method remove the given extension from the list of known
     * extensions.
     * 
     * @param extInfo
     * 		extension to remove
     */
    public void removeExtenison(IExtensionInfo extInfo);
}
