package viptool.impl.base.extendedPane.event;

import java.util.EventObject;

import viptool.base.gui.IChildWindow;

/**
 * Class that is given as argument to the activeWindowChanged method of the
 * ActiveWindowChangedListener interface. Contains a reference to the new
 * active window.
 */
public class ActiveWindowChangeEvent extends EventObject
{
    // holds the new active window
    private IChildWindow     activeWindow_;
    
    /**
     * Constructs a new ActiveWindowChangeEvent. Used by ExtendedPane to
     * notify the listeners if a new window is active. 
     * 
     * @param source
     *        the source object the fires the event
     * @param activeWindow
     *        the window that is now active (has the focus)
     */
    public ActiveWindowChangeEvent(Object source, IChildWindow activeWindow)
    {
        super(source);
        activeWindow_ = activeWindow;
    }
    
    /**
     * Gets the new active Window of the ExtendedPane.
     * 
     * @return a reference to the new active (focused) window
     */
    public IChildWindow getActiveWindow()
    {
        return activeWindow_;
    }
}
