package viptool.impl.base.extendedPane.event;

import java.util.EventObject;

import viptool.base.gui.IChildWindow;

/**
 * Class that is given as argument to the activeWindowChanged method of the
 * ActiveWindowChangedListener interface. Contains a reference to the new
 * active window.
 */
public class WindowCloseEvent extends EventObject
{
    // holds the new active window
    private IChildWindow     windowToClose_;
    
    /**
     * Constructs a new ActiveWindowChangeEvent. Used by ExtendedPane to
     * notify the listeners if a new window is active. 
     * 
     * @param source
     *        the source object the fires the event
     * @param windowToClose
     *        the window that is now active (has the focus)
     */
    public WindowCloseEvent(Object source, IChildWindow windowToClose)
    {
        super(source);
        windowToClose_ = windowToClose;
    }
    
    /**
     * Gets the new active Window of the ExtendedPane.
     * 
     * @return a reference to the new active (focused) window
     */
    public IChildWindow getWindowToClose()
    {
        return windowToClose_;
    }
}
