package viptool.impl.base.gui.dialog;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.Rectangle2D;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.LineBorder;

import viptool.base.gui.dialog.DialogConstants;
import viptool.base.gui.dialog.IStandardModalDialog;
import viptool.base.gui.dialog.event.ButtonClickedEvent;
import viptool.base.gui.dialog.event.IStandardDialogListener;
import viptool.impl.base.MainFrame;
import viptool.util.GeneralUtils;
import viptool.util.resources.ResourceConstants;

/**
 * Implementation of the IStandardModalDialog interface. This class is final and
 * cannot be subclassed. It should be used in a class not derived.
 * 
 * @see viptool.base.gui.dialog.IStandardModalDialog
 */
//TODO:TI: ceck size, max screen size
public final class StandardModalDialog extends JDialog implements
        IStandardModalDialog
{
    // the panel that is centered in the dialog an holds all content
    private JPanel      contentPanel_ = null;

    // the panel that is in top of the content panel and contains title, images
    private TopPanel    topPanel_;

    // the main panel that is centered in the content panel and contains the
    // client component that should be shown in the dialog
    private JComponent  mainPanel_    = null;

    // the button panel is at bottom of the content panel and holds the buttons
    private ButtonPanel buttonPanel_;

    // list for the listeners of the standard dialog
    private Set         listeners_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Inner classes
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * This class handles the resizing restrictives of the dialog.
     */
    private class ComponentHandler extends ComponentAdapter
    {
        // remember the original bounds
        private Rectangle bounds_ = getBounds();

        // <><><><><><><><><><><><><><><><><><><><><>
        // Overriden methods from ComponentAdapter
        // <><><><><><><><><><><><><><><><><><><><><>
        /* @see java.awt.events.ComponentListener */
        public void componentResized(ComponentEvent evt)
        {
            int oldX = bounds_.x;
            int oldY = bounds_.y;
            int newX = getX();
            int newY = getY();
            int newWidth = getWidth();
            int newHeight = getHeight();
            Dimension dimension = getMinimumSize();

            // if the user has made the dialog to small, resize it to the
            // minimum size
            if(newWidth < dimension.width || newHeight < dimension.height)
            {
                int xDiff = dimension.width - newWidth;
                int yDiff = dimension.height - newHeight;

                // calculate the new width (if xDiff or yDiff is lesser than
                // zero, the user has made the dialog bigger)
                newWidth += Math.max(0, xDiff);
                newHeight += Math.max(0, yDiff);

                // perhaps the user has changed the left or top border
                if(xDiff > 0 && newX != oldX)
                {
                    newX -= xDiff;
                }
                if(yDiff > 0 && newY != oldY)
                {
                    newY -= yDiff;
                }

                setBounds(newX, newY, newWidth, newHeight);
            }

            // store the new width and height in our remembering rectangle for
            // later use
            bounds_.setBounds(newX, newY, newWidth, newHeight);
        }
    }

    /**
     * This class the for the top panel in the dialog.
     */
    private class TopPanel extends JPanel
    {
        // the image that is displayed at the top right corner of the dialog
        private Image     image_;

        // the main description (describes what the dialog does)
        private String    mainDescription_;

        // the detailed description (describes the current action, can depend
        // on the selected element in the dialog)
        private String    detailedDescription_;

        // the size of the panel
        private Dimension dimension_ = new Dimension(100, 78);

        // the font for the main description
        private Font      titleFont_;

        // the font for the detailed description
        private Font      descriptionFont_;

        // the label that contain notices
        private Notice    notice_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // Inner classes
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * This class is for notices in the dialog
         */
        private class Notice
        {
            // height of a notice
            private static final int NOTICE_HEIGHT = 18;

            // the font for the notice text
            private Font             noticeFont_;

            // the notice next
            private String           notice_;

            // the type of the notice (warning, error) given in DialogConstants
            private int              noticeType_;

            // <><><><><><><><><><><><><><><><><><><><><>
            // Inner classes
            // <><><><><><><><><><><><><><><><><><><><><>
            // for later implementation
            // private class Slider extends Thread
            // {
            // private int ystart_;
            //
            // private int yend_;
            //
            // private Graphics graphics_;
            //
            // public Slider(int ystart, int yend, Graphics graphics)
            // {
            // setPriority(MAX_PRIORITY);
            //
            // ystart_ = ystart;
            // yend_ = yend;
            // graphics_ = graphics;
            // }
            //
            // public void run()
            // {
            // for(int y = ystart_; y >= yend_; y--)
            // {
            // paintNotice(graphics_, y);
            // }
            // }
            // }

            // <><><><><><><><><><><><><><><><><><><><><>
            // Constructors
            // <><><><><><><><><><><><><><><><><><><><><>
            /**
             * Creates a new notice panel. By default, the notice panel is
             * hidden, contains no notice and has type warning.
             */
            public Notice()
            {
                notice_ = null;
                noticeType_ = DialogConstants.TYPE_WARNING;
                noticeFont_ = new Font("SansSerif", Font.ITALIC, 12);
            }

            // <><><><><><><><><><><><><><><><><><><><><>
            // Supporting methods
            // <><><><><><><><><><><><><><><><><><><><><>
            /**
             * Sets a new notice. To remove a notice set it to null. The notice
             * panel is automatically shown if the notice is not null and
             * automatically hidden, if a null notice is applied.<br>
             * The type must be one of the values in DialogConstants. At the
             * moment warning and error is supported and expressed by colours
             * (yellow and red).
             * 
             * @param notice
             *            the new notice to display or null to remove a notice
             * @param type
             *            the new type for the notice (value from
             *            DialogConstants)
             */
            public void setNotice(String notice, int type)
            {
                notice_ = notice;
                noticeType_ = type;
            }

            public boolean hasNotice()
            {
                return (notice_ != null && notice_.length() > 0);
            }

            // TODO: for later implementation
            // public void slideNotice()
            // {
            // int height;
            // Slider slider;
            //
            // height = getBounds().height;
            // slider = new Slider(height, height - Notice.NOTICE_HEIGHT - 1,
            // getGraphics());
            //                
            // slider.start();
            // try
            // {
            // slider.join();
            // }
            // catch(InterruptedException exception)
            // {
            //                    
            // }
            // }

            public void paintNotice(Graphics graphics, int y)
            {
                Graphics2D graphics2D;
                TextLayout textLayout;
                Rectangle2D textBounds;
                RenderingHints oldRenderingHints;
                Rectangle noticeBounds;
                Color backgroundColor;
                Color borderColor;

                noticeBounds = TopPanel.this.getBounds();
                noticeBounds.y = y;
                noticeBounds.height = NOTICE_HEIGHT;
                noticeBounds.width -= 1;

                // determine colors
                if(notice_ == null)
                {
                    // make notice invisible
                    backgroundColor = TopPanel.this.getBackground();
                    borderColor = TopPanel.this.getBackground();
                }
                else
                {
                    // set color depending on the notice type
                    switch (noticeType_)
                    {
                    default:
                    case DialogConstants.TYPE_WARNING:
                        backgroundColor = ResourceConstants.COLOR_WARNING_LIGHT;
                        borderColor = ResourceConstants.COLOR_WARNING_DARK;
                        break;

                    case DialogConstants.TYPE_ERROR:
                        backgroundColor = ResourceConstants.COLOR_ERROR_LIGHT;
                        borderColor = ResourceConstants.COLOR_ERROR_DARK;
                        break;
                    }
                }

                graphics2D = (Graphics2D) graphics;

                oldRenderingHints = graphics2D.getRenderingHints();
                graphics2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                        RenderingHints.VALUE_ANTIALIAS_ON);

                // background
                graphics2D.setColor(backgroundColor);
                graphics2D.fillRect(noticeBounds.x, noticeBounds.y,
                        noticeBounds.width, noticeBounds.height);
                // border
                graphics2D.setColor(borderColor);
                graphics2D.drawRect(noticeBounds.x, noticeBounds.y,
                        noticeBounds.width, noticeBounds.height);

                if(notice_ != null)
                {
                    // draw text
                    graphics2D.setColor(Color.BLACK);

                    // paint the notice
                    textLayout = new TextLayout(notice_, noticeFont_,
                            graphics2D.getFontRenderContext());
                    textBounds = textLayout.getBounds();
                    textLayout.draw(graphics2D, noticeBounds.x + 5
                            - (float) textBounds.getX(), noticeBounds.y + 5
                            - (float) textBounds.getY());
                }

                graphics2D.setRenderingHints(oldRenderingHints);
            }
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Creates a new top panel with a main description and an image. There
         * is no detailed description by default. The detailed description must
         * be manually set by the setDetailedDescription method.<br>
         * The image can be null if no image should be displayed.
         * 
         * @param mainDescription
         *            the main description for the dialog
         * @param image
         *            the image to display or null for no image
         */
        public TopPanel(String mainDescription, Image image)
        {
            image_ = image;
            mainDescription_ = mainDescription;
            detailedDescription_ = null;

            setBackground(Color.WHITE);
            setDoubleBuffered(true);

            notice_ = new Notice();
            titleFont_ = new Font("SansSerif", Font.BOLD | Font.ITALIC, 22);
            descriptionFont_ = new Font("SansSerif", Font.PLAIN, 12);
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Supporting methods.
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Sets the new image for the dialog. To remove an existing image give
         * null as parameter.
         * 
         * @image the new image for the dialog
         */
        public void setImage(Image image)
        {
            image_ = image;

            repaint();
        }

        /**
         * Sets the main description for the dialog. The main description is
         * displayed on the top left of the dialog in BIG letters so don't use
         * more than two or three words. The main description must not be empty
         * or null.<br>
         * The dialog is updated automatically so no extra call is necessary.
         * 
         * @param description
         *            the new main description for the dialog.
         */
        public void setMainDescription(String description)
        {
            mainDescription_ = description;

            repaint();
        }

        /**
         * Sets the detailed description for the dialog. The detailed
         * description is displayed under the main description.<br>
         * To remove the detailed description set it to null.<br>
         * The dialog is updated automatically so no extra call is necessary.
         * 
         * @param description
         *            the new detailed description for the dialog.
         */
        public void setDetailedDescription(String description)
        {
            detailedDescription_ = description;

            repaint();
        }

        /**
         * Sets a new notice. To remove a notice set it to null. The notice
         * panel is automatically shown if the notice is not null and
         * automatically hidden, if a null notice is applied.<br>
         * The type must be one of the values in DialogConstants. At the moment
         * warning and error is supported and expressed by colours (yellow and
         * red).
         * 
         * @param notice
         *            the new notice to display or null to remove a notice
         * @param type
         *            the new type for the notice (value from DialogConstants)
         */
        public void setNotice(String notice, int type)
        {
            notice_.setNotice(notice, type);
            repaint();
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Overriden methods from JComponent
        // <><><><><><><><><><><><><><><><><><><><><>
        /* @see javax.swing.JComponent */
        protected void paintComponent(Graphics graphics)
        {
            int height;
            Graphics2D graphics2D;
            TextLayout textLayout;
            Rectangle bounds = getBounds();
            Rectangle2D textBounds;
            RenderingHints oldRenderingHints;

            super.paintComponent(graphics);

            graphics2D = (Graphics2D) graphics;
            oldRenderingHints = graphics2D.getRenderingHints();
            graphics2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);

            if(image_ != null)
            {
                graphics.drawImage(image_, ((int) bounds.getMaxX()) - 70, 2,
                        56, 56, null);
            }

            // paint the main description
            textLayout = new TextLayout(mainDescription_, titleFont_,
                    new FontRenderContext(null, true, true));
            textBounds = textLayout.getBounds();
            textLayout.draw(graphics2D, 2 - (float) textBounds.getX(),
                    10 - (float) textBounds.getY());

            // don't use anti-aliasing for description because its better to
            // read
            // uncomment because in linux its nicer. retest under windows
            // graphics2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            // RenderingHints.VALUE_ANTIALIAS_OFF);

            // paint the description
            if(detailedDescription_ != null
                    && detailedDescription_.length() > 0)
            {
                textLayout = new TextLayout(detailedDescription_,
                        descriptionFont_, graphics2D.getFontRenderContext());
                textBounds = textLayout.getBounds();
                textLayout.draw(graphics2D, 2 - (float) textBounds.getX(),
                        40 - (float) textBounds.getY());
            }

            graphics2D.setRenderingHints(oldRenderingHints);

            // paint the notice
            height = getBounds().height;
            notice_.paintNotice(graphics, height - Notice.NOTICE_HEIGHT - 1);
        }

        /* @see javax.swing.JComponent */
        public Dimension getMinimumSize()
        {
            return getPreferredSize();
        }

        /* @see javax.swing.JComponent */
        public Dimension getMaximumSize()
        {
            return getPreferredSize();
        }

        /* @see javax.swing.JComponent */
        public Dimension getPreferredSize()
        {
            Dimension size;

            if(getParent() != null)
            {
                size = getParent().getSize();
                dimension_ = new Dimension(size.width, dimension_.height);
            }

            return dimension_;
        }
    }

    /**
     * This class is for the default buttons in the dialog
     */
    private class ButtonPanel extends JPanel implements ActionListener
    {
        private static final int    BUTTON_GAP             = 5;

        private static final int    BUTTON_VERTICAL_OFFSET = 13;

        private static final String TEXT_OK                = "OK";

        private static final String TEXT_CANCEL            = "Cancel";

        private static final String TEXT_RETRY             = "Retry";

        private static final String TEXT_IGNORE            = "Ignore";

        private static final String TEXT_FINISH            = "Finish";

        private static final String TEXT_NEXT              = "Next >";

        private static final String TEXT_PREVIOUS          = "< Back";

        private static final String TEXT_YES               = "Yes";

        private static final String TEXT_NO                = "No";

        // the retry (previous) button
        private DialogButton        retryButton_           = null;

        // the ignore (next) button
        private DialogButton        ignoreButton_          = null;

        // the ok (finish, yes) button
        private DialogButton        okButton_              = null;

        // the cancel (no) button
        private DialogButton        cancelButton_          = null;

        // the dimension
        private Dimension           dimension_             = new Dimension(100,
                                                                   50);

        // the minimal needed width
        private int                 minimumWidth_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // inner classes
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * This class is for the buttons in the button panel
         */
        private class DialogButton extends JButton
        {
            /** the minimal with for a dialog button */
            public static final int BUTTON_WIDTH  = 70;

            /** the minimal height for a dialog button */
            public static final int BUTTON_HEIGHT = 24;

            // the minimal dimension for a dialog button
            private Dimension       dimension_    = new Dimension(BUTTON_WIDTH,
                                                          BUTTON_HEIGHT);

            // type of the button (DialogConstants)
            private int             type_;

            // <><><><><><><><><><><><><><><><><><><><><>
            // Constructors
            // <><><><><><><><><><><><><><><><><><><><><>
            /**
             * Constructs a new dialog button and sets its title to the given
             * title. The title is what is displayed on top of the button.
             * 
             * @param title
             *            the text for the button
             */
            public DialogButton(int type)
            {
                super();

                type_ = type;
                switch (type_)
                {
                case DialogConstants.BUTTON_FINISH:
                    setText(TEXT_FINISH);
                    break;

                case DialogConstants.BUTTON_IGNORE:
                    setText(TEXT_IGNORE);
                    break;

                case DialogConstants.BUTTON_NEXT:
                    setText(TEXT_NEXT);
                    break;

                case DialogConstants.BUTTON_NO:
                    setText(TEXT_NO);
                    break;

                case DialogConstants.BUTTON_OK:
                    setText(TEXT_OK);
                    break;

                case DialogConstants.BUTTON_PREVIOUS:
                    setText(TEXT_PREVIOUS);
                    break;

                case DialogConstants.BUTTON_RETRY:
                    setText(TEXT_RETRY);
                    break;

                case DialogConstants.BUTTON_YES:
                    setText(TEXT_YES);
                    break;

                default:
                case DialogConstants.BUTTON_CANCEL:
                    setText(TEXT_CANCEL);
                    break;
                }
            }

            // <><><><><><><><><><><><><><><><><><><><><>
            // Supporting methods
            // <><><><><><><><><><><><><><><><><><><><><>
            /**
             * Returns the type (a value from DialogConstants) of the button.
             * 
             * @return the type of the button
             */
            public int getType()
            {
                return type_;
            }

            //
            // /* @see javax.swing.JComponent */
            // public Dimension getMaximumSize()
            // {
            // return getPreferredSize();
            // }

            // /* @see javax.swing.JComponent */
            // public Dimension getPreferredSize()
            // {
            // return dimension_;
            // }
            // <><><><><><><><><><><><><><><><><><><><><>
            // Overriden methods from JComponent
            // <><><><><><><><><><><><><><><><><><><><><>
            /* @see javax.swing.JComponent */
            public Dimension getMinimumSize()
            {
                // return getPreferredSize();
                return dimension_;
            }
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Constructs a new button panel with the buttons given in buttons. This
         * can be a combination of button values in DialogConstants. Other
         * buttons are not created an can't be added in the future until
         * creating a new button panel.<br>
         * The buttons are enabled by default.
         * 
         * @param buttons
         *            the buttons that should be created in the button panel
         */
        public ButtonPanel(int buttons)
        {
            setLayout(new FlowLayout(FlowLayout.RIGHT, BUTTON_GAP,
                    BUTTON_VERTICAL_OFFSET));
            setBackground(Color.WHITE);

            minimumWidth_ = 5;
            // retry previous
            if((buttons & DialogConstants.BUTTON_RETRY) == DialogConstants.BUTTON_RETRY)
            {
                retryButton_ = new DialogButton(DialogConstants.BUTTON_RETRY);
                retryButton_.setMnemonic(KeyEvent.VK_R);
                retryButton_.addActionListener(this);
                add(retryButton_);

                minimumWidth_ += DialogButton.BUTTON_WIDTH + BUTTON_GAP;
            }
            else if((buttons & DialogConstants.BUTTON_PREVIOUS) == DialogConstants.BUTTON_PREVIOUS)
            {
                retryButton_ = new DialogButton(DialogConstants.BUTTON_PREVIOUS);
                retryButton_.setMnemonic(KeyEvent.VK_P);
                retryButton_.addActionListener(this);
                add(retryButton_);

                minimumWidth_ += DialogButton.BUTTON_WIDTH + BUTTON_GAP;
            }

            // ignore (next)
            if((buttons & DialogConstants.BUTTON_IGNORE) == DialogConstants.BUTTON_IGNORE)
            {
                ignoreButton_ = new DialogButton(DialogConstants.BUTTON_IGNORE);
                ignoreButton_.setMnemonic(KeyEvent.VK_I);
                ignoreButton_.addActionListener(this);
                add(ignoreButton_);

                minimumWidth_ += DialogButton.BUTTON_WIDTH + BUTTON_GAP;
            }
            else if((buttons & DialogConstants.BUTTON_NEXT) == DialogConstants.BUTTON_NEXT)
            {
                ignoreButton_ = new DialogButton(DialogConstants.BUTTON_NEXT);
                ignoreButton_.setMnemonic(KeyEvent.VK_X);
                ignoreButton_.addActionListener(this);
                add(ignoreButton_);

                minimumWidth_ += DialogButton.BUTTON_WIDTH + BUTTON_GAP;
            }

            // ok (finish, yes)
            if((buttons & DialogConstants.BUTTON_OK) == DialogConstants.BUTTON_OK)
            {
                okButton_ = new DialogButton(DialogConstants.BUTTON_OK);
                okButton_.setMnemonic(KeyEvent.VK_O);
                okButton_.addActionListener(this);
                add(okButton_);

                minimumWidth_ += DialogButton.BUTTON_WIDTH + BUTTON_GAP;
            }
            else if((buttons & DialogConstants.BUTTON_FINISH) == DialogConstants.BUTTON_FINISH)
            {
                okButton_ = new DialogButton(DialogConstants.BUTTON_FINISH);
                okButton_.setMnemonic(KeyEvent.VK_F);
                okButton_.addActionListener(this);
                add(okButton_);

                minimumWidth_ += DialogButton.BUTTON_WIDTH + BUTTON_GAP;
            }
            else if((buttons & DialogConstants.BUTTON_YES) == DialogConstants.BUTTON_YES)
            {
                okButton_ = new DialogButton(DialogConstants.BUTTON_YES);
                okButton_.setMnemonic(KeyEvent.VK_Y);
                okButton_.addActionListener(this);
                add(okButton_);

                minimumWidth_ += DialogButton.BUTTON_WIDTH + BUTTON_GAP;
            }

            // cancel (no)
            if((buttons & DialogConstants.BUTTON_CANCEL) == DialogConstants.BUTTON_CANCEL)
            {
                cancelButton_ = new DialogButton(DialogConstants.BUTTON_CANCEL);
                cancelButton_.setMnemonic(KeyEvent.VK_C);
                cancelButton_.addActionListener(this);
                add(cancelButton_);

                minimumWidth_ += DialogButton.BUTTON_WIDTH + BUTTON_GAP;
            }
            else if((buttons & DialogConstants.BUTTON_NO) == DialogConstants.BUTTON_NO)
            {
                cancelButton_ = new DialogButton(DialogConstants.BUTTON_NO);
                cancelButton_.setMnemonic(KeyEvent.VK_N);
                cancelButton_.addActionListener(this);
                add(cancelButton_);

                minimumWidth_ += DialogButton.BUTTON_WIDTH + BUTTON_GAP;
            }
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Supporting methods
        // <><><><><><><><><><><><><><><><><><><><><>
        /**
         * Enables or disable one or more buttons in the button panel. The
         * Parameter buttons can be a combination of several buttons in
         * DialogConstants.
         * 
         * @param buttons
         *            a combination of buttons from DialogConstants
         * @param enable
         *            set to true to enable the buttons, set to false to disable
         *            the buttons
         */
        public void setButtonsEnabled(int buttons, boolean enable)
        {
            if(GeneralUtils
                    .containsFlag(buttons, DialogConstants.BUTTON_CANCEL)
                    || GeneralUtils.containsFlag(buttons,
                            DialogConstants.BUTTON_NO))
            {
                if(cancelButton_ != null)
                {
                    cancelButton_.setEnabled(enable);
                }
            }
            if(GeneralUtils.containsFlag(buttons, DialogConstants.BUTTON_OK)
                    || GeneralUtils.containsFlag(buttons,
                            DialogConstants.BUTTON_FINISH)
                    || GeneralUtils.containsFlag(buttons,
                            DialogConstants.BUTTON_YES))
            {
                if(okButton_ != null)
                {
                    okButton_.setEnabled(enable);
                }
            }
            if(GeneralUtils
                    .containsFlag(buttons, DialogConstants.BUTTON_IGNORE)
                    || GeneralUtils.containsFlag(buttons,
                            DialogConstants.BUTTON_NEXT))
            {
                if(ignoreButton_ != null)
                {
                    ignoreButton_.setEnabled(enable);
                }
            }
            if(GeneralUtils.containsFlag(buttons, DialogConstants.BUTTON_RETRY)
                    || GeneralUtils.containsFlag(buttons,
                            DialogConstants.BUTTON_PREVIOUS))
            {
                if(retryButton_ != null)
                {
                    retryButton_.setEnabled(enable);
                }
            }
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementation of the ActionListener interface
        // <><><><><><><><><><><><><><><><><><><><><>
        /* @see java.awt.event.ActionListener */
        public void actionPerformed(ActionEvent event)
        {
            DialogButton button;

            if(event.getSource() instanceof DialogButton)
            {
                button = (DialogButton) event.getSource();
                fireButtonClickedEvent(button, button.getType());
            }
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Overriden methods from JComponent
        // <><><><><><><><><><><><><><><><><><><><><>
        /* @see javax.swing.JComponent */
        public Dimension getMinimumSize()
        {
            return new Dimension(minimumWidth_, dimension_.height);
        }

        /* @see javax.swing.JComponent */
        public Dimension getMaximumSize()
        {
            return getPreferredSize();
        }

        /* @see javax.swing.JComponent */
        public Dimension getPreferredSize()
        {
            Dimension size;

            if(getParent() != null)
            {
                size = getParent().getSize();
                dimension_ = new Dimension(size.width, dimension_.height);
            }

            return dimension_;
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Creates a new StandardModalDialog class. A standard modal dialog should
     * be used for all modal dialogs in viptool.
     */
    public StandardModalDialog(String title, String mainDescription,
            JComponent content, Image image, int buttons, int defaultButton)
    {
        super(MainFrame.getInstance(), title, true);

        if(content == null)
        {
            content = new JPanel();
        }

        // create the set for the listeners
        listeners_ = new HashSet();

        // container for all conmponents in the dialog
        contentPanel_ = new JPanel();
        contentPanel_.setLayout(new BorderLayout());

        // top panel
        topPanel_ = new TopPanel(mainDescription, image);

        // main panel, provided by the caller
        mainPanel_ = content;
        mainPanel_.setBorder(new LineBorder(Color.DARK_GRAY));

        // the button panel
        buttonPanel_ = new ButtonPanel(buttons);

        // add all components in the right order
        contentPanel_.add(topPanel_, BorderLayout.NORTH);
        contentPanel_.add(mainPanel_, BorderLayout.CENTER);
        contentPanel_.add(buttonPanel_, BorderLayout.SOUTH);

        // add the content panel to the dialog and layout
        setLayout(new BorderLayout());
        add(contentPanel_, BorderLayout.CENTER);

        // add the component listener to control resizing
        addComponentListener(new ComponentHandler());

        // set size and make the dialog resizeable
        setSize(getPreferredSize());
        setResizable(true);
        
        // center the component relative to the main frame
        GeneralUtils.centerComponent(this, MainFrame.getInstance());

    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Support of the listener pattern
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Fires the button clicked event. The JButton that was clicked as well as
     * the type (OK, CANCEL, ...) must be given. The type is of type
     * DialogConstans
     * 
     * @param button
     *            the button that was clicked
     * @param type
     *            the type of the clicked button (OK, CANCEL)
     */
    private void fireButtonClickedEvent(JButton button, int type)
    {
        Iterator it;
        IStandardDialogListener listener;

        it = listeners_.iterator();
        while (it.hasNext())
        {
            listener = (IStandardDialogListener) it.next();
            listener.buttonClicked(new ButtonClickedEvent(this, button, type));
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IStandardModalDialog interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.base.gui.dialog.IStandardModalDialog interface */
    public void setImage(Image image)
    {
        topPanel_.setImage(image);
    }

    /* @see viptool.base.gui.dialog.IStandardModalDialog interface */
    public void setMainDescription(String description)
    {
        topPanel_.setMainDescription(description);
    }

    /* @see viptool.base.gui.dialog.IStandardModalDialog interface */
    public void setDetailedDescription(String description)
    {
        topPanel_.setDetailedDescription(description);
    }

    /* @see viptool.base.gui.dialog.IStandardModalDialog interface */
    public void setContent(JComponent content)
    {
        contentPanel_.remove(mainPanel_);

        mainPanel_ = content;
        mainPanel_.setBorder(new LineBorder(Color.DARK_GRAY));
        contentPanel_.add(content, 1);

        mainPanel_.invalidate();
        validate();

        setSize(getMinimumSize());
        setResizable(true);
    }

    /* @see viptool.base.gui.dialog.IStandardModalDialog interface */
    public void setButtonsEnabled(int buttons, boolean enabled)
    {
        buttonPanel_.setButtonsEnabled(buttons, enabled);
    }

    /* @see viptool.base.gui.dialog.IStandardModalDialog interface */
    public void setNotice(String notice, int type)
    {
        topPanel_.setNotice(notice, type);
    }

    /* @see viptool.base.gui.dialog.IStandardModalDialog interface */
    public void removeNotice()
    {
        topPanel_.setNotice(null, DialogConstants.TYPE_WARNING);
    }

    /* @see viptool.base.gui.dialog.IStandardModalDialog interface */
    public void addStandardDialogListener(IStandardDialogListener listener)
    {
        listeners_.add(listener);
    }

    /* @see viptool.base.gui.dialog.IStandardModalDialog interface */
    public void removeStandardDialogListener(IStandardDialogListener listener)
    {
        listeners_.remove(listener);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Overriden methods from JComponent
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see javax.swing.JComponent */
    public Dimension getPreferredSize(){
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		return new Dimension((super.getPreferredSize().width<screen.width/2)?super.getPreferredSize().width:screen.height/2, (super.getPreferredSize().height<screen.height/2)?super.getPreferredSize().height:screen.height/2);
	}
    
}
