package viptool.impl.base.gui.dialog.filefilter;

import java.io.File;

import javax.swing.filechooser.FileFilter;

import viptool.base.error.FileInterpreterProviderException;
import viptool.base.io.IFileInterpreterProvider;
import viptool.impl.base.Workbench;
import viptool.project.FileSystem;
import viptool.project.IAbstractFile;

/**
 * This filter shows directories and files which end with ".prj" by assumption
 * that such files are project-files.
 */
public class Filter extends FileFilter
{

    // Provider to check if files are from this type
    private IFileInterpreterProvider provider_;

    public Filter(IFileInterpreterProvider provider)
    {
        provider_ = provider;
    }

    /**
     * This methode implements the filter, this means it returns true if the the
     * file match the format of the provider.
     */
    public boolean accept(File file)
    {
        IAbstractFile abstractFile = FileSystem.getInstance().createAbstractFile(
                file);
        try
        {
            if (file.isDirectory())
            {
                return true;
            }
            else if (provider_.matchFormat(abstractFile))
            {
                return true;
            }
        }
        catch (FileInterpreterProviderException e)
        {
            Workbench.getInstance().showExceptionInDialog(e);
            e.printStackTrace();
        }
        return false;
    }

    /**
     * This methode returns a description for the files which are shown by the
     * filter. This String will be shown at the JFileChooser as description for
     * the filter.
     * 
     * @return description of type
     */
    public String getDescription()
    {
        return provider_.getFileType();
    }
}
