package viptool.impl.base.gui.layoutmanager.listlayout;


/**
 * This class is a container for the constrains of the ListLayout. It contains
 * the offsets and the position into the list.
 */
public class ListLayoutConstraints {

	/**
	 * default offset
	 */
	public static final int DEFAULTSPACE = 5;
	
	private int pos_;

	private int spaceTop_;

	private int spaceBottom_;

	private int spaceLeft_;

	private int spaceRight_;
	
	
	/**
	 * This constructor create a new ListLayoutConstraint with default
	 * parameter set.
	 */
	public ListLayoutConstraints(){
		pos_ = -1;
		spaceTop_ = DEFAULTSPACE;
		spaceBottom_ = DEFAULTSPACE;
		spaceLeft_ = DEFAULTSPACE;
		spaceRight_ = DEFAULTSPACE;
	}
	
	/**
	 * This constructor create a new ListLayoutConstraint with the
	 * given position. This means the GUI component will be added
	 * at the given position into the list.
	 * 
	 * @param pos
	 * 		list position of the component
	 */
	public ListLayoutConstraints(int pos){
		pos_ = pos;
		spaceTop_ = DEFAULTSPACE;
		spaceBottom_ = DEFAULTSPACE;
		spaceLeft_ = DEFAULTSPACE;
		spaceRight_ = DEFAULTSPACE;
	}
	
	/**
	 * This constructor create a new ListLayoutConstraint with the
	 * given position and the given offset. This means the GUI 
	 * component will be added at the given position into the list.
	 * 
	 * @param pos
	 * 		list position of the component
	 * 
	 * @param allSpaces
	 * 		padding in px
	 */
	public ListLayoutConstraints(int pos, int allSpaces){
		pos_ = pos;
		spaceTop_ = allSpaces;
		spaceBottom_ = allSpaces;
		spaceLeft_ = allSpaces;
		spaceRight_ = allSpaces;
	}
	
	/**
	 * This constructor create a new ListLayoutConstraint with the
	 * given position and the given offset. This means the GUI 
	 * component will be added at the given position into the list.
	 * 
	 * @param pos
	 * 		list position of the component
	 * @param top
	 * 		padding over the component
	 * @param bottom
	 * 		padding under the component
	 * @param left
	 * 		padding left of the component
	 * @param right
	 * 		padding right of the component
	 */
	public ListLayoutConstraints(int pos, int top, int bottom, int left, int right){
		pos_ = pos;
		spaceTop_ = top;
		spaceBottom_ = bottom;
		spaceLeft_ = left;
		spaceRight_ = right;
	}

	/**
	 * This method returns the list position of the component
	 * 
	 * @return
	 * 		list position of the component
	 */
	public int getPos() {
		return pos_;
	}

	/**
	 * This method sets the list position of the component. This information is only
	 * used by adding the component!
	 * 
	 * @param pos
	 * 		list position of the component
	 */
	public void setPos(int pos) {
		this.pos_ = pos;
	}

	/**
	 * This method returns the padding under the component.
	 * 
	 * @return
	 * 		padding in px
	 */
	public int getSpaceBottom() {
		return spaceBottom_;
	}

	/**
	 * This method sets the padding under the component.
	 * 
	 * @param spaceBottom
	 * 		padding in px
	 */
	public void setSpaceBottom(int spaceBottom) {
		this.spaceBottom_ = spaceBottom;
	}

	/**
	 * This method returns the padding left of the component.
	 * 
	 * @return
	 * 		padding in px
	 */
	public int getSpaceLeft() {
		return spaceLeft_;
	}

	/**
	 * This method sets the padding left of the component.
	 * 
	 * @param spaceLeft
	 * 		padding in px
	 */
	public void setSpaceLeft(int spaceLeft) {
		this.spaceLeft_ = spaceLeft;
	}

	/**
	 * This method returns the padding right of the component.
	 * 
	 * @return
	 * 		padding in px
	 */
	public int getSpaceRight() {
		return spaceRight_;
	}

	/**
	 * This method sets the padding right of the component.
	 * 
	 * @param spaceRight
	 * 		padding in px
	 */
	public void setSpaceRight(int spaceRight) {
		this.spaceRight_ = spaceRight;
	}

	/**
	 * This method returns the padding over the component.
	 * 
	 * @return
	 * 		padding in px
	 */
	public int getSpaceTop() {
		return spaceTop_;
	}

	/**
	 * This method sets the padding over the component.
	 * 
	 * @param spaceTop
	 * 		padding in px
	 */
	public void setSpaceTop(int spaceTop) {
		this.spaceTop_ = spaceTop;
	}

}
