package viptool.impl.base.gui.list;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import viptool.impl.util.resources.ResourceLoader;
import viptool.util.resources.ResourceConstants;

public class ChangeableList extends JPanel implements ActionListener{
	
	private JButton up_;
	private JButton down_;
	private JButton kill_;
	private ChangeableListModel model_;
	private JList list_;
	private JPanel buttons_;
	
	public static ChangeableList generateList(Vector data, boolean showButtons){
		return new ChangeableList(data, showButtons);
	}
	
	private ChangeableList(Vector data, boolean showButtons){
		super();
		setLayout(new BorderLayout());
		
		model_ = new ChangeableListModel(data);
		list_ = new JList(model_);
		list_.setCellRenderer(new ChangeableListRenderer());
		list_.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		list_.addListSelectionListener(new ListAutoscrollListener());
		add(new JScrollPane(list_), BorderLayout.CENTER);
		
		
		buttons_ = new JPanel();
		buttons_.setLayout(new GridLayout(6,1));
		buttons_.setBackground(Color.WHITE);
		
		Icon icon = ResourceLoader.getInstance().getIcon(ResourceConstants.ICON_UP);
		up_ = new JButton(icon);
		up_.addActionListener(this);
		up_.setActionCommand("up");
		buttons_.add(up_);
		
		icon = ResourceLoader.getInstance().getIcon(ResourceConstants.ICON_DOWN);
		down_ = new JButton(icon);
		down_.addActionListener(this);
		down_.setActionCommand("down");
		buttons_.add(down_);
		
		icon = ResourceLoader.getInstance().getIcon(ResourceConstants.ICON_KILL);
		kill_ = new JButton(icon);
		kill_.addActionListener(this);
		kill_.setActionCommand("remove");
		buttons_.add(kill_);
		
		if(showButtons){
			JPanel buttpan = new JPanel();
			buttpan.setBackground(Color.WHITE);
			buttpan.setLayout(new FlowLayout());
			buttpan.add(buttons_);
			add(buttpan, BorderLayout.EAST);
		}
	}

	public JButton getDown() {
		return down_;
	}

	public JButton getKill() {
		return kill_;
	}

	public JButton getUp() {
		return up_;
	}

	public JList getList() {
		return list_;
	}

	public ChangeableListModel getModel() {
		return model_;
	}

	public void actionPerformed(ActionEvent e) {
		String com = e.getActionCommand();
		
		int index = list_.getSelectedIndex();
		int[] indices = list_.getSelectedIndices();
		
		if(com.equals("up") && index > 0){
			model_.switchElements(index, index-1);
			list_.setSelectedIndex(index-1);
		}else if(com.equals("down")){
			model_.switchElements(index, index+1);
			list_.setSelectedIndex(index+1);
		}else if(com.equals("remove")){
			for(int i=indices.length-1; i>=0; i--){
				model_.removeElement(indices[i]);
			}
		}
	}
	
	public void addButton(JButton button){
		buttons_.add(button);
	}
}
