package viptool.impl.base.gui.list;

import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.ListModel;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import viptool.base.event.ExtendedListDataEvent;

public class ChangeableListModel implements ListModel{

	Vector listeners_;
	Vector data_;
	HashMap dataToObject_;
	
	public ChangeableListModel(){
		listeners_ = new Vector();
		data_ = new Vector();
		dataToObject_ = new HashMap();
	}
	
	public ChangeableListModel(Vector data){
		listeners_ = new Vector();
		data_ = new Vector();
		dataToObject_ = new HashMap();
		addElements(data);
	}
	
	//-----------------------------------------------------------------
	
	public void addListDataListener(ListDataListener l) {
		listeners_.add(l);
	}
	
	public void removeListDataListener(ListDataListener l) {
		listeners_.remove(l);
	}
		
	private void callListenersAdded(Object obj){
		int index = data_.indexOf(obj);
		ListDataEvent event = new ExtendedListDataEvent(this, ListDataEvent.INTERVAL_ADDED, index, index, new Object[]{obj});
		
		Enumeration enumer = listeners_.elements();
		while(enumer.hasMoreElements()){
			ListDataListener listener = (ListDataListener)enumer.nextElement();
			listener.contentsChanged(event);
		}
	}
	
	private void callListenersAdded(Vector data){
		ListDataEvent event = new ExtendedListDataEvent(this, ListDataEvent.INTERVAL_ADDED, data_.indexOf(data.get(0)), data_.indexOf(data.get(data.size()-1)),data.toArray());
		
		Enumeration enumer = listeners_.elements();
		while(enumer.hasMoreElements()){
			ListDataListener listener = (ListDataListener)enumer.nextElement();
			listener.intervalAdded(event);
		}
	}
	
	private void callListenersChanged(Object obj){
		int index = data_.indexOf(obj);
		ListDataEvent event = new ExtendedListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, index, index,new Object[]{obj});
		
		Enumeration enumer = listeners_.elements();
		while(enumer.hasMoreElements()){
			ListDataListener listener = (ListDataListener)enumer.nextElement();
			listener.contentsChanged(event);
		}
	}
	
	private void callListenersChanged(int index){
		ListDataEvent event = new ExtendedListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, index, index,new Object[]{data_.get(index)});
		
		Enumeration enumer = listeners_.elements();
		while(enumer.hasMoreElements()){
			ListDataListener listener = (ListDataListener)enumer.nextElement();
			listener.contentsChanged(event);
		}
	}
	
	private void callListenersRemoved(int index, Object obj){
		ListDataEvent event = new ExtendedListDataEvent(this, ListDataEvent.INTERVAL_REMOVED, index, index, new Object[]{obj});
		
		Enumeration enumer = listeners_.elements();
		while(enumer.hasMoreElements()){
			ListDataListener listener = (ListDataListener)enumer.nextElement();
			listener.intervalRemoved(event);
		}
	}	
	
	//----------------------------------------------------------------

	public Object getElementAt(int index) {
		return data_.get(index);
	}

	public int getSize() {
		return data_.size();
	}

	//-----------------------------------------------------------------
	
	public void addElement(Object data){
		if(data != null && !data_.contains(data)){
			data_.add(data);
			callListenersAdded(data);
		}
	}
	
	public void addElements(Vector data){
		if(data != null && data.size() > 0){
			data_.addAll(data);
			callListenersAdded(data);
		}
	}
	
	public void addElements(Collection data){
		if(data != null && data.size() > 0){
			data_.addAll(data);
			callListenersAdded(data);
		}
	}
	
	public void insertElement(Object data, int index){
		if(data != null && index < data_.size()){
			data_.insertElementAt(data, index);
			callListenersAdded(data);
		}else if(index == data_.size()){
			addElement(data);
		}
	}
	
	public void insertElements(Vector data, int index){
		if(data != null && data.size() > 0 && index < data_.size()){
			for(int i=0; i<data.size(); i++){
				data_.insertElementAt(data.get(i), index+i);
			}
			callListenersAdded(data);
		}else if(index == data_.size()){
			addElements(data);
		}
	}
	
	public void changeElement(int index, Object newObject){
		if(newObject != null && index < data_.size())
		{
			data_.remove(index);
			data_.insertElementAt(newObject, index);
			callListenersChanged(index);
		}
	}

	public void changeElement(Object oldObject, Object newObject){
		if(newObject != null && newObject != null)
		{
			int index = data_.indexOf(oldObject);
			data_.remove(oldObject);
			data_.insertElementAt(newObject, index);
			callListenersChanged(index);
		}
	}
	
	public void removeElement(Object obj){
		if(obj != null){
			int index = data_.indexOf(obj);
			dataToObject_.remove(obj);
			data_.remove(obj);
			callListenersRemoved(index, obj);
		}
	}
	
	public void removeElement(int index){
		if(index < data_.size()){
			dataToObject_.remove(data_.get(index));
			Object obj = data_.get(index);
			data_.remove(index);
			callListenersRemoved(index, obj);
		}
	}
	
	public void removeElements(Vector objects){
		if(objects != null && objects.size() > 0){
			Enumeration enumer = objects.elements();
			while(enumer.hasMoreElements()){
				Object obj = enumer.nextElement();
				removeElement(obj);
			}
		}
	}
	
	public void removeElements(int startIndex, int endIndex){
		for(int i=endIndex; i>=startIndex; i--){
			dataToObject_.remove(data_.get(i));
			Object obj = data_.get(i);
			data_.remove(i);
			callListenersRemoved(i, obj);
		}
	}
	
	public void assignElementToObject(Object element, Object obj){
		if(element != null && obj != null){
			dataToObject_.put(element, obj);
			callListenersChanged(element);
		}
	}
	
	public void assignElementToObject(int index, Object obj){
		if(index < data_.size() && obj != null){
			dataToObject_.put(data_.get(index), obj);
			callListenersChanged(index);
		}
	}
	
	public void removeElementForObject(int index){
		if(index < data_.size()){
			dataToObject_.remove(data_.get(index));
			callListenersChanged(index);
		}
	}
	
	public void removeElementForObject(Object element){
		if(element != null){
			dataToObject_.remove(element);
			callListenersChanged(element);
		}
	}
	
	public Object getObjectForElement(Object element){
		if(element != null){
			return dataToObject_.get(element);
		}
		return null;
	}
	
	public Object getObjectForElement(int index){
		if(index < data_.size()){
			Object element = data_.get(index);
			return dataToObject_.get(element);
		}
		return null;
	}
	
	public void switchElements(int index1, int index2){
		if(index1 >= 0 && index1 < data_.size() && index2 >= 0 && index2 < data_.size()){
			Object element1 = data_.get(index1);
			Object element2 = data_.get(index2);
			data_.remove(index1);
			data_.insertElementAt(element2, index1);
			data_.remove(index2);
			data_.insertElementAt(element1, index2);
			callListenersChanged(index1);
			callListenersChanged(index2);
			
		}
	}
	
	public Vector getData(){
		return data_;
	}
	
	public void clear(){
		removeElements(0, data_.size()-1);
	}
}
