package viptool.impl.base.gui.menu;

//import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

//import org.w3c.dom.Document;
//import org.w3c.dom.Element;
//import org.w3c.dom.Node;
//import org.w3c.dom.NodeList;

import viptool.Constants;
import viptool.base.action.AbstractFileSelectionAction;
import viptool.base.action.ExtendedAction;
import viptool.base.action.IActionFilter;
//import viptool.base.action.IActionHandler;
import viptool.base.action.IActionPostProcessor;
//import viptool.base.action.WindowAction;
//import viptool.base.error.VipToolException;
//import viptool.base.error.XMLException;
import viptool.base.gui.menu.IContextMenuManager;
import viptool.base.gui.menu.IExtendedContextMenu;
import viptool.base.gui.menu.IMenuTemplate;
import viptool.base.gui.selection.IAbstractFileSelection;
//import viptool.base.gui.selection.ICustomFilter;
//import viptool.base.gui.selection.IFileTypeFilter;
import viptool.base.gui.selection.IFileTypeFilter;
import viptool.base.gui.selection.ISelection;
import viptool.base.gui.selection.ISelectionFilter;
import viptool.base.gui.selection.ISelectionModel;
//import viptool.impl.base.Workbench;
import viptool.impl.base.gui.selection.FileClosedFilter;
import viptool.impl.base.gui.selection.FileOpenedFilter;
import viptool.impl.base.gui.selection.FileTypeFilter;
import viptool.impl.base.gui.selection.SelectionModel;
//import viptool.impl.extension.extensionloader.ExtensionLoader;
import viptool.impl.logger.Logger;

/* @see viptool.base.gui.menu.IMenuTemplateManager */
public class ContextMenuManager implements IContextMenuManager {
	/** the element name for the the root element for xml menu files */
	public static final String ELEMENT_CONTEXTMENU = "contextmenu";

	public static final String ELEMENT_ENTRY = "entry";

	public static final String ELEMENT_SELECTIONMODEL = "selectionModel";

	public static final String ELEMENT_FILETYPEFILTER = "fileTypeFilter";

	public static final String ELEMENT_CUSTOMFILTER = "customFilter";

	// the one and only menu template manager instance
	private static IContextMenuManager instance_ = null;

	private HashMap selectionModels_;

	private IMenuTemplate contextMenuTemplate_;

	// private class Entry
	// {
	// private String id_;
	//
	// private String insertAt_;
	//
	// private int seek_;
	//
	// private ArrayList selectionModels_;
	//
	// public Entry(String id, String insertAt, int seek)
	// {
	// id_ = id;
	// insertAt_ = insertAt;
	// seek_ = seek;
	// }
	//
	// public String getId()
	// {
	// return id_;
	// }
	//
	// public String getInsertAt()
	// {
	// return insertAt_;
	// }
	//
	// public int getSeek()
	// {
	// return seek_;
	// }
	//
	// public void setSelectionModels(ArrayList models)
	// {
	// selectionModels_ = models;
	// }
	//
	// public ArrayList getSelectionModels()
	// {
	// return selectionModels_;
	// }
	// }

	private class ActionFilter implements IActionFilter {
		private ISelection selection_;

		public ActionFilter(ISelection selection) {
			selection_ = selection;
		}

		@Override
		public boolean matchesFilter(ExtendedAction action) {
			Iterator it;
			ArrayList list;
			ISelectionModel model;

			list = (ArrayList) selectionModels_.get(action.getIdKey());
			if (list == null || list.size() == 0) {
				return true;
			}

			it = list.iterator();
			while (it.hasNext()) {
				model = (ISelectionModel) it.next();

				if (model.isSelectionAccepted(selection_)) {
					return true;
				}
			}

			return false;
		}
	}

	private class ActionPostProcessor implements IActionPostProcessor {
		private IAbstractFileSelection selection_;

		public ActionPostProcessor(IAbstractFileSelection selection) {
			selection_ = selection;
		}

		@Override
		public void processAction(ExtendedAction action) {
			if (action instanceof AbstractFileSelectionAction) {
				((AbstractFileSelectionAction) action).setSelection(selection_);
			} else {
				Logger.getInstance().debug(
						"ContextMenuManager: Found an action that is "
								+ "not an AbstractFileSelectionAction ("
								+ action.getIdKey() + ")");
			}
		}

	}

	// <><><><><><><><><><><><><><><><><><><><><>
	// Constructors
	// <><><><><><><><><><><><><><><><><><><><><>
	/**
	 * Constructs an instance of the menu template manager.
	 */
	private ContextMenuManager() {
		selectionModels_ = new HashMap();
		contextMenuTemplate_ = new MenuTemplate(ExtendedContextMenu.class,
				Constants.ID_DEFAULT_CONTEXTMENU);
	}

	// <><><><><><><><><><><><><><><><><><><><><>
	// Implementation of the singletion pattern
	// <><><><><><><><><><><><><><><><><><><><><>
	/**
	 * Returns the one and only instance of the menu template manager. If no
	 * instance exists, a new one is created. Null will never be returned.
	 * 
	 * @return an instance of the menu template manager
	 */
	public static synchronized IContextMenuManager getInstance() {
		if (instance_ == null) {
			instance_ = new ContextMenuManager();
		}

		return instance_;
	}

	// private ISelectionModel readSelectionModel(Element parent)
	// {
	// int min;
	// int max;
	// Node child;
	// NodeList childs;
	// Element element;
	// String clazz;
	// String namePattern;
	// String contentProvider;
	// String type;
	// ISelectionFilter filter;
	// ISelectionModel selectionModel;
	//
	// selectionModel = new SelectionModel();
	// childs = parent.getChildNodes();
	// for (int i = 0; i < childs.getLength(); i++)
	// {
	// child = childs.item(i);
	// filter = null;
	//
	// if (child instanceof Element)
	// {
	// element = (Element) child;
	//
	// // the schema says that every filter has min and max attributes
	// min =
	// Integer.parseInt(element.getAttribute(IFileTypeFilter.ATTR_MINIMUM_MATCHES));
	// max =
	// Integer.parseInt(element.getAttribute(IFileTypeFilter.ATTR_MAXIMUM_MATCHES));
	//
	// // read the filter
	// if (child.getNodeName().equals(ELEMENT_FILETYPEFILTER))
	// {
	// namePattern = element.getAttribute(IFileTypeFilter.ATTR_NAMEPATTERN);
	// contentProvider =
	// element.getAttribute(IFileTypeFilter.ATTR_CONTENTPROVIDER);
	// type = element.getAttribute(IFileTypeFilter.ATTR_FILETYPE);
	//
	// filter = new FileTypeFilter(namePattern, type,
	// contentProvider, min, max);
	// }
	// else if (child.getNodeName().equals(ELEMENT_CUSTOMFILTER))
	// {
	// clazz = element.getAttribute(ICustomFilter.ATTR_CLASS);
	//
	// try
	// {
	// filter = (ISelectionFilter) ExtensionLoader.getInstance().loadClass(
	// clazz).newInstance();
	//
	// filter.setMaximumMatches(max);
	// filter.setMinimumMatches(min);
	// }
	// catch (Exception exception)
	// {
	// Workbench.getInstance().showExceptionInDialog(
	// new VipToolException(exception));
	// }
	// }
	//
	// // add the filter to the model
	// selectionModel.addSelectionFilter(filter);
	// }
	// }
	//
	// return selectionModel;
	// }
	//
	// private int readSelectionModels(Element parentNode, ArrayList models)
	// {
	// int i = 0;
	// Node child;
	// NodeList childs;
	//
	// childs = parentNode.getChildNodes();
	// for (i = 0; i < childs.getLength(); i++)
	// {
	// child = childs.item(i);
	//
	// if (child instanceof Element)
	// {
	// if (child.getNodeName().equals(ELEMENT_SELECTIONMODEL))
	// {
	// models.add(readSelectionModel((Element) child));
	// }
	// else
	// {
	// // end here, if this child is no selectionModel element no
	// // further will be. See the xml-schema for context menus
	// break;
	// }
	// }
	// }
	//
	// return i;
	// }
	//
	// private Entry readEntry(Element entryElement)
	// {
	// int seek;
	// String id;
	// String insertAt;
	// ArrayList selectionModels;
	//
	// Entry entry;
	// Element element;
	//
	// // read the action attributes id, insertAt and seek
	// element = (Element) entryElement.getElementsByTagName(
	// IActionHandler.ELEMENT_ACTION).item(0);
	//
	// id = element.getAttribute(WindowAction.ATTR_ID);
	// insertAt = element.getAttribute(WindowAction.ATTR_INSERTAT);
	// seek =
	// Integer.valueOf(element.getAttribute(WindowAction.ATTR_SEEK)).intValue();
	//
	// // now we can create the entry
	// entry = new Entry(id, insertAt, seek);
	//
	// // we hav to add the selection models now
	// selectionModels = new ArrayList();
	// readSelectionModels(entryElement, selectionModels);
	// entry.setSelectionModels(selectionModels);
	//
	// return entry;
	// }

	/**
	 * Stores a menu group (the fixed context menu) into the given template.
	 * 
	 * @param template
	 *            the template that should receive the group
	 */
	private void readMenuContainer(IMenuTemplate template) {
		// the container for the menuitems
		String container = "";

		// contains a list of selection models for a menuitem or a container
		ArrayList<ISelectionModel> models = new ArrayList<ISelectionModel>();
		// one filter for a menuitem or a container
		ISelectionFilter filter = new FileTypeFilter(".*", "", "", 0, 1);
		// one selection model containing filters
		ISelectionModel selecMo = new SelectionModel();
		selecMo.addSelectionFilter(filter);
		models.add(selecMo);

		// the list of selection models for the menuitem is saved
		selectionModels_.put("file.new", models);
		// the menuitem gets added with the attributes (id, container,
		// predecessor, slots behind predecessor)
		template.addMenuItem("file.new", container, "", 0);

		// the container gets added
		template.addMenuItem("file.load", container, "file.new", 1);

		selectionModels_.put("file.load.project", new ArrayList());
		template.addMenuItem("file.load.project", "file.load", "", 0);

		selectionModels_.put("file.load.workspace", new ArrayList());
		template.addMenuItem("file.load.workspace", "file.load",
				"file.load.project", 1);

		// the list of selection models for the container is saved
		selectionModels_.put("file.load", models);

		template.addMenuItem("separator", container, "file.load", 1); // seperator

		models = new ArrayList();
		filter = new FileTypeFilter(".*", "f", "", 1, -1);
		selecMo = new SelectionModel();
		selecMo.addSelectionFilter(filter);
		models.add(selecMo);
		filter = new FileTypeFilter(".*", "d", "", 1, -1);
		selecMo = new SelectionModel();
		selecMo.addSelectionFilter(filter);
		models.add(selecMo);

		selectionModels_.put("file.open", models);
		template.addMenuItem("file.open", container, "file.load", 2);

		models = new ArrayList();
		filter = new FileTypeFilter(".*", "f", "", 1, -1);
		selecMo = new SelectionModel();
		selecMo.addSelectionFilter(filter);
		models.add(selecMo);

		selectionModels_.put("file.openwith", models);
		template.addMenuItem("file.openwith", container, "file.open", 1);

		filter = new FileTypeFilter(".*", "f", "", 1, -1);
		selecMo = new SelectionModel();
		selecMo.addSelectionFilter(filter);
		models.add(selecMo);

		selectionModels_.put("file.openwith.empty", new ArrayList());
		template.addMenuItem("file.openwith.empty", "file.openwith", "", 0);

		selectionModels_.put("file.openwith", models);

		models = new ArrayList();
		filter = new FileTypeFilter(".*", "f", "", 1, -1);
		selecMo = new SelectionModel();
		selecMo.addSelectionFilter(filter);
		// customized filter
		filter = new FileClosedFilter();
		filter.setMaximumMatches(-1);
		filter.setMinimumMatches(1);
		selecMo.addSelectionFilter(filter);
		models.add(selecMo);
		filter = new FileTypeFilter(".*", "d", "", 1, -1);
		selecMo = new SelectionModel();
		selecMo.addSelectionFilter(filter);
		models.add(selecMo);

		selectionModels_.put("file.delete", models);
		template.addMenuItem("file.delete", container, "file.openwith", 1);

		models = new ArrayList();
		filter = new FileTypeFilter(".*", "f", "", 1, -1);
		selecMo = new SelectionModel();
		selecMo.addSelectionFilter(filter);
		models.add(selecMo);
		filter = new FileTypeFilter(".*", "d", "", 1, -1);
		selecMo = new SelectionModel();
		selecMo.addSelectionFilter(filter);
		models.add(selecMo);

		selectionModels_.put("file.copy", models);
		template.addMenuItem("file.copy", container, "file.delete", 1);

		selectionModels_.put("file.paste", models);
		template.addMenuItem("file.paste", container, "file.copy", 1);

		models = new ArrayList();
		filter = new FileTypeFilter(".*", "f", "", 1, -1);
		selecMo = new SelectionModel();
		selecMo.addSelectionFilter(filter);
		filter = new FileOpenedFilter();
		filter.setMaximumMatches(-1);
		filter.setMinimumMatches(1);
		selecMo.addSelectionFilter(filter);
		models.add(selecMo);

		selectionModels_.put("cm.file.save", models);
		template.addMenuItem("cm.file.save", container, "file.paste", 1);

		selectionModels_.put("cm.file.close", models);
		template.addMenuItem("cm.file.close", container, "cm.file.save", 1);

		models = new ArrayList();
		filter = new FileTypeFilter(".*", "f", "", 1, -1);
		selecMo = new SelectionModel();
		selecMo.addSelectionFilter(filter);
		models.add(selecMo);
		filter = new FileTypeFilter(".*", "d", "", 1, -1);
		selecMo = new SelectionModel();
		selecMo.addSelectionFilter(filter);
		models.add(selecMo);

		selectionModels_.put("file.rename", models);
		template.addMenuItem("file.rename", container, "cm.file.close", 1);

		template.addMenuItem("separator", container, "file.rename", 1); // seperator

		selectionModels_.put(container, new ArrayList());

		// /**
		// * Reads a menu group from an xml element into the given template. The
		// group
		// * element must represent a valid group as defined in the according
		// xml
		// * schema in viptool sdk.
		// *
		// * @param containerElement
		// * the group element that contains the information
		// * @param template
		// * the template that should receive the group
		// * @param container
		// * the container of the container that should be added (the new
		// * container will be a child of the given container)
		// */
		// private void readMenuContainer(Element containerElement,
		// IMenuTemplate template, String container)
		// {
		//
		// Node child;
		// NodeList childs;
		// Element element = null;
		//
		// int seek = 0;
		// int lastSeek = 0;
		// int firstIndex = 0;
		// String id = "";
		// String insertAt = "";
		// String lastId = "";
		//
		// Entry entry;
		// ArrayList selectionModels = new ArrayList();
		//
		// // read the selection models first
		// firstIndex = readSelectionModels(containerElement, selectionModels);
		//
		// // now read the menu
		// // we can do this with the index because the schema from context
		// menus
		// // require that the selectionModel elements came first
		// childs = containerElement.getChildNodes();
		// for (int i = firstIndex; i < childs.getLength(); i++)
		// {
		// child = childs.item(i);
		//
		// // if we have an element and the element has attributes we have an
		// // action or a group. Read the common attributes
		// if (child instanceof Element)
		// {
		// element = (Element) child;
		// if (element.getNodeName().equals(ELEMENT_ENTRY))
		// {
		// entry = readEntry(element);
		//
		// id = entry.getId();
		// insertAt = entry.getInsertAt();
		// seek = entry.getSeek();
		//
		// selectionModels_.put(id, entry.getSelectionModels());
		// }
		// else
		// {
		// id = element.getAttribute(WindowAction.ATTR_ID);
		// insertAt = element.getAttribute(WindowAction.ATTR_INSERTAT);
		// seek = Integer.valueOf(
		// element.getAttribute(WindowAction.ATTR_SEEK)).intValue();
		// }
		//
		// // if the insertAt parameter was not set, use the last id as
		// // insert at parameter
		// if (insertAt.length() == 0)
		// {
		// insertAt = lastId;
		// seek = lastSeek;
		// }
		//
		// // add the item to the template
		// template.addMenuItem(id, container, insertAt, seek);
		//
		// // if we had read a container, read the subelements too
		// if (child.getNodeName().equals(IActionHandler.ELEMENT_CONTAINER))
		// {
		// // now read the subelements of the group. The containing
		// // group should have the id of the group
		// readMenuContainer(element, template, id);
		// }
		//
		// // store the id as last id so we can set insertAt for the next
		// // elementif not given
		// if (!id.equals(IActionHandler.ELEMENT_SEPARATOR))
		// {
		// lastId = id;
		// lastSeek = 1;
		// }
		// else
		// {
		// // the separator has no own id, so we take the insertAt as
		// // id (must be set) and increment seek
		// lastId = insertAt;
		// lastSeek++;
		// }
		//
		// }
		// } // end for
		//
		// // store the selection models for the container
		// selectionModels_.put(container, selectionModels);
	}

	// <><><><><><><><><><><><><><><><><><><><><>
	// Implementation of the IContextMenuManager interface
	// <><><><><><><><><><><><><><><><><><><><><>

	/* @see viptool.impl.base.gui.menu.IMenuTemplateManager */

	/**
     * 
     */
	@Override
	public void addMenuEntry(ArrayList<ISelectionModel> modelList,
			String actionID, String container, String insertAt,
			int seek) {

		// the list of selection models for the menuitem is saved
		selectionModels_.put(actionID, modelList);
		// the menuitem gets added with the attributes (id, container,
		// predecessor, slots behind predecessor)
		contextMenuTemplate_.addMenuItem(actionID, container, insertAt, seek);
	}
	
	@Override
	public void removeMenuEntry(String actionID) {
		selectionModels_.remove(actionID);
		contextMenuTemplate_.removeMenuItem(actionID);
	}
	
	public IFileTypeFilter createFileTypeFilter(String name, String type, String content, int min,
            int max) {
		return new FileTypeFilter(name, type, content, min,
            max);
	}
	
	public ISelectionModel createSelectionModel(ArrayList<ISelectionFilter> filterList) {
		SelectionModel selecMo = new SelectionModel();
		for (ISelectionFilter filter : filterList) {
			selecMo.addSelectionFilter(filter);
		}
		return selecMo;
	}

	@Override
	public void insertContextMenuTemplate() {
		readMenuContainer(contextMenuTemplate_);
	}

	// public void addContextMenu(URI xmlFile)
	// {
	//
	// Document document = null;
	// Element menu = null;
	//
	// try
	// {
	// // parse the document using the current schema to validate
	// document = XMLUtils.parseXMLFile(xmlFile,
	// ResourceLoader.getInstance().getResource(
	// ResourceConstants.SCHEMA_CONTEXTMENU));
	//
	// // get the root element of the menu definition
	// menu = XMLUtils.findFirstElement(document, ELEMENT_CONTEXTMENU);
	//
	// // read the menu element (this is the first group with no id)
	// readMenuContainer(contextMenuTemplate_);
	// }
	// catch (XMLException exception)
	// {
	// Workbench.getInstance().showExceptionInDialog(exception);
	// }
	// }

	/* @see viptool.impl.base.gui.menu.IMenuTemplateManager */
	@Override
	public synchronized IExtendedContextMenu createContextMenu(
			IAbstractFileSelection selection) {
		return (IExtendedContextMenu) contextMenuTemplate_.createMenu(
				hashCode(), new ActionFilter(selection),
				new ActionPostProcessor(selection));
	}
}
