package viptool.impl.base.gui.menu;

import java.util.HashMap;

import javax.swing.JMenuBar;

import viptool.base.action.WindowActionPostProcessor;
import viptool.base.gui.IWindow;
import viptool.base.gui.menu.IMenuBarManager;
import viptool.base.gui.menu.IMenuTemplate;
import viptool.impl.base.action.ActionHandler;
import viptool.impl.logger.Logger;


/* @see viptool.base.gui.menu.IMenuTemplateManager */
public class MenuBarManager implements IMenuBarManager
{
    /** the element name for the the root element for xml menu files */
    public static final String     ELEMENT_MENU = "menu";

    // the one and only menu template manager instance
    private static IMenuBarManager instance_    = null;

    // the loaded templates
    // key: java.lang.String - the id of the template
    // value: viptool.base.gui.menu.IMenuTemplate - the template itself
    private HashMap<String, IMenuTemplate>                templates_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs an instance of the menu template manager.
     */
    private MenuBarManager()
    {
        templates_ = new HashMap<String, IMenuTemplate>();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the singletion pattern
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns the one and only instance of the menu template manager. If no
     * instance exists, a new one is created. Null will never be returned.
     * 
     * @return an instance of the menu template manager
     */
    public static synchronized IMenuBarManager getInstance()
    {
        if(instance_ == null)
        {
            instance_ = new MenuBarManager();
        }

        return instance_;
    }

//    /**
//     * Reads a menu group from an xml element into the given template. The group
//     * element must represent a valid group as defined in the according xml
//     * schema in viptool sdk.
//     * 
//     * @param groupElement
//     *            the group element that contains the information
//     * @param template
//     *            the template that should receive the group
//     * @param container
//     *            the container of the container that should be added (the new
//     *            container will be a child of the given container)
//     */
//    private void readMenuContainer(Element groupElement,
//            IMenuTemplate template, String container)
//    {
//        Node child;
//        NodeList childs;
//        Element element = null;
//
//        int seek = 0;
//        int lastSeek = 1;
//        String id = "";
//        String insertAt = "";
//        String lastId = "";
//
//        // now read the menu
//        childs = groupElement.getChildNodes();
//        for (int i = 0; i < childs.getLength(); i++)
//        {
//            child = childs.item(i);
//
//            // if we have an element and the element has attributes we have an
//            // action or a group. Read the common attributes
//            if(child instanceof Element)
//            {
//                element = (Element) child;
//
//                id = element.getAttribute(WindowAction.ATTR_ID);
//                insertAt = element.getAttribute(WindowAction.ATTR_INSERTAT);
//                seek = Integer.valueOf(
//                        element.getAttribute(WindowAction.ATTR_SEEK))
//                        .intValue();
//
//                // if the insertAt parameter was not set, use the last id as
//                // insert at parameter
//                if(insertAt.length() == 0)
//                {
//                    insertAt = lastId;
//                    seek = lastSeek;
//                }
//
//                // add the item to the template
//                template.addMenuItem(id, container, insertAt, seek);
//
//                // if we had read a container, read the subelements too
//                if(child.getNodeName().equals(IActionHandler.ELEMENT_CONTAINER))
//                {
//                    // now read the subelements of the group. The containing
//                    // group should have the id of the group
//                    readMenuContainer(element, template, id);
//                }
//
//                // store the id as last id so we can set insertAt for the next
//                // elementif not given
//                if(!id.equals(IActionHandler.ELEMENT_SEPARATOR))
//                {
//                    lastId = id;
//                    lastSeek = 1;
//                }
//                else
//                {
//                    // the separator has no own id, so we take the insertAt as
//                    // id (must be set) and increment seek
//                    lastId = insertAt;
//                    lastSeek++;
//                }
//
//            }
//        } // end for
//    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IMenuTemplateManager interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.impl.base.gui.menu.IMenuTemplateManager */
    public IMenuTemplate getTemplate(String id)
    {
        return (IMenuTemplate) templates_.get(id);
    }

    /* @see viptool.impl.base.gui.menu.IMenuTemplateManager */
    public void addTemplate(IMenuTemplate template)
    {
        templates_.put(template.getId(), template);
    }

//    /* @see viptool.impl.base.gui.menu.IMenuTemplateManager */
//    public void addTemplate(URI xmlFile)
//    {
//        Document document = null;
//        Element menu = null;
//        IMenuTemplate menuTemplate = null;
//        String inheritFrom = null;
//
//        try
//        {
//            // parse the document using the current schema to validate
//            document = XMLUtils.parseXMLFile(xmlFile, 
//                    ResourceLoader.getInstance().getResource(
//                            ResourceConstants.SCHEMA_MENU));
//
//            // get the root element of the menu definition
//            menu = XMLUtils.findFirstElement(document, ELEMENT_MENU);
//
//            // create a destination template
//            menuTemplate = new MenuTemplate(ExtendedMenuBar.class, menu
//                    .getAttribute(WindowAction.ATTR_ID));
//
//            // read if the menu should inherit from another menu
//            if(menu.hasAttribute(WindowAction.ATTR_INHERITFROM))
//            {
//                inheritFrom = menu.getAttribute(WindowAction.ATTR_INHERITFROM);
//
//                // stets the parent, this will prepare the menu.
//                // In fact this will clone the parent menu, install an listener
//                // on the parent menu and add all this to the template :-)
//                menuTemplate.setParent(getTemplate(inheritFrom));
//            }
//
//            // read the menu element (this is the first group with no id)
//            readMenuContainer(menu, menuTemplate, "");
//
//            // add the new template to the template manager
//            addTemplate(menuTemplate);
//        }
//        catch (XMLException exception)
//        {
//            Workbench.getInstance().showExceptionInDialog(exception);
//        }
//    }
    
    public void addEmptyTemplate(String id, String inheritFrom) {
        IMenuTemplate menuTemplate = null;

            // create a destination template
            menuTemplate = new MenuTemplate(ExtendedMenuBar.class, id);

            // read if the menu should inherit from another menu
            if(inheritFrom != null) {
            	
                menuTemplate.setParent(getTemplate(inheritFrom));
            }

            // add the new template to the template manager
            addTemplate(menuTemplate);
    }

    /* @see viptool.impl.base.gui.menu.IMenuTemplateManager */
    public JMenuBar createMenuBar(String id, IWindow window)
    {
        int instance;

        if(templates_.containsKey(id))
        {
            instance = window.hashCode();
            ActionHandler.getInstance().registerActionPostProcessor(instance,
                    new WindowActionPostProcessor(window));

            return (JMenuBar) getTemplate(id).createMenu(instance).getMenuItem(
                    "");
        }
        else
        {
            Logger.getInstance().warning(
                    "MenuTemplate '" + id
                            + "' not present in the MenuBarManager");

            return null;
        }
    }
}
