package viptool.impl.base.gui.selection;

import java.util.ArrayList;
import java.util.Iterator;

import viptool.base.error.FileInterpreterProviderException;
import viptool.base.gui.selection.IFileTypeFilter;
import viptool.impl.base.io.FileInterpreterManager;
import viptool.impl.extension.extensionloader.ExtensionLoader;
import viptool.impl.logger.Logger;
import viptool.project.IAbstractFile;
import viptool.project.IAbstractFolder;

public class FileTypeFilter implements IFileTypeFilter
{
    private int    min_;

    private int    max_;

    private int    type_;

    private String name_;

    private String content_;

    public FileTypeFilter(String name, String type, String content, int min,
            int max)
    {
        name_ = name;
        content_ = content;

        if(type.equals("f"))
        {
            type_ = IFileTypeFilter.FILETYPE_NORMALFILE;
        }
        else if(type.equals("d"))
        {
            type_ = IFileTypeFilter.FILETYPE_DIRECTORY;
        }
        else
        {
            type_ = IFileTypeFilter.FILETYPE_ALL;
        }

        setMinimumMatches(min);
        setMaximumMatches(max);
    }

    private int getFileType(IAbstractFile file)
    {
        return ((file instanceof IAbstractFolder) ? FILETYPE_DIRECTORY
                : FILETYPE_NORMALFILE);
    }

    public String getAllowedContent()
    {
        return content_;
    }

    public String getAllowedName()
    {
        return name_;
    }

    public int getAllowedType()
    {
        return type_;
    }

    public int getMaximumMatches()
    {
        return max_;
    }

    public int getMinimumMatches()
    {
        return min_;
    }

    public void setMinimumMatches(int min)
    {
        min_ = min;
    }

    public void setMaximumMatches(int max)
    {
        max_ = (max == -1) ? Integer.MAX_VALUE : max;
    }

    public boolean isSeletcionAccepted(ArrayList selection)
    {
        int matchCount;
        int fileType;
        boolean matches;
        Object object;
        Iterator it;
        IAbstractFile file;

        // iterate over the selection and search for not accepted entries
        matchCount = 0;
        it = selection.iterator();
        while (it.hasNext())
        {
            object = it.next();
            // we must have a IAbstractFileSelection, because the file type
            // filer
            // can only match files
            if(!(object instanceof IAbstractFile))
            {
                Logger.getInstance().warning(
                        "The given selection to "
                                + "FileTypeFilter.isSelectionAccepted is "
                                + "not an IAbstractFileSelection so the "
                                + "selection is not accepted.");
                return false;
            }

            matches = true;
            file = (IAbstractFile) object;

            // first check the file type
            fileType = getFileType(file);
            if(!((fileType & getAllowedType()) == fileType))
            {
                matches = false;
            }

            // check the name
            if(!file.getName().matches(getAllowedName()))
            {
                matches = false;
            }

            // check the content
            if(getAllowedContent().length() > 0)
            {
                try
                {
                    matches = FileInterpreterManager.getInstance()
                            .getFileInterpreterProvider(
                                    ExtensionLoader.getInstance().loadClass(
                                            getAllowedContent())).matchFormat(
                                    file);
                }
                catch (ClassNotFoundException exception)
                {
                    Logger.getInstance().debug(exception.getMessage() + "");
                    matches = false;
                }
                catch (FileInterpreterProviderException exception)
                {
                    Logger.getInstance().debug(exception.getMessage() + "");
                    matches = false;
                }
            }

            if(matches)
            {
                matchCount++;
                // it.remove();
            }
        }

        return matchCount >= getMinimumMatches()
                && matchCount <= getMaximumMatches()
                && matchCount == selection.size();
    }

}
