package viptool.impl.base.gui.toolbar;

import java.awt.Component;
import java.awt.Container;

import javax.swing.AbstractButton;
import javax.swing.JSeparator;

import viptool.base.action.ExtendedAction;
import viptool.base.action.IActionFilter;
import viptool.base.action.IActionHandler;
import viptool.base.action.IActionPostProcessor;
import viptool.base.gui.toolbar.IExtendedToolBar;
import viptool.base.gui.toolbar.IToolBarTemplate;
import viptool.base.gui.toolbar.event.ExtendedToolBarEvent;
import viptool.base.gui.toolbar.event.IExtendedToolBarListener;
import viptool.impl.base.action.StandardActionFilter;
import viptool.impl.base.action.StandardActionPostProcessor;

/* @see viptool.base.gui.toolbar.IToolBarTemplate */
public class ToolBarTemplate implements IToolBarTemplate,
        IExtendedToolBarListener
{
    // the id of the template
    private String          id_;

    // the toolbar of the template
    private ExtendedToolBar toolBar_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Creates a new toolbar template with the specified id. The toolbar
     * template uses an ExtendedToolBar to store the template. If a toolbar is
     * requested the template toolbar is cloned.
     * 
     * @param id
     *            the id for the new template
     */
    public ToolBarTemplate(String id)
    {
        id_ = id;
        toolBar_ = new ExtendedToolBar(0);
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // supporting methods
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * This method clones the given container to the given destination.
     * 
     * @param destination
     *            the toolbar where the clone should be set in
     * @param toolBar
     *            the container that has to be cloned
     */
    private void cloneContainer(IExtendedToolBar destination,
            Container toolBar, IActionFilter filter,
            IActionPostProcessor postProcessor)
    {
        ExtendedAction action;
        Component[] components;
        ExtendedAction addedAction;

        components = toolBar.getComponents();
        for (int i = 0; i < components.length; i++)
        {
            if(components[i] instanceof AbstractButton)
            {
                action = (ExtendedAction) ((AbstractButton) components[i])
                        .getAction();

                if(filter.matchesFilter(action))
                {
                    addedAction = destination
                            .addToolBarItem(action.getIdKey(), "", "", 0);
                    postProcessor.processAction(addedAction);
                }
            }
            else if(components[i] instanceof JSeparator)
            {
                // Its a separator, add it to the end of the current container
                // separators have all the same id
                destination
                        .addToolBarItem(IActionHandler.ELEMENT_SEPARATOR, "", "", 0);
            }
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IToolBarTemplate interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.base.gui.toolbar.IToolBarTemplate */
    public String getId()
    {
        return id_;
    }

    /* @see viptool.base.gui.toolbar.IToolBarTemplate */
    public void addToolBarItem(String id, String path, String insertAt, int seek)
    {
        toolBar_.addToolBarItem(id, path, insertAt, seek);
    }

    /* @see viptool.base.gui.toolbar.IToolBarTemplate */
    public void removeToolBarItem(String id)
    {
        toolBar_.removeToolBarItem(id);
    }

    /* @see viptool.base.gui.toolbar.IToolBarTemplate */
    public Component getToolBarItem(String id)
    {
        return toolBar_.getToolBarItem(id);
    }

    /* @see viptool.base.gui.toolbar.IToolBarTemplate */
    public void setParent(IToolBarTemplate parent)
    {
        IExtendedToolBar parentBar;

        // create a new toolbar
        toolBar_ = new ExtendedToolBar(0);

        // clone the parent bar as base for the new template
        parentBar = parent.createToolBar(0);

        // add us as listener to the parent, so we can change us, if the parent
        // changes
        parentBar.addExtendedToolBarListener(toolBar_);

        cloneContainer(toolBar_, parentBar.getToolBar(), new StandardActionFilter(), new StandardActionPostProcessor());
    }

    /* @see viptool.base.gui.toolbar.IToolBarTemplate */
    public IExtendedToolBar createToolBar(int instance)
    {
        return createToolBar(instance, new StandardActionFilter());
    }

    /* @see viptool.base.gui.toolbar.IToolBarTemplate */
    public IExtendedToolBar createToolBar(int instance, IActionFilter filter)
    {
        return createToolBar(instance, filter, new StandardActionPostProcessor());
    }

    /* @see viptool.base.gui.toolbar.IToolBarTemplate */
    public IExtendedToolBar createToolBar(int instance, IActionFilter filter,
            IActionPostProcessor postProcessor)
    {
        ExtendedToolBar toolBar;

        // if the template is requested we can just return our toolbar
        if(instance == 0)
        {
            return toolBar_;
        }

        // create a new toolbar that will be a clone of the template but with
        // an instance id that is not equal to zero
        toolBar = new ExtendedToolBar(instance);
        // let the new toolbar be notified about changes in the template
        toolBar_.addExtendedToolBarListener(toolBar);

        cloneContainer(toolBar, toolBar_, filter, postProcessor);

        return toolBar;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IExtendedToolBarListener interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.base.gui.toolbar.event.IExtendedToolBarListener */
    public void toolBarElementAdded(ExtendedToolBarEvent event)
    {
        toolBar_.addToolBarItem(event.getId(), event.getContainer(), event
                .getInsertAt(), event.getSeek());
    }

    /* @see viptool.base.gui.toolbar.event.IExtendedToolBarListener */
    public void toolBarElementRemoved(ExtendedToolBarEvent event)
    {
        toolBar_.removeToolBarItem(event.getId());
    }
}
