package viptool.impl.extension.gui.manager;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Date;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.filechooser.FileFilter;

import viptool.base.error.UnsupportedFileException;
import viptool.base.error.VipToolException;
import viptool.base.gui.dialog.DialogConstants;
import viptool.base.gui.dialog.event.ButtonClickedEvent;
import viptool.base.gui.dialog.event.IStandardDialogListener;
import viptool.base.io.IFileInterpreterProvider;
import viptool.extension.IExtensionInfo;
import viptool.extension.IExtensionListener;
import viptool.impl.base.Workbench;
import viptool.impl.base.gui.dialog.FileChooserDialog;
import viptool.impl.base.gui.dialog.StandardModalDialog;
import viptool.impl.base.gui.list.ChangeableList;
import viptool.impl.extension.ExtensionManager;
import viptool.impl.extension.gui.selector.ExtensionSelectorPanel;
import viptool.impl.logger.Logger;
import viptool.util.ClassFile;
import viptool.util.ClassFileInterpreter;

/**
 * This GUI component presents all information about an extension and supports it to edit these information.
 */
public class ExtensionInfoPanel extends JPanel implements MouseListener, ActionListener, IExtensionListener, IStandardDialogListener{

	//HTML format for description labels
	private static String TITLE_START = "<html><h3>";
	private static String TITLE_END = "</h3></html>";
	
	//HTML format for information
	private static String TEXT_START = "<html><span style='size:11; color:#0000FF;'>";
	private static String TEXT_END = "</span></html>";
	
	//HTML format for extension description
	private static String DESCRIPTION_START = "<html><span style='size:11; color:#0000FF; font-style:italic;'>";
	private static String DESCRIPTION_END = "</span></html>";
	
	//HTML format for button labels
	private static String BUTTON_START = "<html><span style='size:11; color:#0000FF; text-decoration:underline;'>";
	private static String BUTTON_END = "</span></html>";
	
	//background color
	private static final Color COLOR = new Color(250,250,183);
	
	//pointer to the extension info which is shown
	IExtensionInfo extInfo_;
	
	//buttons and textarea for extension description
	JLabel editDescription_;
	JTextArea descriptionText_;
	JLabel description_;
	JLabel cancelDescription_;
	JLabel saveDescription_;
	
	//button to show and change running state
	JLabel state_;
	
	//buttons to show extended information
	JPanel buttons_;
	JLabel editRequiredExtensions_;
	JLabel editRequiredClasses_;
	JLabel generateMainifest_;
	
	//buttons for classes
	JLabel addClass_;
	JLabel removeClass_;
	JLabel closeClass_;
	
	//buttons for extensions
	JLabel addExt_;
	JLabel removeExt_;
	JLabel closeExt_;
	
	//list of required extensions
	ChangeableList requiredExt_;
	//list of required classes
	ChangeableList requiredClass_;
	
	JPanel reqExtPan_;
	JPanel reqClassPan_;
	
	//load on start up check box
	JCheckBox autoload_;
	
	//cursor for buttons
	Cursor cursor_ = new Cursor(Cursor.HAND_CURSOR);
	
	ExtensionSelectorPanel selector_;
	
	/**
	 * This constructor create a new ExtensionInfoPanel for the given ExtensionInfo
	 * 
	 * @param info
	 * 		instance of IExtensionInfo
	 */
	public ExtensionInfoPanel(IExtensionInfo info){
		//initialize pointer
		extInfo_ = info;
		
		setBackground(COLOR);
		setLayout(new BorderLayout());
		
		//this part contains all information which are always shown
		JPanel firstPart = new JPanel();
		firstPart.setBackground(COLOR);
		firstPart.setLayout(new BoxLayout(firstPart, BoxLayout.Y_AXIS));
		
		JPanel panel;
		JLabel name;
		JLabel text;
		
		//Plugin-Type
		panel = new JPanel();
		panel.setBackground(COLOR);
		panel.setLayout(new FlowLayout(FlowLayout.LEFT));
		name = new JLabel(TITLE_START+"Plugin-Typ: "+TITLE_END);
		panel.add(name);
		text = new JLabel(TEXT_START+((extInfo_.isJarPlugin())?"packed as JAR":"packed into directory")+TEXT_END);
		panel.add(text);
		firstPart.add(panel);
		
		//Class-Name
		panel = new JPanel();
		panel.setBackground(COLOR);
		panel.setLayout(new FlowLayout(FlowLayout.LEFT));
		name = new JLabel(TITLE_START+"Class-Name: "+TITLE_END);
		panel.add(name);
		text = new JLabel(TEXT_START+extInfo_.getClassName()+TEXT_END);
		panel.add(text);
		firstPart.add(panel);
		
		//Extension Version
		panel = new JPanel();
		panel.setBackground(COLOR);
		panel.setLayout(new FlowLayout(FlowLayout.LEFT));
		name = new JLabel(TITLE_START+"Version: "+TITLE_END);
		panel.add(name);
		long version = extInfo_.getVersion();
		Date date = new Date(version);
		text = new JLabel(TEXT_START+date.toString()+TEXT_END);
		panel.add(text);
		firstPart.add(panel);
		
		//Extension description
		panel = new JPanel();
		panel.setBackground(COLOR);
		panel.setLayout(new FlowLayout(FlowLayout.LEFT));
		name = new JLabel(TITLE_START+"Description: "+TITLE_END);
		panel.add(name);
		description_ = new JLabel(DESCRIPTION_START+extInfo_.getDescription()+DESCRIPTION_END);
		panel.add(description_);
		
		//if extension is a file plugin the description is editable
		if(extInfo_.isFilePlugin()){
			descriptionText_ = new JTextArea(extInfo_.getDescription(),1,30);
			descriptionText_.setCursor(cursor_);
			descriptionText_.setVisible(false);
			panel.add(descriptionText_);
						
			editDescription_ = new JLabel(BUTTON_START+"edit"+BUTTON_END);
			editDescription_.setCursor(cursor_);
			editDescription_.addMouseListener(this);
			panel.add(editDescription_);
			
			saveDescription_ = new JLabel(BUTTON_START+"save"+BUTTON_END);
			saveDescription_.setVisible(false);
			saveDescription_.setCursor(cursor_);
			saveDescription_.addMouseListener(this);
			panel.add(saveDescription_);
			
			cancelDescription_ = new JLabel(BUTTON_START+"cancel"+BUTTON_END);
			cancelDescription_.setVisible(false);
			cancelDescription_.setCursor(cursor_);
			cancelDescription_.addMouseListener(this);
			panel.add(cancelDescription_);
		}
		firstPart.add(panel);
		
		//Path to the extension
		panel = new JPanel();
		panel.setBackground(COLOR);
		panel.setLayout(new FlowLayout(FlowLayout.LEFT));
		name = new JLabel(TITLE_START+"Path: "+TITLE_END);
		panel.add(name);
		String path = null;
		if(extInfo_.isFilePlugin()){
			path = extInfo_.getExtenisonClassFile().getFile().getParent();
		}else{
			path = new File(extInfo_.getJarFile().getName()).getParent();
		}
		text = new JLabel(TEXT_START+path+TEXT_END);
		panel.add(text);
		firstPart.add(panel);
		
		//main file of the extension
		panel = new JPanel();
		panel.setBackground(COLOR);
		panel.setLayout(new FlowLayout(FlowLayout.LEFT));
		name = new JLabel(TITLE_START+"File: "+TITLE_END);
		panel.add(name);
		String file = null;
		if(extInfo_.isFilePlugin()){
			file = extInfo_.getExtenisonClassFile().getFile().getName();
		}else{
			file = new File(extInfo_.getJarFile().getName()).getName();
		}
		text = new JLabel(TEXT_START+file+TEXT_END);
		panel.add(text);
		firstPart.add(panel);
		
		//load on start up switch
		panel = new JPanel();
		panel.setBackground(COLOR);
		panel.setLayout(new FlowLayout(FlowLayout.LEFT));
		name = new JLabel(TITLE_START+"Automatically load on start up: "+TITLE_END);
		panel.add(name);
		autoload_ = new JCheckBox();
		autoload_.setSelected(extInfo_.isAutoload());
		autoload_.setBackground(COLOR);
		autoload_.addActionListener(this);
		panel.add(autoload_);
		firstPart.add(panel);
		
		//running status display and switch
		panel = new JPanel();
		panel.setBackground(COLOR);
		panel.setLayout(new FlowLayout(FlowLayout.LEFT));
		name = new JLabel(TITLE_START+"State: "+TITLE_END);
		panel.add(name);
		String state = BUTTON_START + ((ExtensionManager.getInstance().isActivated(info))?"running":"stopped") + BUTTON_END;
		state_ = new JLabel(state);
		state_.setCursor(cursor_);
		state_.addMouseListener(this);
		panel.add(state_);
		firstPart.add(panel);
		
		add(firstPart, BorderLayout.NORTH);
		//end of first part
		
		//this part contains variable information which are not always shown like
		//required extensions or in case of a file plugin required classes
		JPanel secondPart = new JPanel();
		secondPart.setBackground(COLOR);
		secondPart.setLayout(new FlowLayout(FlowLayout.LEFT));
		
		buttons_ = new JPanel();
		buttons_.setBackground(COLOR);
		buttons_.setLayout(new FlowLayout(FlowLayout.LEFT));
		
		//buttons to show extended information
		editRequiredExtensions_ = new JLabel(BUTTON_START+"edit required extensions"+BUTTON_END);
		editRequiredExtensions_.addMouseListener(this);
		editRequiredExtensions_.setCursor(cursor_);
		buttons_.add(editRequiredExtensions_);
		if(extInfo_.isFilePlugin()){ //information only for file plugins 
			editRequiredClasses_ = new JLabel(BUTTON_START+"edit required classes"+BUTTON_END);
			editRequiredClasses_.addMouseListener(this);
			editRequiredClasses_.setCursor(cursor_);
			buttons_.add(editRequiredClasses_);
			generateMainifest_ = new JLabel(BUTTON_START+"generate Manifest"+BUTTON_END);
			generateMainifest_.addMouseListener(this);
			generateMainifest_.setCursor(cursor_);
			buttons_.add(generateMainifest_);
		}
		secondPart.add(buttons_);
		
		//generate removable panels
		panel = new JPanel();
		panel.setBackground(COLOR);
		panel.setLayout(new FlowLayout(FlowLayout.LEFT));
		
		//required extensions
		reqExtPan_ = new JPanel();
		reqExtPan_.setBackground(COLOR);
		reqExtPan_.setLayout(new BorderLayout());
		Vector<String> required =  new Vector<String>(extInfo_.getRequiredNames());
		requiredExt_ = ChangeableList.generateList(required, false);
		reqExtPan_.add(requiredExt_, BorderLayout.CENTER);
		JPanel butts = new JPanel();
		butts.setBackground(COLOR);
		butts.setLayout(new FlowLayout(FlowLayout.LEFT));
		addExt_ = new JLabel(BUTTON_START+"add Extension"+BUTTON_END);
		addExt_.setCursor(cursor_);
		addExt_.addMouseListener(this);
		butts.add(addExt_);
		removeExt_ = new JLabel(BUTTON_START+"remove Extension"+BUTTON_END);
		removeExt_.setCursor(cursor_);
		removeExt_.addMouseListener(this);
		butts.add(removeExt_);
		closeExt_ = new JLabel(BUTTON_START+"close Extensions"+BUTTON_END);
		closeExt_.setCursor(cursor_);
		closeExt_.addMouseListener(this);
		butts.add(closeExt_);
		reqExtPan_.add(butts, BorderLayout.SOUTH);
		reqExtPan_.setVisible(false);
		panel.add(reqExtPan_);
		secondPart.add(reqExtPan_);
		
		if(extInfo_.isFilePlugin()){
			//required classes
			reqClassPan_ = new JPanel();
			reqClassPan_.setBackground(COLOR);
			reqClassPan_.setLayout(new BorderLayout());
			requiredClass_ = ChangeableList.generateList(extInfo_.getClassFiles(), false);
			reqClassPan_.add(requiredClass_, BorderLayout.CENTER);
			butts = new JPanel();
			butts.setBackground(COLOR);
			butts.setLayout(new FlowLayout(FlowLayout.LEFT));
			addClass_ = new JLabel(BUTTON_START+"add Class"+BUTTON_END);
			addClass_.setCursor(cursor_);
			addClass_.addMouseListener(this);
			butts.add(addClass_);
			removeClass_ = new JLabel(BUTTON_START+"remove Class"+BUTTON_END);
			removeClass_.setCursor(cursor_);
			removeClass_.addMouseListener(this);
			butts.add(removeClass_);
			closeClass_ = new JLabel(BUTTON_START+"close Class list"+BUTTON_END);
			closeClass_.setCursor(cursor_);
			closeClass_.addMouseListener(this);
			butts.add(closeClass_);
			reqClassPan_.add(butts, BorderLayout.SOUTH);
			reqClassPan_.setVisible(false);
			panel.add(reqClassPan_);
			secondPart.add(reqClassPan_);
		}
			
		add(secondPart, BorderLayout.SOUTH);
		//end of second part
		
		//register as extension listener for running status
		ExtensionManager.getInstance().addExtensionListener(this);
	}

	/**
	 * implementation of mouse listener interface
	 * This method provides the functionality for the label buttons 
	 */
	public void mouseClicked(MouseEvent e) {
		
		if(e.getSource().equals(editDescription_) && e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1){
			description_.setVisible(false);
			descriptionText_.setVisible(true);
			editDescription_.setVisible(false);
			saveDescription_.setVisible(true);
			cancelDescription_.setVisible(true);
		}
		
		else if(e.getSource().equals(saveDescription_)&& e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1){
			extInfo_.setDescription(descriptionText_.getText());
			description_.setText(DESCRIPTION_START+descriptionText_.getText()+DESCRIPTION_END);
			
			descriptionText_.setVisible(false);
			description_.setVisible(true);
			
			editDescription_.setVisible(true);
			saveDescription_.setVisible(false);
			cancelDescription_.setVisible(false);
		}
		
		else if(e.getSource().equals(cancelDescription_) && e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1){
			descriptionText_.setVisible(false);
			description_.setVisible(true);
			
			editDescription_.setVisible(true);
			saveDescription_.setVisible(false);
			cancelDescription_.setVisible(false);
		}
		
		else if(e.getSource().equals(editRequiredExtensions_) && e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1){
			buttons_.setVisible(false);
			reqExtPan_.setVisible(true);
		}
		
		else if(e.getSource().equals(closeExt_) && e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1){
			buttons_.setVisible(true);
			reqExtPan_.setVisible(false);
		}
		
		else if(e.getSource().equals(editRequiredClasses_) && e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1){
			buttons_.setVisible(false);
			reqClassPan_.setVisible(true);
		}
		
		else if(e.getSource().equals(closeClass_) && e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1){
			buttons_.setVisible(true);
			reqClassPan_.setVisible(false);
		}
		
		
		else if(e.getSource().equals(addClass_) && e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1){
			ClassFileInterpreter interpreter = new ClassFileInterpreter();
			interpreter.testForSuperClass(false);
			FileChooserDialog dialog = new FileChooserDialog(new IFileInterpreterProvider[]{interpreter});
			File file = dialog.getSelectedFile();
			if(file != null){
				try {
					ClassFile cfile = ClassFile.getClassFile(file);
					extInfo_.addClassFile(cfile);
					requiredClass_.getModel().addElement(cfile);
				} catch (UnsupportedFileException e1) {
					Logger.getInstance().error("The ClassFile "+file.getName()+" seems to be no valid java class file!");
				} catch (VipToolException e1) {
					Workbench.getInstance().showExceptionInDialog(e1);
				}
			}
		}
		
		else if(e.getSource().equals(removeClass_) && e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1){
			ClassFile cf = (ClassFile)requiredClass_.getList().getSelectedValue();
			requiredClass_.getModel().removeElement(cf);
			extInfo_.removeClassFile(cf);
		}
		
		if(e.getSource().equals(addExt_) && e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1){
			Vector extensions = ExtensionManager.getInstance().getOrderedExtensions();
			extensions.remove(extInfo_);
			selector_ = new ExtensionSelectorPanel(extensions);
			StandardModalDialog dialog = new StandardModalDialog("Add Extension", "please select an extension", new JScrollPane(selector_), null, DialogConstants.BUTTON_SET_OKCANCEL, DialogConstants.BUTTON_OK);
			dialog.addStandardDialogListener(this);
			dialog.setVisible(true);
		}
		
		if(e.getSource().equals(removeExt_) && e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1){
			IExtensionInfo extInfo = (IExtensionInfo)requiredExt_.getList().getSelectedValue();
			requiredExt_.getModel().removeElement(extInfo);
			extInfo_.removeRequiredExtenison(extInfo);
		}
		
		//button to load/stop extension
		if(e.getSource().equals(state_) && e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1){
			System.out.println("change running!");
			try {
				if(ExtensionManager.getInstance().isActivated(extInfo_)){
					ExtensionManager.getInstance().stopExtension(extInfo_);
				}else{
					ExtensionManager.getInstance().loadExtension(extInfo_);
				}
			} catch (VipToolException e1) {
				Workbench.getInstance().getLogger().error(e1.getMessage());
			}
		}
		
		if(e.getSource().equals(generateMainifest_) && e.getClickCount() == 1 && e.getButton() == MouseEvent.BUTTON1){
			String manifest = extInfo_.getManifest();
			JFileChooser fchooser = new JFileChooser();
			fchooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			fchooser.setMultiSelectionEnabled(false);
			fchooser.setFileFilter(new FileFilter(){
				public boolean accept(File f) {
					if(f.isDirectory())
						return true;
					return false;
				}

				public String getDescription() {
					return "directories only";
				}	
			});
			
			fchooser.showSaveDialog(this);
			
			File file = fchooser.getSelectedFile();
			if(file != null && file.isDirectory()){
				File mani = new File(file.getAbsolutePath() + "/" + "MANIFEST.MF");
				try {
					if(mani.createNewFile()){
						FileOutputStream fout = new FileOutputStream(mani);
						BufferedOutputStream bout = new BufferedOutputStream(fout);
						OutputStreamWriter writer = new OutputStreamWriter(bout);
						writer.write(manifest);
						writer.flush();
						writer.close();
						bout.flush();
						bout.close();
						fout.flush();
						fout.close();
					}
				} catch (IOException e1) {
					Workbench.getInstance().showExceptionInDialog(new VipToolException(e1));
				}
			}
		}
		
		revalidate();
	}

	public void mouseEntered(MouseEvent e) {
		//not used here
	}

	public void mouseExited(MouseEvent e) {
		//not used here
	}

	public void mousePressed(MouseEvent e) {
		//not used here
	}

	public void mouseReleased(MouseEvent e) {
		//not used here
	}


	/**
	 * implementation of action listener
	 * This method change the load on start up state of the extensions
	 * if the user clicks on the checkbox.
	 */
	public void actionPerformed(ActionEvent e) {
		if(e.getSource().equals(autoload_)){
			extInfo_.setAutoload(autoload_.isSelected());
		}
	}


	public void extensionAdded(IExtensionInfo extInfo) {
		//nothing to do here
	}


	public void extensionRemoved(IExtensionInfo extInfo) {
		//nothing to do here
	}


	/**
	 * implementation of the ExtensionListener interface
	 * This method updates the running state of the extension if it was changed.
	 */
	public void extensionStarted(IExtensionInfo extInfo) {
		if(extInfo.equals(extInfo_)){
			state_.setText(BUTTON_START+((ExtensionManager.getInstance().isActivated(extInfo))?"running":"stopped")+BUTTON_END);
		}
	}

	/**
	 * implementation of the ExtensionListener interface
	 * This method updates the running state of the extension if it was changed.
	 */
	public void extensionStopped(IExtensionInfo extInfo) {
		if(extInfo.equals(extInfo_)){
			state_.setText(BUTTON_START+((ExtensionManager.getInstance().isActivated(extInfo))?"running":"stopped")+BUTTON_END);
		}
	}
	
	/**
	 * implementation of standard dialog listener
	 * This method belongs to mouseClicked. Its the second part of the add required extension functionality.
	 * The mouse listener open a StandardModalDialog with all known extensions. This method adds the extensions
	 * which was selected as required by user.
	 */
	public void buttonClicked(ButtonClickedEvent event) {
		StandardModalDialog dialog = (StandardModalDialog)event.getStandardDialog();
		dialog.setVisible(false);
		if(event.getType() == DialogConstants.BUTTON_OK){
			Object[] obj = selector_.getSelected();
			for(int i=0; i<obj.length; i++){
				IExtensionInfo extInfo = (IExtensionInfo)obj[i];
				extInfo_.addRequiredExtenison(extInfo);
				requiredExt_.getModel().addElement(extInfo);
			}
		}
		selector_ = null;
		dialog.dispose();
	}
}
