package viptool.impl.extension.gui.manager;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import viptool.base.gui.dialog.event.ButtonClickedEvent;
import viptool.base.gui.dialog.event.IStandardDialogListener;
import viptool.extension.IExtensionInfo;
import viptool.impl.base.gui.dialog.StandardModalDialog;
import viptool.impl.base.gui.dialog.WizardDialog;
import viptool.impl.base.gui.layoutmanager.listlayout.ListLayout;
import viptool.impl.extension.ExtensionManager;
import viptool.impl.extension.wizard.ExtensionsWizard;
import viptool.impl.util.resources.ResourceLoader;
import viptool.util.resources.ResourceConstants;

/**
 * This class is the container which represents the known extensions.
 */
public class ExtensionManagerPanel extends JPanel implements IStandardDialogListener, ActionListener{

	//list with all known extensions
	private ExtensionsList extensionsList_; 
	
	//background color
	private static final Color color_ = Color.WHITE;
	
	/**
	 * This constructor create a new instance of the ExtensionManager GUI.
	 */
	public ExtensionManagerPanel(){
		
		setLayout(new BorderLayout());
		setBackground(color_);
		
		JPanel panel = new JPanel();
		panel.setBackground(color_);
		panel.setLayout(new FlowLayout(FlowLayout.CENTER));
		
		JButton addExtension = new JButton("add Extension");
		addExtension.addActionListener(this);
		addExtension.setActionCommand("add");
		panel.add(addExtension);
		
		JButton removeExtension = new JButton("remove selected Extension");
		removeExtension.addActionListener(this);
		removeExtension.setActionCommand("remove");
		panel.add(removeExtension);
		
		add(panel, BorderLayout.NORTH);
		
		Icon iconup = ResourceLoader.getInstance().getIcon(ResourceConstants.ICON_UP);
		Icon icondown = ResourceLoader.getInstance().getIcon(ResourceConstants.ICON_DOWN);
		
		//up/down buttons
		panel = new JPanel();
		panel.setBackground(color_);
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		
		JButton up = new JButton(iconup);
		up.addActionListener(this);
		up.setActionCommand("up");
		panel.add(up);
		
		JButton down = new JButton(icondown);
		down.addActionListener(this);
		down.setActionCommand("down");
		panel.add(down);
		
		add(panel, BorderLayout.EAST);
		
		extensionsList_ = new ExtensionsList();
		add(new JScrollPane(extensionsList_), BorderLayout.CENTER);
	}
	

	//------------------------------------dialog listener-----------------
	
	/**
	 * This method close the Dialog if the OK-Button was pressed.
	 */
	public void buttonClicked(ButtonClickedEvent event) {
		StandardModalDialog dialog = (StandardModalDialog)event.getStandardDialog();
		dialog.setVisible(false);
		dialog.dispose();
	}

	//------------------------------------action listener-----------------
	
	/**
	 * This methode implements the functions of the buttons.
	 */
	public void actionPerformed(ActionEvent e) {
		
		if(e.getActionCommand().equals("add")){
			//open ExtensionWizzard if extensions should be added
			new WizardDialog(new ExtensionsWizard());
		}
		else if(e.getActionCommand().equals("remove")){
			//get current instance of ExtensionManager
			ExtensionManager extManager = ExtensionManager.getInstance();
			//get selected extensions
			Vector selectedExt = extensionsList_.getSelectedExtensions();
			Enumeration<IExtensionInfo> enumer = selectedExt.elements();
			while(enumer.hasMoreElements()){
				IExtensionInfo extInfo = enumer.nextElement();
				extManager.removeExtenison(extInfo);
			}
		}
		else if(e.getActionCommand().equals("up")){
			ExtensionManager extManager = ExtensionManager.getInstance();
			
			Vector selected = ((ListLayout)extensionsList_.getLayout()).getSelected();
			Enumeration enumer = selected.elements();
			while(enumer.hasMoreElements()){
				Component cmp = (Component)enumer.nextElement();
				if(cmp instanceof ListRowPanel){
					IExtensionInfo extinfo = ((ListRowPanel)cmp).getExtensionInfo();
					extManager.moveExtensionUp(extinfo);
				}
			}
			
			((ListLayout)extensionsList_.getLayout()).moveSelectedUp();
		}
		else if(e.getActionCommand().equals("down")){
			ExtensionManager extManager = ExtensionManager.getInstance();
			
			Vector selected = ((ListLayout)extensionsList_.getLayout()).getSelected();
			//it is necessary to start with the last selected extension
			for(int i=selected.size()-1; i>=0; i--){
				Component cmp = (Component)selected.get(i);
				if(cmp instanceof ListRowPanel){
					IExtensionInfo extinfo = ((ListRowPanel)cmp).getExtensionInfo();
					extManager.moveExtensionDown(extinfo);
				}
			}
			
			((ListLayout)extensionsList_.getLayout()).moveSelectedDown();
		}
	}
}
