package viptool.impl.extension.gui.manager;

import java.awt.Color;
import java.awt.Component;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.JPanel;

import viptool.extension.IExtensionInfo;
import viptool.extension.IExtensionListener;
import viptool.impl.base.gui.layoutmanager.listlayout.ListLayout;
import viptool.impl.base.gui.layoutmanager.listlayout.ListLayoutConstraints;
import viptool.impl.extension.ExtensionManager;

/**
 * This class presents all known extensions as list.
 */
public class ExtensionsList extends JPanel implements IExtensionListener{

	//background color
	private static final Color color_ = Color.WHITE;
	
	//extension to GUI translation
	private HashMap extensionsToGUI_; 
	
	//remove lock
	boolean removelock_ = false;
	
	
	/**
	 * create a new instance of ExtensionList
	 */
	public ExtensionsList(){
		
		extensionsToGUI_ = new HashMap();
		
		setLayout(new ListLayout(this));
		setBackground(color_);
		
		Vector extensions = ExtensionManager.getInstance().getOrderedExtensions();
		Enumeration<IExtensionInfo> enumer = extensions.elements();
		
		while(enumer.hasMoreElements()){
			IExtensionInfo extInfo = enumer.nextElement();
			add(new ListRowPanel(extInfo, false));
		}
		
		ExtensionManager.getInstance().addExtensionListener(this);
	}	

	//------------------------------------extension listener--------------
	
	public void extensionAdded(IExtensionInfo extInfo) {
		add(new ListRowPanel(extInfo, false), new ListLayoutConstraints());
		revalidate();
		repaint();
	}

	public void extensionRemoved(IExtensionInfo extInfo) {
		ListRowPanel row = (ListRowPanel)extensionsToGUI_.get(extInfo);
		((ListLayout)getLayout()).removeComponent(row);
		revalidate();
		repaint();
	}

	public void extensionStarted(IExtensionInfo extInfo) {
		//not required
	}

	public void extensionStopped(IExtensionInfo extInfo) {
		//not required		
	}
	
	//---------------overwritten methods of container--------------------
	public Component add(Component cmp){
		if(cmp instanceof ListRowPanel){
			extensionsToGUI_.put(((ListRowPanel)cmp).getExtensionInfo(), cmp);
		}
		return super.add(cmp);
	}
	
	public Component add(Component cmp, int index){
		if(cmp instanceof ListRowPanel){
			extensionsToGUI_.put(((ListRowPanel)cmp).getExtensionInfo(), cmp);
		}
		return super.add(cmp, index);
	}
	
	public void add(Component cmp, Object constraints){
		if(cmp instanceof ListRowPanel){
			extensionsToGUI_.put(((ListRowPanel)cmp).getExtensionInfo(), cmp);
		}
		super.add(cmp, constraints);
	}
	
	public void add(Component cmp, Object constraints, int index){
		if(cmp instanceof ListRowPanel){
			extensionsToGUI_.put(((ListRowPanel)cmp).getExtensionInfo(), cmp);
		}
		super.add(cmp, constraints, index);
	}
	
	public Component add(String name, Component cmp){
		if(cmp instanceof ListRowPanel){
			extensionsToGUI_.put(((ListRowPanel)cmp).getExtensionInfo(), cmp);
		}
		return super.add(name, cmp);
	}
	
	public void remove(Component cmp){
		removelock_ = true;
		if(cmp instanceof ListRowPanel){
			extensionsToGUI_.remove(((ListRowPanel)cmp).getExtensionInfo());
		}
		super.remove(cmp);
		removelock_ = false;
	}
	
	public void remove(int index){
		if(removelock_){
			super.remove(index);
		}else{
			remove(getComponent(index));
		}
	}
	
	//---------------supporting methods---------------------------------
	
	/**
	 * This method return a vector which contains the selected extensions.
	 */
	public Vector getSelectedExtensions(){
		Vector selectedExt = new Vector();
		Vector selected = ((ListLayout)getLayout()).getSelected();
		Enumeration enumer = selected.elements();
		while(enumer.hasMoreElements()){
			Object obj = enumer.nextElement();
			if(obj instanceof ListRowPanel){
				selectedExt.add(((ListRowPanel)obj).getExtensionInfo());
			}
		}
		return selectedExt;
	}
}
