package viptool.impl.extension.wizard;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import viptool.base.wizard.IWizardPage;
import viptool.extension.IExtensionInfo;
import viptool.impl.base.gui.dialog.FileChooserDialog;
import viptool.impl.base.gui.list.ChangeableList;
import viptool.impl.base.gui.list.ChangeableListModel;
import viptool.impl.extension.ExtensionInfo;
import viptool.impl.logger.Logger;
import viptool.util.ClassFile;
import viptool.util.ClassFileInterpreter;

/**
 * This WizzardPage enables it to set details for new class extensions before they are added.
 */
class ClassPage implements IWizardPage, ActionListener, DocumentListener{

	Vector classfiles_ = null;
	JTextField name_;
	JTextField desc_;
	ChangeableList list_;
	JCheckBox box_;
	JButton nameButton_;
	JButton descButton_;
	
	IExtensionInfo extInfo_;
	
	/**
	 * This constructor create a new classpage for the given classfile. 
	 * @param cfile
	 */
	public ClassPage(ClassFile cfile)
	{
		classfiles_ = new Vector();
		extInfo_ = new ExtensionInfo(cfile);
	}
	
	/**
	 * returns always true
	 */
	public boolean canGoToPrevoius() {
		return true;
	}

	/**
	 * Implementation of the IWizzardPage interface
	 * This method creates the content of the classpage
	 */
	public JComponent getContent() {
		JPanel panel = new JPanel();
		panel.setBackground(Color.WHITE);
		panel.setLayout(new BorderLayout());
		
		
		JPanel fields = new JPanel();
		fields.setBackground(Color.WHITE);
		fields.setLayout(new GridLayout(2,1));
		
		JPanel row = new JPanel();
		row.setBackground(Color.WHITE);
		row.setLayout(new FlowLayout(FlowLayout.LEFT));
		row.add(new JLabel("Name: "));
		name_ = new JTextField(extInfo_.getName(),30);
		name_.addActionListener(this);
		name_.getDocument().addDocumentListener(this);
		row.add(name_);
		nameButton_ = new JButton("OK");
		nameButton_.setEnabled(false);
		nameButton_.addActionListener(this);
		nameButton_.setActionCommand("name");
		row.add(nameButton_);
		fields.add(row);
		
		row = new JPanel();
		row.setBackground(Color.WHITE);
		row.setLayout(new FlowLayout(FlowLayout.LEFT));
		row.add(new JLabel("Description: "));
		desc_ = new JTextField(extInfo_.getDescription(),30);
		desc_.addActionListener(this);
		desc_.getDocument().addDocumentListener(this);
		row.add(desc_);
		descButton_ = new JButton("OK");
		descButton_.setEnabled(false);
		descButton_.addActionListener(this);
		descButton_.setActionCommand("desc");
		row.add(descButton_);
		fields.add(row);
		
		panel.add(fields, BorderLayout.NORTH);
		
		row = new JPanel();
		row.setBackground(Color.WHITE);
		row.setLayout(new BorderLayout());
		row.add(new JLabel("Required classes:"),BorderLayout.NORTH);
		list_ = ChangeableList.generateList(classfiles_, false);
		row.add(new JScrollPane(list_), BorderLayout.CENTER);
		JPanel buttons = new JPanel();
		buttons.setBackground(Color.WHITE);
		buttons.setLayout(new FlowLayout(FlowLayout.LEFT));
		JButton change = new JButton("add class-File");
		change.addActionListener(this);
		change.setActionCommand("addClass");
		buttons.add(change);
		change = new JButton("remove class-File");
		change.addActionListener(this);
		change.setActionCommand("removeClass");
		buttons.add(change);
		row.add(buttons, BorderLayout.SOUTH);
		panel.add(row, BorderLayout.CENTER);
		
		row = new JPanel();
		row.setBackground(Color.WHITE);
		row.setLayout(new FlowLayout(FlowLayout.RIGHT));
		row.add(new JLabel("start extension when VipTool starts"));
		box_ = new JCheckBox();
		row.add(box_);
		panel.add(row,BorderLayout.SOUTH);
		
		return panel;
	}

	/**
	 * Implementation of the IWizzardPage interface
	 * returns null
	 */
	public Image getImage() {
		return null;
	}

	/**
	 * Implementation of the IWizzardPage interface
	 */
	public String getMainDescription() {
		return "Add VipTool-Extension";
	}

	/**
	 * Implementation of the functionality of the buttons.
	 */
	public void actionPerformed(ActionEvent e) {
		String com = e.getActionCommand();
		
		if(com.equals("name")){
			extInfo_.setName(name_.getText());
			nameButton_.setEnabled(false);
		}
		else if(com.equals("desc")){
			extInfo_.setDescription(desc_.getText());
			descButton_.setEnabled(false);
		}
		else if(com.equals("addClass")){
			FileChooserDialog dialog = new FileChooserDialog(new ClassFileInterpreter[]{new ClassFileInterpreter()});
			File file = dialog.getSelectedFile();
			if(file != null){
				try{
					ClassFile cf = ClassFile.getClassFile(file);
					list_.getModel().addElement(cf);
					extInfo_.addClassFile(cf);
				}catch (Exception ex){
					Logger.getInstance().error("The file "+file.getName()+" isn't a valid class file!");
				}
			}
		}else if(com.equals("removeClass")){
			int[] indices = list_.getList().getSelectedIndices();
			ChangeableListModel model = list_.getModel();
			for(int i=0; i<indices.length; i++){
				extInfo_.removeClassFile((ClassFile)model.getElementAt(indices[i]));
				model.removeElement(indices[i]);
			}
		}
	}

	public void changedUpdate(DocumentEvent e) {
		if(e.getDocument() == name_.getDocument()){
			nameButton_.setEnabled(true);
		}else if(e.getDocument() == desc_.getDocument()){
			descButton_.setEnabled(true);
		}
	}

	public void insertUpdate(DocumentEvent e) {
		if(e.getDocument() == name_.getDocument()){
			nameButton_.setEnabled(true);
		}else if(e.getDocument() == desc_.getDocument()){
			descButton_.setEnabled(true);
		}
	}

	public void removeUpdate(DocumentEvent e) {
		if(e.getDocument() == name_.getDocument()){
			nameButton_.setEnabled(true);
		}else if(e.getDocument() == desc_.getDocument()){
			descButton_.setEnabled(true);
		}
	}	
	
	public IExtensionInfo getExtensionInfo(){
		return extInfo_;
	}
}