package viptool.impl.extension.wizard;

import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;
import java.util.jar.JarFile;

import viptool.base.error.UnsupportedFileException;
import viptool.base.error.VipToolException;
import viptool.base.gui.dialog.DialogConstants;
import viptool.base.gui.dialog.IStandardModalDialog;
import viptool.base.wizard.IWizard;
import viptool.base.wizard.IWizardPage;
import viptool.extension.Extension;
import viptool.extension.IExtensionInfo;
import viptool.impl.base.Workbench;
import viptool.impl.base.gui.dialog.filefilter.Filter;
import viptool.impl.extension.ExtensionInfo;
import viptool.impl.extension.ExtensionJarInterpreter;
import viptool.impl.extension.ExtensionManager;
import viptool.util.ClassFile;
import viptool.util.ClassFileInterpreter;

/**
 * This Wizard adds new extensions to the list of known extensions.
 */
public class ExtensionsWizard implements IWizard{

	//Array for WizzardPages
	IWizardPage[] pages_;
	//array-index of the shown page
	int page_;
	//dialog for this wizzard
	IStandardModalDialog dialog_;
	//ExtensionInfo of the new extension
	IExtensionInfo extInfo_;

	public ExtensionsWizard()
	{
		pages_ = new IWizardPage[2];
		pages_[0] = new FileChooserPage();
	}
	
	public void cancel() {
	}

	public void finish() {
		if(pages_[1] instanceof TextPage){
			TextPage page = (TextPage)pages_[1];
			if(page.check_.isSelected()){
				extInfo_.setAutoload(true);
			}else{
				extInfo_.setAutoload(false);
			}
		}
		else if(pages_[1] instanceof ClassPage){
			ClassPage page = (ClassPage)pages_[1];
			extInfo_ = page.getExtensionInfo();
			if(page.box_.isSelected()){
				extInfo_.setAutoload(true);
			}else{
				extInfo_.setAutoload(false);
			}
		}
		else if(pages_[1] instanceof MultiExtensionsPage){
			Object[] exts = ((MultiExtensionsPage)pages_[1]).getSelectedExtensions();
			boolean autoload = ((MultiExtensionsPage)pages_[1]).getAutoload();
			boolean load = ((MultiExtensionsPage)pages_[1]).getLoad();
			boolean added;
			Vector<IExtensionInfo> loadExt = new Vector<IExtensionInfo>();
			ExtensionManager extManager = ExtensionManager.getInstance();
			for(int i=0; i<exts.length; i++){
				if(autoload){
					((IExtensionInfo)exts[i]).setAutoload(true);
				}
				added = extManager.addExtension(((IExtensionInfo)exts[i]));
				if(added && load){
					loadExt.add(((IExtensionInfo)exts[i]));
				}
			}
			Enumeration<IExtensionInfo> enumer = loadExt.elements();
			IExtensionInfo extinfo = null;
			while(enumer.hasMoreElements()){
				try {
					extinfo = enumer.nextElement();
					extManager.loadExtension(extinfo);
				} catch (VipToolException e) {
					Workbench.getInstance().getLogger().error(extinfo+": "+e.getMessage());
				}
			}
		}
		
		if(extInfo_ != null){
			ExtensionManager.getInstance().addExtension(extInfo_);
		}
		
		
	}

	public IWizardPage getFirstPage(IStandardModalDialog dialog) {
		dialog_ = dialog;
		dialog.setButtonsEnabled(DialogConstants.BUTTON_NEXT, true);
		dialog.setButtonsEnabled(DialogConstants.BUTTON_FINISH, false);
		page_=0;
		return pages_[0];
	}

	public IWizardPage getNextPage() {
		if(pages_.length>page_+1)
		{
			File file = ((FileChooserPage)pages_[0]).getSelectedFile();
			if(file != null && file.getName().endsWith(".jar"))
			{
				try{
					JarFile jfile = new JarFile(file);
					extInfo_ = new ExtensionInfo(jfile);
					IExtensionInfo[] otherExt = extInfo_.getRequiredExtensions();
					String other = null;
					if(otherExt.length > 0){
						for(int i=0; i<otherExt.length; i++){
							other += " "+otherExt[i].getName(); 	
						}
					}else{
						other = "no other extensions required";
					}
					String text =  "<html><basefont size='16' color='#FF0000'>"+extInfo_.getName()+"</font><br><basefont size='12'"+
					" color='#0000FF'>"+extInfo_.getDescription()+"</font><br><basefont size='12' color='#000000'>"+extInfo_.getClassName()+"</font><br>"+
					" <basefont size='12' color='#000000'>"+other+"</font><br></html>";
					
					
					pages_[1] = new TextPage("Add VipTool-Extension",text,"start extension when VipTool starts", true);
					dialog_.setButtonsEnabled(DialogConstants.BUTTON_FINISH, true);
				}catch (Exception e){
					pages_[1] = new TextPage("No valid VipTool-Extension selected",
							"The jar-File which you have selected is no valid VipTool-Extension.", "", false);
					dialog_.setButtonsEnabled(DialogConstants.BUTTON_FINISH, false);
				}
			}
			else if(file != null && file.isFile() && file.getName().endsWith(".class"))
			{
				try {
					ClassFile cfile = ClassFile.getClassFile(file);
					pages_[1]=new ClassPage(cfile);
					dialog_.setButtonsEnabled(DialogConstants.BUTTON_FINISH, true);
				} catch (UnsupportedFileException e) {
					pages_[1] = new TextPage("No valid VipTool-Extension selected",
							"The class-File which you have selected is no valid VipTool-Extension.", "", false);
					dialog_.setButtonsEnabled(DialogConstants.BUTTON_FINISH, false);
				}
			}else if(file != null && file.isDirectory()){
				Vector files = new Vector();
				searchExtensions(files, file);
				Vector<IExtensionInfo> extensions = new Vector<IExtensionInfo>();
				
				if(files.size() > 0){
					Enumeration enumer = files.elements();
					while(enumer.hasMoreElements()){
						File f = (File)enumer.nextElement();
						//IExtensionInfo extInfo = null;
						if(f.getName().endsWith(".jar")){
							try {
								extensions.add(new ExtensionInfo(new JarFile(f)));
							} catch (IOException e) {
								e.printStackTrace();
							} catch (VipToolException e) {
								e.printStackTrace();
							}
						}else if(f.getName().endsWith(".class")){
							try {
								extensions.add(new ExtensionInfo(ClassFile.getClassFile(f)));
							} catch (UnsupportedFileException e) {
								e.printStackTrace();
							}
						}
						
						
						pages_[1] = new MultiExtensionsPage(extensions);
						dialog_.setButtonsEnabled(DialogConstants.BUTTON_FINISH, true);
					}
					
				}else{
					pages_[1] = new TextPage("no Extensions found!", "Into this foler and it's subfolders were no Extensions found.", "", false);
					dialog_.setButtonsEnabled(DialogConstants.BUTTON_FINISH, false);
				}
			}else
			{
				pages_[1] = new TextPage("no File selected!", "Please go back and select a file or press cancel.", "", false);
				dialog_.setButtonsEnabled(DialogConstants.BUTTON_FINISH, false);
			}
			
			if(pages_.length <= page_+1)
			{
				dialog_.setButtonsEnabled(DialogConstants.BUTTON_NEXT, false);
			}
			
			page_++;
			return pages_[page_];
		}
		return null;
	}

	public IWizardPage getPreviousPage() {
		if(page_>0)
		{
			page_--;
			return pages_[page_];
		}
		return null;
	}

	public String getTitle() {
		return "load Extensions";
	}

	private void searchExtensions(Vector files, File path){
		File[] subfiles = path.listFiles();
		
		ClassFileInterpreter interpreter = new ClassFileInterpreter();
		interpreter.setSuperClass(Extension.class.getName());
		interpreter.testForSuperClass(true);
		Filter classfilter = new Filter(interpreter);
		
		Filter jarFilter = new Filter(new ExtensionJarInterpreter());
		
		for(int i=0; i<subfiles.length; i++){
			File file = subfiles[i];
			if(file.getName().endsWith(".jar")){
				if(jarFilter.accept(file)){
					files.add(file);
				}
			}else if(file.getName().endsWith(".class")){
				if(classfilter.accept(file)){
					files.add(file);
				}
			}else if(file.isDirectory()){
				searchExtensions(files, file);
			}
		}
	}
	
	
	public void windowActivated(WindowEvent e) {}

	public void windowClosed(WindowEvent e) {}

	public void windowClosing(WindowEvent e) {}

	public void windowDeactivated(WindowEvent e) {}

	public void windowDeiconified(WindowEvent e) {}

	public void windowIconified(WindowEvent e) {}

	public void windowOpened(WindowEvent e) {}
	
}


