package viptool.impl.extension.wizard;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Image;
import java.io.File;

import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JPanel;

import viptool.base.wizard.IWizardPage;
import viptool.extension.Extension;
import viptool.impl.base.gui.dialog.filefilter.Filter;
import viptool.impl.extension.ExtensionJarInterpreter;
import viptool.util.ClassFileInterpreter;

/**
 * This WizzarPage contains a file chooser. It can be choosen class files which 
 * extend Extension, jar-files which contains an extension and directories into
 * which extensions will be searched. 
 */
public class FileChooserPage implements IWizardPage{

	//reference to filechooser
	JFileChooser chooser_;
	
	/**
	 * Implementation of the IWizzardPage interface
	 * returns always false because its the first page
	 */
	public boolean canGoToPrevoius() {
		return false;
	}

	/**
	 * Implementation of the IWizzardPage interface
	 * create the content of the filechooser
	 */
	public JComponent getContent() {
		JPanel panel = new JPanel();
		panel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
		chooser_ = new JFileChooser();
		chooser_.setBackground(Color.WHITE);
		chooser_.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		chooser_.setMultiSelectionEnabled(false);
		chooser_.setControlButtonsAreShown(false);
		Filter jarFilter = new Filter(new ExtensionJarInterpreter());
		chooser_.addChoosableFileFilter(jarFilter);
		ClassFileInterpreter interpreter = new ClassFileInterpreter();
		interpreter.setSuperClass(Extension.class.getName());
		interpreter.testForSuperClass(true);
		chooser_.addChoosableFileFilter(new Filter(interpreter));
		chooser_.setFileFilter(jarFilter);
		panel.add(chooser_);
		return panel;
	}
	
	/**
	 * This method returns the selected file.
	 * 
	 * @return
	 * 		selected file
	 */
	public File getSelectedFile(){
		return chooser_.getSelectedFile();
	}

	/**
	 * Implementation of the IWizzardPage interface
	 * This method returns null because there is no image for this dialog.
	 */
	public Image getImage() {
		return null;
	}

	/**
	 * Implementation of the IWizzardPage interface
	 */
	public String getMainDescription() {
		return "Select a directory or a jar-file";
	}
	
}