package viptool.impl.filesystem;

import java.util.Iterator;

import viptool.filesystem.IFileSystem;
import viptool.filesystem.IFileSystemSession;
import viptool.filesystem.error.FileExistsException;
import viptool.filesystem.error.FileSystemException;
import viptool.filesystem.error.InvalidContentException;
import viptool.filesystem.error.InvalidNameException;
import viptool.filesystem.error.NoSuchFileException;

public class FileSystemSession implements IFileSystemSession
{
    private IFileSystem fileSystem_;

    private String      currentFolder_;

    public FileSystemSession(IFileSystem fileSystem)
    {
        fileSystem_ = fileSystem;
        
        try
        {
            changeFolder("/");
        }
        catch(NoSuchFileException exception)
        {
            // impossible
        }
    }

    private String makeAbsolute(String name) throws InvalidNameException
    {
        int index;
        String currentFolder = currentFolder_;
        
        // remove /./ constructs
        name = name.replaceAll("/\\./", "");
        
        if(name.equals("."))
        {
            return getCurrentFolder();
        }

        while (name.startsWith("../"))
        {
            if(currentFolder.length() <= 1)
            {
                throw new InvalidNameException();
            }

            index = currentFolder.lastIndexOf('/', currentFolder.length() - 1);

            currentFolder = currentFolder.substring(0, index + 1);
            name = name.substring(3);
        }

        if(!name.startsWith("/"))
        {
            name = currentFolder_ + name;
        }

        return name;
    }

    public boolean isFolder(String folder)
    {
        try
        {
            return fileSystem_.isFolder(makeAbsolute(folder));
        }
        catch (FileSystemException exception)
        {
            return false;
        }
    }

    public String[] getFolderContent(String folder)
            throws InvalidNameException, NoSuchFileException
    {
        return fileSystem_.getFolderContent(makeAbsolute(folder));
    }

    public Iterator getFolderIterator(String folder)
            throws NoSuchFileException, InvalidNameException
    {
        return fileSystem_.getFolderIterator(makeAbsolute(folder));
    }

    public void changeFolder(String folder) throws NoSuchFileException
    {
        int index;
        String currentFolder = currentFolder_;

        while (folder.startsWith(".."))
        {
            if(currentFolder.length() <= 1)
            {
                throw new NoSuchFileException();
            }

            index = currentFolder.lastIndexOf('/', currentFolder.length() - 2);

            currentFolder = currentFolder.substring(0, index + 1);

            if(folder.length() > 3)
            {
                folder = folder.substring(3);
            }
            else
            {
                folder = "";
            }
        }

        if(!folder.startsWith("/"))
        {
            folder = currentFolder + folder;
        }

        if(fileSystem_.isFolder(folder))
        {
            currentFolder_ = folder;
        }
        else
        {
            throw new NoSuchFileException();
        }

        if(!currentFolder_.endsWith("/"))
        {
            currentFolder_ += "/";
        }
    }

    public void createFile(String name, Object content)
            throws FileExistsException, InvalidNameException,
            NoSuchFileException, InvalidContentException
    {
        fileSystem_.createFile(makeAbsolute(name), content);
    }

    public void createFolder(String name) throws FileExistsException,
            InvalidNameException
    {
        fileSystem_.createFolder(makeAbsolute(name));
    }

    public String getCurrentFolder()
    {
        String currentFolder = currentFolder_;

        if(currentFolder.length() > 1)
        {
            currentFolder = currentFolder_.substring(0,
                    currentFolder_.length() - 1);
        }

        return currentFolder;
    }

    public Object getFile(String name) throws NoSuchFileException,
            InvalidNameException
    {
        return fileSystem_.getFile(makeAbsolute(name));

    }

    public void moveFile(String source, String destination)
            throws NoSuchFileException, FileExistsException,
            InvalidNameException
    {
        fileSystem_.moveFile(makeAbsolute(source), makeAbsolute(destination));
    }

    public void removeFile(String name) throws NoSuchFileException,
            InvalidNameException
    {
        fileSystem_.removeFile(makeAbsolute(name));
    }

    public void removeFolder(String name) throws NoSuchFileException,
            InvalidNameException
    {
        fileSystem_.removeFolder(makeAbsolute(name));
    }

}
