package viptool.impl.job;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;

import viptool.job.IProgressMonitor;

/**
 * A progress monitor is used to indiciate the progress of a job. This will be
 * visualised with a progress bar. A job calls methods from the IProgressMonitor
 * interface to indicate a new state.
 */
public class ProgressMonitor extends JPanel implements IProgressMonitor
  {
    private boolean      isStarted_        = false;

    // number of steps of the job
    private int          steps_;

    // position of the job
    private int          counter_          = 0;

    private JLabel       descriptionLabel_ = null;

    private JProgressBar progressBar_      = null;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructor
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Create a ProgressMonitor object, that visulise the progress of a job.
     * This monitor can also show descriptions of single steps.
     */
    public ProgressMonitor()
      {
        super();
        initialize();
      }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Intitialization
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * This method initializes the ProgressMonitor. A progress bar and a label
     * for descriptions are created. For those components constraints are
     * specified.
     */
    private void initialize()
      {
        // constraints for the description label
        GridBagConstraints labelConstraints = new GridBagConstraints();
        labelConstraints.gridx = 0;
        labelConstraints.gridy = 0;
        labelConstraints.weightx = 1.0;
        labelConstraints.fill = GridBagConstraints.HORIZONTAL;
        labelConstraints.insets = new Insets(0, 0, 2, 0);

        // constraints for the progress bar
        GridBagConstraints progressBarConstraints = new GridBagConstraints();
        progressBarConstraints.gridx = 0;
        progressBarConstraints.gridy = 1;
        progressBarConstraints.fill = GridBagConstraints.HORIZONTAL;
        progressBarConstraints.weightx = 1.0;

        descriptionLabel_ = new JLabel();
        progressBar_ = new JProgressBar();
        setLayout(new GridBagLayout());

        add(descriptionLabel_, labelConstraints);
        add(progressBar_, progressBarConstraints);
      }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IProgressMonitor
    // interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.job.IProgressMonitor */
    public void start(int steps)
      {
        if (!isStarted_)
          {
            isStarted_ = true;
            steps_ = steps;
            progressBar_.setMaximum(steps);
          }
      }

    /* @see viptool.job.IProgressMonitor */
    public void step()
      {
        if (isStarted_)
          {
            counter_ = counter_ + 1;
            progressBar_.setValue(counter_);
          }
      }

    /* @see viptool.job.IProgressMonitor */
    public void setPosition(int position)
      {
        if (isStarted_)
          {
            if (position >= 0 && position <= steps_)
              {
                counter_ = position;
                progressBar_.setValue(counter_);
              }
          }
      }

    /* @see viptool.job.IProgressMonitor */
    public void step(String description)
      {
        if (isStarted_)
          {
            counter_ = counter_ + 1;
            descriptionLabel_.setText(description);
            progressBar_.setValue(counter_);
          }
      }

    /* @see viptool.job.IProgressMonitor */
    public void setDescription(String description)
      {
        if (isStarted_)
          {
            descriptionLabel_.setText(description);
          }
      }

    /* @see viptool.job.IProgressMonitor */
    public void end()
      {
        if (isStarted_)
          {
            progressBar_.setValue(steps_);
            isStarted_ = false;
          }
      }
  }
