package viptool.impl.job.gui;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.LineBorder;

import viptool.impl.base.Workbench;
import viptool.impl.job.ProgressMonitor;
import viptool.job.event.JobEvent;
import viptool.util.GeneralUtils;

public class JobPanel extends JPanel
{
    // saving the job whose progress is displayed in this panel
    private JobEvent jobEvent_;

    private class CancelButtonListener implements ActionListener
    {
        public void actionPerformed(ActionEvent event)
        {
            // create a dialog that shows the user, that the cancel is in
            // progress.

            // The dialog variable must be final, because it is used
            // in another thread
            final JDialog dialog;
            JLabel label;

            // create dialog and label
            dialog = new JDialog(Workbench.getInstance().getMainFrame(),
                    "Cancel Job", false);
            label = new JLabel("Try to cancel '" + jobEvent_.getJob().getName()
                    + "'. Please wait...");

            // layput the dialog
            dialog.setLayout(new FlowLayout());
            dialog.add(label);
            dialog.pack();
            GeneralUtils.centerComponent(dialog, Workbench.getInstance()
                    .getMainFrame());

            // create the cancel thread
            Thread cancel = new Thread()
            {
                public void run()
                {
                    Workbench.getInstance().getJobScheduler().cancelJob(
                            jobEvent_.getJob());

                    // if the job was cancelled, we can close the dialog
                    dialog.setVisible(false);
                };

            };

            // show the dialog and start the thread, disbable the button to
            // avoid a second click
            ((JButton)event.getSource()).setEnabled(false);
            dialog.setVisible(true);
            cancel.start();
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    public JobPanel(JobEvent event)
    {
        GridBagLayout gridBagLayout;
        JLabel jobNameLabel;
        Font labelFont;
        ProgressMonitor progressMonitor;
        JButton cancelButton;

        jobEvent_ = event;

        gridBagLayout = new GridBagLayout();
        setLayout(gridBagLayout);
        setBorder(LineBorder.createGrayLineBorder());
        setBackground(Color.WHITE);

        // setting the constraints of the label for the right displaying in the
        // GridBagLayout
        GridBagConstraints titleLabelConstraints = new GridBagConstraints();
        // setting the start cell of the label
        titleLabelConstraints.gridx = 0;
        titleLabelConstraints.gridy = 0;
        // setting how much cells in a row should be used by the label
        titleLabelConstraints.gridwidth = 2;
        // the label should fill the JobPanel horizontally
        titleLabelConstraints.fill = GridBagConstraints.HORIZONTAL;
        // the label should not touch the neighbour components
        titleLabelConstraints.insets = new Insets(5, 5, 0, 5);

        jobNameLabel = new JLabel(jobEvent_.getJob().getName());
        labelFont = jobNameLabel.getFont();
        labelFont.deriveFont(labelFont.getSize2D() + 8.0F);
        jobNameLabel.setFont(labelFont);
        add(jobNameLabel, titleLabelConstraints);

        // setting the constraints of the label for the right displaying in the
        // GridBagLayout
        GridBagConstraints monitorConstraints = new GridBagConstraints();
        // setting the start cell of the label
        monitorConstraints.gridx = 0;
        monitorConstraints.gridy = 1;
        // the label should fill the JobPanel horizontally
        monitorConstraints.fill = GridBagConstraints.HORIZONTAL;
        monitorConstraints.weightx = 1.0;
        // the monitor should not touch the neighbour components
        monitorConstraints.insets = new Insets(5, 5, 5, 0);

        progressMonitor = (ProgressMonitor) event.getProgressMonitor();
        if(progressMonitor != null)
        {
            progressMonitor.setBackground(getBackground());
            add(progressMonitor, monitorConstraints);
        }

        // setting the constraints of the label for the right displaying in the
        // GridBagLayout
        GridBagConstraints cancelButtonConstraints = new GridBagConstraints();
        // setting the start cell of the label
        cancelButtonConstraints.gridx = 1;
        cancelButtonConstraints.gridy = 1;
        // the label should fill the JobPanel horizontally
        cancelButtonConstraints.fill = GridBagConstraints.HORIZONTAL;
        // the label should be aligned at the bottom of the cell
        cancelButtonConstraints.anchor = GridBagConstraints.SOUTH;
        // the button should not touch the neighbour components
        cancelButtonConstraints.insets = new Insets(0, 5, 5, 5);

        cancelButton = new JButton("Cancel Job");
        cancelButton.addActionListener(new CancelButtonListener());
        add(cancelButton, cancelButtonConstraints);
    }
}
