package viptool.impl.job.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.HashMap;
import java.util.Iterator;

import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;

import viptool.base.gui.INonCloseableChildWindow;
import viptool.base.gui.INonFocusableChildWindow;
import viptool.base.gui.IView;
import viptool.impl.job.JobScheduler;
import viptool.job.event.IJobSchedulerListener;
import viptool.job.event.JobEvent;

public class JobWindow extends JScrollPane implements IView, INonFocusableChildWindow,  INonCloseableChildWindow
{
    // the one and only instance for the singleton pattern
    private static JobWindow   instance_;
    // a map that maps the JobPanels on its JobEvents
    // key: viptool.job.event.JobEvent
    // value: viptool.impl.job.gui.JobPanel
    private HashMap            jobPanels_;
    // a map that maps the constraints on its panels
    // key: viptool.impl.job.gui.JobPanel
    // value: java.awt.GridBagConstraints
    private HashMap            constraints_;

    private JPanel             container_;

    private InfoPanel          infoPanel_;

    private GridBagConstraints infoPanelConstraints_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // inner classes
    // <><><><><><><><><><><><><><><><><><><><><>
    private class InfoPanel extends JPanel implements IJobSchedulerListener
    {
        private JLabel waitingBackgroundJobsLabel_;

        private JLabel waitingForegroundJobsLabel_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        public InfoPanel()
        {
            super();
            GridBagLayout gridBagLayout;

            gridBagLayout = new GridBagLayout();
            setLayout(gridBagLayout);
            addLabelForWaitingBackgroundJobs();
            addLabelForWaitingForegroundJobs();

            JobScheduler.getInstance().addJobSchedulerListener(this);
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // supporting methods
        // <><><><><><><><><><><><><><><><><><><><><>
        private void addLabelForWaitingBackgroundJobs()
        {
            GridBagConstraints labelConstraints;

            waitingBackgroundJobsLabel_ = new JLabel();
            updateWaitingBackgroundJobsLabel();
            labelConstraints = new GridBagConstraints();
            labelConstraints.gridx = 0;
            labelConstraints.gridy = 0;
            labelConstraints.weightx = 1.0;
            labelConstraints.weighty = 1.0;
            labelConstraints.anchor = GridBagConstraints.SOUTHWEST;
            labelConstraints.insets = new Insets(5, 5, 5, 5);

            add(waitingBackgroundJobsLabel_, labelConstraints);
        }

        private void updateWaitingBackgroundJobsLabel()
        {
            int number = JobScheduler.getInstance().getNumberOfWaitingBackgroundJobs();
            waitingBackgroundJobsLabel_.setText("Number of waiting background jobs: "
                    + number);
        }

        private void addLabelForWaitingForegroundJobs()
        {
            GridBagConstraints labelConstraints;

            waitingForegroundJobsLabel_ = new JLabel();
            updateWaitingForegroundJobsLabel();
            labelConstraints = new GridBagConstraints();
            labelConstraints.gridx = 0;
            labelConstraints.gridy = 1;
            labelConstraints.weightx = 1.0;
            // labelConstraints.weighty = 0.2;
            labelConstraints.anchor = GridBagConstraints.SOUTHWEST;
            labelConstraints.insets = new Insets(5, 5, 5, 5);

            add(waitingForegroundJobsLabel_, labelConstraints);
        }

        private void updateWaitingForegroundJobsLabel()
        {
            int number = JobScheduler.getInstance().getNumberOfWaitingForegroundJobs();
            waitingForegroundJobsLabel_.setText("Number of waiting foreground jobs: "
                    + number);
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementation of the IJobSchedulerListener
        // interface
        // <><><><><><><><><><><><><><><><><><><><><>
        /* @see viptool.job.event.IJobSchedulerListener */
        public void jobCancelled(JobEvent jobEvent)
        {
            updateWaitingBackgroundJobsLabel();
            updateWaitingForegroundJobsLabel();
        }

        /* @see viptool.job.event.IJobSchedulerListener */
        public void jobFinished(JobEvent jobEvent)
        {
            updateWaitingBackgroundJobsLabel();
            updateWaitingForegroundJobsLabel();
        }

        /* @see viptool.job.event.IJobSchedulerListener */
        public void jobStarted(JobEvent jobEvent)
        {
            updateWaitingBackgroundJobsLabel();
            updateWaitingForegroundJobsLabel();
        }

        /* @see viptool.job.event.IJobSchedulerListener */
        public void newJobWaiting(int mode)
        {
            updateWaitingBackgroundJobsLabel();
            updateWaitingForegroundJobsLabel();
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    public JobWindow()
    {
        GridBagLayout gridBagLayout;

        container_ = new JPanel();
        gridBagLayout = new GridBagLayout();
        container_.setLayout(gridBagLayout);
        container_.setBackground(Color.WHITE);

        infoPanel_ = new InfoPanel();
        infoPanel_.setBackground(container_.getBackground());
        // setting the constraints of the panel for the right displaying in the
        // GridBagLayout
        infoPanelConstraints_ = new GridBagConstraints();
        // setting the start cell of the panel
        infoPanelConstraints_.gridx = 0;
        infoPanelConstraints_.gridy = 0;
        // the panel should fill the JobWindow horizontally
        infoPanelConstraints_.fill = GridBagConstraints.BOTH;
        // the panel should fill all free space
        infoPanelConstraints_.weighty = 1.0;
        infoPanelConstraints_.weightx = 1.0;

        container_.add(infoPanel_, infoPanelConstraints_);

        setViewportView(container_);

        jobPanels_ = new HashMap();
        constraints_ = new HashMap();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of singleton pattern
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * If no JobWindow has been created yet, create one. Otherwise return the
     * existing JobWindow.
     */
    public synchronized static JobWindow getInstance()
    {
        if (instance_ == null)
        {
            instance_ = new JobWindow();
        }

        return instance_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // managing JobPanels
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * this methods add a JobPanel to the JobWindow. It will be added to the
     * vertical end of the before added JobPanels. Also it will be stored in a
     * HashMap referred by the given JobEvent
     * 
     * @param jobEvent
     *            the JobEvent that contains the job whose progress will be
     *            displayed in the panel to add (also the key for the new panel
     *            in a HashMap)
     */
    public void addJobPanel(JobEvent jobEvent)
    {
        JobPanel jobPanel;

        container_.remove(infoPanel_);

        // setting the constraints of the panel for the right displaying in the
        // GridBagLayout
        GridBagConstraints panelConstraints = new GridBagConstraints();
        // setting the start cell of the panel
        panelConstraints.gridx = 0;
        panelConstraints.gridy = container_.getComponentCount();
        // the panel should fill the JobWindow horizontally
        panelConstraints.fill = GridBagConstraints.HORIZONTAL;
        panelConstraints.insets = new Insets(1, 0, 0, 0);
        panelConstraints.weightx = 1.0;

        // creating and adding the panel to the window and the map
        jobPanel = new JobPanel(jobEvent);
        container_.add(jobPanel, panelConstraints);
        jobPanels_.put(jobEvent.getJob(), jobPanel);
        constraints_.put(jobPanel, panelConstraints);

        infoPanelConstraints_.gridy = container_.getComponentCount();
        container_.add(infoPanel_, infoPanelConstraints_);

        container_.validate();
    }

    /**
     * method will manage the remove of the JobPanel which belongs to the given
     * JobEvent. Therefor it has to be removed from the JobWindow as well as
     * from the HashMaps. Because of the problem that the gridy constraint is
     * not changed internally when a component is removed out of the GridBag-
     * Layout every Component that has a higher gridy constraint has to be
     * removed its gridy value decremented and added again. Only to change the
     * gridy value without removing and adding won't bring the requested effect!
     * 
     * @param jobEvent
     *            the JobEvent whose JobPanel should be removed
     */
    public synchronized void removeJobPanel(JobEvent jobEvent)
    {
        JobPanel jobPanel;
        Iterator it;
        GridBagConstraints tempConstraints;
        int removedGridY;

        jobPanel = (JobPanel) jobPanels_.get(jobEvent.getJob());
        if (jobPanel == null)
        {
            // panel already removed
            return;
        }

        tempConstraints = (GridBagConstraints) constraints_.get(jobPanel);

        container_.remove(jobPanel);
        jobPanels_.remove(jobEvent.getJob());
        constraints_.remove(jobPanel);

        removedGridY = tempConstraints.gridy;
        it = constraints_.keySet().iterator();
        while (it.hasNext())
        {
            jobPanel = (JobPanel) it.next();
            tempConstraints = (GridBagConstraints) constraints_.get(jobPanel);
            if (tempConstraints.gridy > removedGridY)
            {
                container_.remove(jobPanel);
                tempConstraints.gridy = tempConstraints.gridy - 1;
                container_.add(jobPanel, tempConstraints);
            }
        }

        container_.invalidate();
        container_.validate();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the IView interface
    // <><><><><><><><><><><><><><><><><><><><><>
    public boolean canClose(boolean force)
    {
        return true;
    }

    public Component getComponent()
    {
        // TODO Auto-generated method stub
        return this;
    }

    public Icon getIcon()
    {
        // TODO Auto-generated method stub
        return null;
    }

    public JMenuBar getMenuBar()
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String getTitle()
    {
        // TODO Auto-generated method stub
        return "Job Monitor";
    }

    public JToolBar getToolBar()
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String getToolTipText()
    {
        // TODO Auto-generated method stub
        return null;
    }

    public void windowActivated()
    {

    }

}
