package viptool.impl.logger;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.util.Vector;

import viptool.logger.AbstractLogger;
import viptool.logger.LogObject;
import viptool.util.resources.ResourceConstants;

/**
 * Manage all messages like errors or warnings. So we can store them in an
 * ArrayList or/and in a file.
 */
public class Logger extends AbstractLogger
{
    private static int       minLevel_ = 0;

    // the only instance
    private static Logger    instance_ = null;

    private FileOutputStream output_   = null;

    // all LogObjects
    private Vector           logs_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Creates a new ConsoleLogger. There can only exist one logger. The Logger
     * stores all messages, that occures.
     */
    private Logger()
    {
        super();
        logs_ = new Vector();
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of singleton pattern
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * If no Logger has been created, yet, create one. Otherwise return the
     * created Logger.
     */
    public synchronized static Logger getInstance()
    {
        if (instance_ == null)
        {
            instance_ = new Logger();
        }

        return instance_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Supporting Methods
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * This method stores all messages with level and date as a LogObject in an
     * ArrayList. If you FileOutputStream is not null, additionally it is saved
     * in a file.
     */
    public void log(int level, String msg)
    {
        if (level < minLevel_ || msg == null)
        {
            return;
        }

        LogObject logObject = new LogObject(level, msg);

        logs_.add(logObject);

        // save the LogObject in you file, if you have set one.
        if (output_ != null)
        {

            StringBuffer buffer = new StringBuffer(
                    logObject.toString(ResourceConstants.DEFAULT_LOG_FORMAT));

            PrintWriter output = new PrintWriter(new BufferedOutputStream(
                    output_));

            output.println(buffer.toString());
            output.flush();
        }

        fireLogObjectAddedEvent(logObject);
    }

    /**
     * Set a file for your logger. If there have been one,yet, append the new
     * information to the existet output stream.
     * 
     * @param file
     *            the file to be opened for writing
     */
    public void setFile(File file)
    {

        if (output_ == null)
        {

            try
            {
                output_ = new FileOutputStream(file);
            }
            catch (FileNotFoundException e)
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
        else
        {

            try
            {
                output_ = new FileOutputStream(file, true);
            }
            catch (FileNotFoundException e)
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
    }

    /*
     * @see viptool.logger.ILogger
     */
    public void setLogLevel(int level)
    {
        minLevel_ = level;
    }

    /**
     * Get the ArrayList with all LogObjects:
     * 
     * @return an ArrayList with LogObjects
     */
    public Vector getLogs()
    {

        return logs_;
    }
}
