package viptool.impl.logger;

import java.util.Iterator;
import java.util.Vector;

import viptool.base.gui.INonCloseableChildWindow;
import viptool.base.gui.INonFocusableChildWindow;
import viptool.impl.console.Console;
import viptool.logger.ILogger;
import viptool.logger.LogObject;
import viptool.logger.event.ILoggerChangedListener;
import viptool.util.resources.ResourceConstants;

/**
 * LoggerConsole extends the class Console. With this LoggerConsole it is
 * possible to display all messages, that will be stored in a logger. Each time
 * the logger will get a new message, the LoggerConsole will display it.
 */
public class LoggerConsole extends Console implements ILoggerChangedListener,
        INonCloseableChildWindow, INonFocusableChildWindow

{

    // the default format of the messge, that will be displayed
    private String  format_ = ResourceConstants.DEFAULT_LOG_FORMAT;

    // the logger, that stores the messages to display
    private ILogger logger_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructor
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * Create a LoggerConsole object. It is a non-editable textpane with the
     * content type text/html. Simultaneously you get the logger, that gets all
     * messages you have to display. So the LoggerChangedListener will be added.
     */
    public LoggerConsole()
    {

        super("log message");

        // get the logger
        logger_ = Logger.getInstance();

        logger_.addLoggerChangedListener(this);

        // show the text, that is allready stored in the buffer
        showBuffer();
    }

    public LoggerConsole(String name, ILogger logger)
    {
        super(name);

        // get the logger
        logger_ = logger;

        logger_.addLoggerChangedListener(this);

        // show the text, that is allready stored in the buffer
        showBuffer();
    }

    /**
     * With this method you can change the format of the message. With the
     * format you can choose which information in which order will be displayed.
     * To see how an input for changing the format looks like, watch the comment
     * of the method toString in class LogObject.
     * 
     * @param format
     *            the text, that should include the LogObject. This text must be
     *            in the correct format to get a output, that makes sense e.g.:
     *            "At {date:EEE, MMM d, ''yy} you got the error {message} with
     *            the level {level}" output:"At Wed, Jul 4, '06 you got the
     *            error NullPointerExeption with the level ERROR."
     */
    public void changeFormat(String format)
    {

        format_ = format;
    }

    /**
     * This method will show all the LogObjects as a String, that are allready
     * stored. This will be necessary when you open the console the first time.
     */
    private void showBuffer()
    {

        // all the LogObjects
        Vector logs = logger_.getLogs();

        Iterator it = logs.iterator();

        while (it.hasNext())
        {

            // show it in the console
            logObjectAdded((LogObject) it.next());
        }
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Overriden methods from Console
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * The LoggerConsole can be closed all the time. When you close this window,
     * the LoggerChangedListener will be closed.
     */
    public boolean canClose(boolean force)
    {

        // sign of the listener, so that changes in the logger class won�t be
        // recognized
        logger_.removeLoggerChangedListener(this);
        return true;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementing the ILoggerChangeListener
    // interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /**
     * All the time a LogObject will be added to the logger, this method is
     * called. The message will be displayed at the console in the given format.
     * 
     * @param logObject
     *            the LogObject, that was added to the logger
     */
    public void logObjectAdded(LogObject logObject)
    {

        int level = logObject.getLevel();

        String text = logObject.toString(format_);

        // display the message in your console
        appendText(text, level);
    }

    public void logObjectRemoved(LogObject logObject)
    {
        // TODO Auto-generated method stub

    }
}
