package viptool.impl.project;

import java.awt.Menu;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import viptool.project.IWorkspace;

public class RecentWorkspacesManager 
{
	
	private static RecentWorkspacesManager recentManager_ = null;
	ShutdownSave save_;
	private final String resourcePath_   = "viptool/impl/util/resources/recent.xml";
	
	class ShutdownSave extends Thread
	{

		private String resourcePath_   = "viptool/impl/util/resources/recent.xml";
		private Vector workspaces_;
		
		
		public ShutdownSave()
		{
			workspaces_ = new Vector();
		}
				
		public void run() {
			saveRecent();
		}
		
		public void saveRecent()
		{
			Document doc = null;
			try 
			{
				// initialize DocumentBuilder
				DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
				// create Document
				doc = docBuilder.newDocument();
			} 
			catch (ParserConfigurationException e) 
			{
				e.printStackTrace();	
			}
			
			if(null != doc) 
			{
				// create root
				Element root = doc.createElement("recent");
				// add root to document
				doc.appendChild(root);
				
				//add workspaces
				Enumeration enumer = workspaces_.elements();
				while(enumer.hasMoreElements())
				{
					Element element = doc.createElement("workspace");
					File file = (File)enumer.nextElement();
					element.setAttribute("path", file.getAbsolutePath());
					root.appendChild(element);
				}	      
			}
			
			//initialize TransformerFactory
			TransformerFactory tf = TransformerFactory.newInstance();
			Transformer t = null;
			try 
			{
				// get transformer
				t = tf.newTransformer();
				// set use whitespaces
				t.setOutputProperty(OutputKeys.INDENT, "yes");
				// set output-type xml
				t.setOutputProperty(OutputKeys.METHOD, "xml");
				// Content-Type
				t.setOutputProperty(OutputKeys.MEDIA_TYPE, "text/xml");
				//set encoding
				t.setOutputProperty(OutputKeys.ENCODING, "iso-8859-1");
				// create FileWriter
				FileWriter fw = new FileWriter(resourcePath_);
				// write to file
				t.transform(new DOMSource(doc), new StreamResult(fw));
			} 
			catch (TransformerConfigurationException e) 
			{
				e.printStackTrace();
			} 
			catch (TransformerException e) 
			{
				e.printStackTrace();
			} 
			catch (IOException e) 
			{
				e.printStackTrace();
			}    
		}
		
		public void addWorkspace(File file)
		{
			boolean addFlag = true;
			Enumeration enumer = workspaces_.elements();
			while(enumer.hasMoreElements())
			{
				File enfile = (File)enumer.nextElement();
				if(enfile.getAbsolutePath().equals(file.getAbsolutePath()))
				{
					addFlag = false;
				}
			}
			
			if(addFlag)
			{
				if(workspaces_.size() > 3)
				{
					workspaces_.remove(0);
				}
				workspaces_.add(file);
			}
		}
		
		public File[] getRecent()
		{
			File[] files = new File[workspaces_.size()];
			for(int i=0; i<workspaces_.size(); i++)
			{
				files[i] = (File)workspaces_.get(i);
			}
			return files;
		}
		
	}
	
	
	private RecentWorkspacesManager()
	{
		 save_ = new ShutdownSave();
	     Runtime rt = Runtime.getRuntime();
	     rt.addShutdownHook(save_);
	     
	     loadRecent();
	}
	
	public static RecentWorkspacesManager getInstance()
	{
		if(recentManager_ == null)
		{
			recentManager_ = new RecentWorkspacesManager();
		}
		return recentManager_;
	}
	
	private void loadRecent()
	{
		File recent = new File(resourcePath_);
		if(recent.exists() && recent.length()>0)
		{
			try
			{
				//create Parser
				DocumentBuilder docBuilder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
				Document doc = docBuilder.parse(recent);
				//analyze document
				analyze(doc);
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
		else
		{
			System.out.println("loadRecent: There is no recent-file!");
		}
	}

	private void analyze(Node node) 
	{
		if(node != null) 
		{
			if(node.getNodeName().equals("workspace"))
			{
				if(node.getAttributes().getLength() > 0)
				{
					String path = node.getAttributes().item(0).getNodeValue();
					File file = new File(path);
					if(file != null && file.exists())
					{
						save_.addWorkspace(file);
					}
				}
			}
			// rekursive analyze whole tree
			if(node.hasChildNodes()) 
			{
				//analyze all childnodes
				for(int i=0,items = node.getChildNodes().getLength(); i<items;i++) {
					// analyze node
					analyze(node.getChildNodes().item(i));
				}
			}
		}
	}
	
	public void addToRecentList(IWorkspace manager)
	{
        /*
		File file = manager.getAbstractFile().getFile();
		if(file != null)
		{
			save_.addWorkspace(file);
		}
		Menu filemenu = (Menu)MenuManager.getInstance().getItem(
				DefaultMenuBar.class.getName()+"/"+Locale.MENUNAME_FILE);
		Component recentmenu = MenuManager.getInstance().getItem(
				DefaultMenuBar.class.getName()+"/"+Locale.MENUNAME_FILE
				+"/"+Locale.MENUNAME_RECENT);
		Component[] entries = filemenu.getMenuComponents();
		int i=0;
		for(; i<entries.length; i++)
		{
			if(entries[i]==recentmenu)
			{
				break;
			}
		}
		filemenu.remove(recentmenu);
		filemenu.add(getRecentWorkspaces(), i);
        */
	}
	
	public Menu getRecentWorkspaces()
	{
	    /*	
    		Menu menu = new Menu(Locale.MENUNAME_RECENT, Locale.MNEMONIC_KEY_RECENT_MENU);
    		File[] files = save_.getRecent();
    		for(int i=0; i<files.length; i++)
        	{
        		//RecentWorkspaceAction action = new RecentWorkspaceAction(files[i]);
    			//menu.add(new JMenuItem(action));
        	}
    		return menu;
        */
        
        return null;
	}
}
