package viptool.impl.project.action;

import java.awt.Cursor;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragSource;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.io.File;
import java.util.Iterator;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.tree.TreePath;

import viptool.impl.project.ProjectGuiLinker;
import viptool.project.IProjectTreeNode;

/**
 * The DropTargetListener for the tree
 */
public class ProjectDropTargetListener implements DropTargetListener{

	//reference to tree
	private JTree tree_;
	//defaultcursor
	private Cursor cursor_;
	
	//<><><><><><><><><><><><><><><><><><><><>
	// Constructors
	//<><><><><><><><><><><><><><><><><><><><>
	/**
	 * creates a new DropTargetListener for the given tree
	 */	
	public ProjectDropTargetListener(JTree tree)
	{
		tree_ = tree;
		cursor_ = tree.getCursor();
	}
	
	//<><><><><><><><><><><><><><><><><><><><>
	// Constructors
	//<><><><><><><><><><><><><><><><><><><><>
	/**
	 * a new Drag entered the tree-area...
	 */
	public void dragEnter(DropTargetDragEvent dtde) {
		setTreeCursor(dtde);
	}

	/**
	 * the Drag exited the tree-area...
	 */
	public void dragExit(DropTargetEvent dte) {
		tree_.setCursor(cursor_);
	}

	/**
	 * the Drag is moved over the tree-area...
	 */
	public void dragOver(DropTargetDragEvent dtde) {
		//do nothing
	}

	/**
	 * The drag was droped over the tree-area, so process dragdata
	 */
	public void drop(DropTargetDropEvent dtde) {
		
		//set default-cursor
		tree_.setCursor(cursor_);
		
		ProjectTreeTransferable.setInternal(true);
		
		TreePath dropPath = tree_.getClosestPathForLocation(dtde.getLocation().x, dtde.getLocation().y);
		
		IProjectTreeNode node = null;
		
		//get folder
		if(dropPath != null)
		{
			node = (IProjectTreeNode)dropPath.getLastPathComponent();
			if(!node.isFolder() && !node.isProject() && !node.isWorkspace())
			{
				node = (IProjectTreeNode)node.getParent();
			}
		}
		
		if(node != null)
		{
			//import
			//get transferable
			Transferable transferable = dtde.getTransferable();
			
			boolean move = false;
			if(dtde.getDropAction() == DnDConstants.ACTION_MOVE)
			{
				dtde.acceptDrop(DnDConstants.ACTION_MOVE);
				move = true;
			}
			else if(dtde.getDropAction() == DnDConstants.ACTION_COPY)
			{
				dtde.acceptDrop(DnDConstants.ACTION_COPY);
			}
			else
			{
				dtde.rejectDrop();
				return;
			}
			
			//if it is a file
			if(transferable.isDataFlavorSupported(DataFlavor.javaFileListFlavor))
			{
				try
				{
					//get files
					List files = (List)transferable.getTransferData(DataFlavor.javaFileListFlavor);
					
					if(files != null && !files.isEmpty() && files.size()>0)
					{
						boolean internal = false;
						boolean error = false;
						if(move)
						{
							File f = (File)files.get(0);
							if(f.getAbsolutePath().startsWith(ProjectGuiLinker.getInstance().getProjectPath(node)))
							{
								internal = true;
							}
						}
						
						//import files
						Iterator iter = files.iterator();
						while(iter.hasNext())
						{
							//get file
							File file = (File)iter.next();
							//add element
							error = !ProjectGuiLinker.getInstance().importFile(file, node);
						}
						
						if(error || (move && !internal))
						{
							dtde.dropComplete(false);
						}
						else
						{
							dtde.dropComplete(true);
						}
					}
				}
				catch(Exception e)
				{
					dtde.dropComplete(false);
					System.err.println("File import error:");
					e.printStackTrace();
					JOptionPane.showMessageDialog(
							tree_, 
							"Files import error: \n" + e.toString(),
							"Error:",
							JOptionPane.ERROR_MESSAGE);
				}
			}	
		}
		else
		{
			dtde.dropComplete(false);
			System.err.println("File import error: Project not found");
			JOptionPane.showMessageDialog(
					tree_, 
					"Files import error: \nProject not found",
					"Error:",
					JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * The action for the drag was changed...
	 */
	public void dropActionChanged(DropTargetDragEvent dtde) {
		setTreeCursor(dtde);
	}
	
	/**
	 * set the cursor for the drag
	 */
	private void setTreeCursor(DropTargetDragEvent dtde)
	{
		tree_.setCursor(DragSource.DefaultCopyDrop);
	}
}
