package viptool.impl.project.action;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.HashSet;

import javax.swing.JPopupMenu;
import javax.swing.tree.TreePath;

import viptool.base.editor.IEditorProvider;
import viptool.base.gui.menu.IExtendedContextMenu;
import viptool.base.io.IFileInterpreterProvider;
import viptool.impl.base.editor.EditorManager;
import viptool.impl.base.gui.menu.ContextMenuManager;
import viptool.impl.base.gui.selection.AbstractFileSelection;
import viptool.impl.base.io.FileInterpreterManager;
import viptool.impl.project.ProjectGuiLinker;
import viptool.impl.project.gui.ProjectTreePane;
import viptool.project.AbstractFile;
import viptool.project.IAbstractFile;
import viptool.project.IAbstractFolder;
import viptool.project.IProjectTreeNode;

/**
 * The MouseListener for the tree. It is needed to open elements by doubleclick
 * and shows the popupmenu.
 */
public class ProjectTreeMouseListener implements MouseListener
{

    // the popupmenu which will be shown
    private JPopupMenu popup_;

    // <><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><>
    /**
     * creats a new DnDTreeMouseListener
     */
    public ProjectTreeMouseListener()
    {
        popup_ = new JPopupMenu();
    }

    // <><><><><><><><><><><><><><><><>
    // methodes
    // <><><><><><><><><><><><><><><><>
    /**
     * open with doubleclick, show popupmenu
     */
    public void mouseClicked(MouseEvent e)
    {
        IAbstractFile files[];
        IFileInterpreterProvider[] fileInterpreterProviders;
        ArrayList<IEditorProvider> editorProviders;
        ArrayList<IEditorProvider> matchingProviders;
        HashSet<IEditorProvider> providersToRemove;
        IExtendedContextMenu contextMenu;

        if (e.getClickCount() > 1)
        {
            TreePath treepath = ProjectTreePane.getInstance().getTree().getClosestPathForLocation(
                    e.getX(), e.getY());
            if (treepath != null && treepath.getPath().length >= 2)
            {
                IProjectTreeNode node = (IProjectTreeNode) treepath.getLastPathComponent();
                ProjectGuiLinker.getInstance().run(node);
            }
        }
        else if (e.isMetaDown() || e.isPopupTrigger())
        {
            // if nothing is selected, select next node
            if (ProjectTreePane.getInstance().getTree().getSelectionPath() == null)
            {
                TreePath path = ProjectTreePane.getInstance().getTree().getClosestPathForLocation(
                        e.getX(), e.getY());
                ProjectTreePane.getInstance().getTree().addSelectionPath(path);
            }

            // by direct click at element, mark element
            TreePath path = ProjectTreePane.getInstance().getTree().getPathForLocation(
                    e.getX(), e.getY());
            TreePath[] paths = ProjectTreePane.getInstance().getTree().getSelectionPaths();
            if (path != null && paths.length < 2)
            {
                ProjectTreePane.getInstance().getTree().setSelectionPath(path);
            }

            paths = ProjectTreePane.getInstance().getTree().getSelectionPaths();

            if (path != null)
            {
                files = new IAbstractFile[paths.length];

                // determine possible editors
                editorProviders = new ArrayList<IEditorProvider>();
                matchingProviders = new ArrayList<IEditorProvider>();
                providersToRemove = new HashSet<IEditorProvider>();

                for (int i = 0; i < paths.length; i++)
                {
                    files[i] = ProjectGuiLinker.getInstance().getAbstractFile(
                            (IProjectTreeNode) paths[i].getLastPathComponent());

                    fileInterpreterProviders = FileInterpreterManager.getInstance().getMatchingFileInterpreterProviders(
                            files[i]);
                    if (i == 0)
                    {
                        if (!(files[i] instanceof IAbstractFolder))
                        {
                            editorProviders.addAll(EditorManager.getInstance().getMatchingEditorProviders(
                                    fileInterpreterProviders));
                        }
                    }
                    else
                    {
                        providersToRemove.clear();
                        matchingProviders.clear();

                        if (!(files[i] instanceof IAbstractFolder))
                        {
                            matchingProviders = EditorManager.getInstance().getMatchingEditorProviders(
                                    fileInterpreterProviders);
                        }

                        for (IEditorProvider editorProvider : editorProviders)
                        {
                            if (!matchingProviders.contains(editorProvider))
                            {
                                providersToRemove.add(editorProvider);
                            }
                        }
                        editorProviders.removeAll(providersToRemove);
                    }

                }
                contextMenu = ContextMenuManager.getInstance().createContextMenu(
                        new AbstractFileSelection(files));

                // add open with entries
                for (IEditorProvider editorProvider : editorProviders)
                {
                    contextMenu.addMenuItem("action." + editorProvider.getId(),
                            "file.openwith", "", 0);
                }
                contextMenu.removeMenuItem("file.openwith.empty");

                popup_ = contextMenu.getContextMenu();
                popup_.show(e.getComponent(), e.getX(), e.getY());
            }
            else
            {
                contextMenu = ContextMenuManager.getInstance().createContextMenu(
                        new AbstractFileSelection(new AbstractFile[] {}));

                popup_ = contextMenu.getContextMenu();
                popup_.show(e.getComponent(), e.getX(), e.getY());

            }

        }
        else
        {
            popup_.setVisible(false);
        }
    }

    public void mouseEntered(MouseEvent e)
    {
        // TODO Auto-generated method stub

    }

    public void mouseExited(MouseEvent e)
    {
        // TODO Auto-generated method stub

    }

    public void mousePressed(MouseEvent e)
    {
        // TODO Auto-generated method stub

    }

    public void mouseReleased(MouseEvent e)
    {
        // TODO Auto-generated method stub

    }

}
