package viptool.impl.project.gui;

import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.event.KeyEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeSelectionModel;

import viptool.impl.base.action.file.LoadProjectAction;
import viptool.impl.project.action.CopyElementAction;
import viptool.impl.project.action.DeleteElementAction;
import viptool.impl.project.action.PasteElementAction;
import viptool.impl.project.action.ProjectDropTargetListener;
import viptool.impl.project.action.ProjectTreeMouseListener;

/**
 * This class is an extended version of the default JTree. It implements
 * the features Drag and Drop and the shortcuts ctrl x for cut, ctrl c 
 * for copy, ctrl v for paste and del for delete.
 * This is class didn't implements the ChildWindow interface, look at DnDTreePane
 * for that.
 */

public class ProjectTree extends JTree{
	
	/**
	 * creates a new DnDTree
	 * 
	 * @param model
	 * 		the model for the tree
	 */
	public ProjectTree(TreeModel model)
	{
		super(model);
		
		//set treeUI
		setUI(new ProjectTreeUI());
		//set selection mode
		getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
		//set TreeCellRenderer
		setCellRenderer(new ProjectTreeCellRenderer());
		//hide root-node
		setRootVisible(false);
		
		// disable auto expand
		setToggleClickCount(0);
		
		//add mouselistener for open-events
		addMouseListener(new ProjectTreeMouseListener());
		
		//implementation of Drag and Drop feature
		//implementation of Drop
		DropTarget dt = new DropTarget(this, new ProjectDropTargetListener(this));
		//set drop-actions
		dt.setDefaultActions(DnDConstants.ACTION_COPY_OR_MOVE);
		//set drop-target
		setDropTarget(dt);
		//implementation of Drag
		setDragEnabled(true);
		//set TransferHandler
		setTransferHandler(TreeDragAndDropSupport.getInstance());
				
		//enable shortcuts
		InputMap imap = getInputMap();
		imap.put(KeyStroke.getKeyStroke("ctrl X"), "cut");
		imap.put(KeyStroke.getKeyStroke("ctrl C"), "copy");
		imap.put(KeyStroke.getKeyStroke("ctrl V"), "paste");
		imap.put(KeyStroke.getKeyStroke("ctrl I"), "import");
		imap.put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0), "delete");
		
		ActionMap map = getActionMap();
		map.put("copy", new CopyElementAction());
		map.put("cut", new CopyElementAction());
		map.put("paste", new PasteElementAction());
		map.put("delete", new DeleteElementAction());
		map.put("import", new LoadProjectAction());
	}
	
	//this methodes are not necessary at the moment (just for testing)
	public void addMouseMotionListener(MouseMotionListener listener)
	{
			super.addMouseMotionListener(listener);
	}
	
	public void addMouseListener(MouseListener listener)
	{
		super.addMouseListener(listener);
	}
}
