package viptool.impl.project.gui;

import java.awt.Color;
import java.awt.Component;

import javax.swing.JTree;
import javax.swing.tree.DefaultTreeCellRenderer;

import viptool.impl.project.ProjectGuiLinker;
import viptool.impl.util.resources.ResourceLoader;
import viptool.project.IProjectTreeNode;
import viptool.util.resources.ResourceConstants;

/**
 * This class is the renderer for the ProjectTree. It extends
 * DefaultTreeCellRenderer and set our icons. It also marks changed elements
 * with a *.
 */
public class ProjectTreeCellRenderer extends DefaultTreeCellRenderer
{

    /**
     * This methode returns the object which is shown at tree. See
     * DefaultTreeCellRenderer for more information
     */
    public Component getTreeCellRendererComponent(JTree tree, Object value,
            boolean selected, boolean expanded, boolean leaf, int row,
            boolean hasFocus)
    {
        super.getTreeCellRendererComponent(tree, value, selected, expanded,
                leaf, row, hasFocus);

        // if element was changed, mark with a *
        if (value instanceof IProjectTreeNode
                && !ProjectGuiLinker.getInstance().isValid(
                        (IProjectTreeNode) value))
        {
            setForeground(Color.RED);
        }
        else
        {
            setForeground(Color.BLACK);
        }

        // set icons, getIcon is not used because the folder didn't knows if it
        // is expanded or not
        if (value instanceof IProjectTreeNode)
        {
            IProjectTreeNode node = (IProjectTreeNode) value;
            if (node.isProject())
            {
                if (expanded)
                {
                    setIcon(ResourceLoader.getInstance().getIcon(
                            ResourceConstants.ICON_PROJECT_OPEN));
                }
                else
                {
                    setIcon(ResourceLoader.getInstance().getIcon(
                            ResourceConstants.ICON_PROJECT));
                }
            }
            else if (node.isWorkspace())
            {
                if (expanded)
                {
                    setIcon(ResourceLoader.getInstance().getIcon(
                            ResourceConstants.ICON_FOLDER_OPEN));
                }
                else
                {
                    setIcon(ResourceLoader.getInstance().getIcon(
                            ResourceConstants.ICON_FOLDER));
                }
            }
            else if (node.isFolder())
            {
                if (expanded)
                {
                    setIcon(ResourceLoader.getInstance().getIcon(
                            ResourceConstants.ICON_FOLDER_OPEN));
                }
                else
                {
                    setIcon(ResourceLoader.getInstance().getIcon(
                            ResourceConstants.ICON_FOLDER));
                }
            }
            else if (ProjectGuiLinker.getInstance().getIconForNode(node) != null)
            {
                // set icon for IProjectElements if the have an icon
                setIcon(ProjectGuiLinker.getInstance().getIconForNode(node));
            }
            else
            {
                // else use the default file icon
                setIcon(ResourceLoader.getInstance().getIcon(
                        ResourceConstants.ICON_DEFAULT_FILE));
            }
        }
        else
        {
            // if it is an unknown object use default icon
            setIcon(ResourceLoader.getInstance().getIcon(
                    ResourceConstants.ICON_DEFAULT_FILE));
        }

        return this;
    }
}
