package viptool.impl.project.gui;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import viptool.impl.util.SortedVector;
import viptool.project.IProjectTreeModel;
import viptool.project.IProjectTreeNode;

public class ProjectTreeModel implements TreeModel, IProjectTreeModel
{

    private class TreeFolder
    {
        public SortedVector childs_;

        public TreeFolder(IProjectTreeNode node)
        {
            childs_ = new SortedVector();
            childs_.setComparator(comparator_);
        }
    }

    private Comparator       comparator_;
    private IProjectTreeNode root_;
    private HashMap          folders_;
    private Vector           listeners_;

    private boolean          show_ = false;

    public ProjectTreeModel(String workspacename)
    {
        root_ = new ProjectTreeNode(workspacename, ProjectTreeNode.WORKSPACE);
        ((ProjectTreeNode) root_).setModel(this);

        folders_ = new HashMap();
        comparator_ = new DefaultTreeComparator();
        folders_.put(root_, new TreeFolder(root_));
        listeners_ = new Vector();
    }

    public void addTreeModelListener(TreeModelListener l)
    {
        listeners_.add(l);
    }

    public Object getChild(Object parent, int index)
    {
        Object ofolder = folders_.get(parent);
        if (ofolder != null)
        {
            TreeFolder folder = (TreeFolder) ofolder;
            if (index >= 0 && index < folder.childs_.size())
            {
                Object o = folder.childs_.get(index);
                return o;
            }
        }
        return null;
    }

    public int getChildCount(Object parent)
    {
        Object ofolder = folders_.get(parent);
        if (ofolder != null)
        {
            return ((TreeFolder) ofolder).childs_.size();
        }
        return 0;
    }

    public int getIndexOfChild(Object parent, Object child)
    {
        Object ofolder = folders_.get(parent);
        if (ofolder != null)
        {
            return ((TreeFolder) ofolder).childs_.indexOf(child);
        }
        return 0;
    }

    public Object getRoot()
    {
        return root_;
    }

    public boolean isLeaf(Object node)
    {
        if (node instanceof IProjectTreeNode)
        {
            if (((IProjectTreeNode) node).getChildCount() > 0)
            {
                return false;
            }
        }
        return true;
    }

    public void removeTreeModelListener(TreeModelListener l)
    {
        listeners_.remove(l);
    }

    public void valueForPathChanged(TreePath path, Object newValue)
    {
        Object node = path.getLastPathComponent();
        if (node instanceof IProjectTreeNode
                && folders_.get(((IProjectTreeNode) node).getParent()) != null)
        {
            ((IProjectTreeNode) node).setUserObject(newValue);
            updateNode((IProjectTreeNode) node);
        }
    }

    public boolean addNode(IProjectTreeNode node)
    {
        TreeNode parent = node.getParent();
        if (parent != null)
        {
            Object oProjectParent = folders_.get(parent);
            if (oProjectParent != null)
            {
                TreeFolder projectParent = (TreeFolder) oProjectParent;
                projectParent.childs_.add(node);

                if (node instanceof IProjectTreeNode
                        && ((IProjectTreeNode) node).getAllowsChildren())
                {
                    TreeFolder newFolder = new TreeFolder(node);
                    folders_.put(node, newFolder);
                }
                insertEvent(node);

                if (!show_)
                {
                    fullStructureUpdate();
                    show_ = true;
                }

                return true;
            }
        }
        return false;
    }

    public Enumeration getChildren(IProjectTreeNode node)
    {
        Object oFolder = folders_.get(node);
        if (oFolder != null)
        {
            TreeFolder folder = (TreeFolder) oFolder;
            return folder.childs_.elements();
        }
        return null;
    }

    public boolean removeNode(IProjectTreeNode parent, int index)
    {
        Object oFolder = folders_.get(parent);
        if (oFolder != null)
        {
            Object node = ((TreeFolder) oFolder).childs_.get(index);
            if (node instanceof IProjectTreeNode)
            {
                IProjectTreeNode projectnode = (IProjectTreeNode) node;
                deleteEvent(projectnode);
                ((TreeFolder) oFolder).childs_.remove(index);
                return true;
            }
        }
        return false;
    }

    public boolean removeNode(IProjectTreeNode node)
    {
        Object oFolder = folders_.get(node.getParent());
        if (oFolder != null)
        {
            deleteEvent(node);
            ((TreeFolder) oFolder).childs_.remove(node);
            return true;
        }
        return false;
    }

    public boolean updateNode(IProjectTreeNode node)
    {
        if (folders_.get(node.getParent()) != null)
        {
            updateEvent(node);
            return true;
        }
        return false;
    }

    private TreeModelEvent generateEvent(IProjectTreeNode node)
    {
        if ((node.getParent() != null && node.getParent() instanceof IProjectTreeNode)
                || node == root_)
        {
            Object[] path;
            if (node != root_)
            {
                path = getPathToNode((IProjectTreeNode) node.getParent());
            }
            else
            {
                path = new Object[]
                { root_ };
            }
            int[] indices = new int[]
            { getIndexOfChild(node.getParent(), node) };
            Object[] objects = new Object[]
            { node };

            TreeModelEvent event = new TreeModelEvent(this, path, indices,
                    objects);

            return event;
        }
        return null;
    }

    private void updateEvent(IProjectTreeNode node)
    {
        TreeModelEvent event = generateEvent(node);
        if (event != null)
        {
            Enumeration enumer = listeners_.elements();
            while (enumer.hasMoreElements())
            {
                ((TreeModelListener) enumer.nextElement()).treeNodesChanged(event);
            }
        }
    }

    private void insertEvent(IProjectTreeNode node)
    {
        TreeModelEvent event = generateEvent(node);
        if (event != null)
        {
            Enumeration enumer = listeners_.elements();
            while (enumer.hasMoreElements())
            {
                TreeModelListener listener = (TreeModelListener) enumer.nextElement();
                listener.treeNodesInserted(event);
            }
        }
    }

    private void deleteEvent(IProjectTreeNode node)
    {
        TreeModelEvent event = generateEvent(node);
        if (event != null)
        {
            Enumeration enumer = listeners_.elements();
            while (enumer.hasMoreElements())
            {
                ((TreeModelListener) enumer.nextElement()).treeNodesRemoved(event);
            }
        }
    }

    public void fullStructureUpdate()
    {
        structureUpdate(root_);
    }

    private void structureUpdate(IProjectTreeNode node)
    {
        TreeModelEvent event = generateEvent(node);

        if (event != null)
        {
            Enumeration enumer = listeners_.elements();
            while (enumer.hasMoreElements())
            {
                ((TreeModelListener) enumer.nextElement()).treeStructureChanged(event);
            }
        }
    }

    private Object[] getPathToNode(IProjectTreeNode node)
    {
        Vector nodes = new Vector();
        TreeNode pointer = node;
        nodes.add(pointer);
        while (pointer.getParent() != null)
        {
            pointer = pointer.getParent();
            nodes.add(pointer);
        }
        Object[] pathNodes = new Object[nodes.size()];
        for (int i = nodes.size() - 1, j = 0; i >= 0; i--, j++)
        {
            pathNodes[j] = nodes.get(i);
        }

        return pathNodes;
    }

    public void setComparator(Comparator cmp)
    {
        comparator_ = cmp;
        Iterator iter = folders_.values().iterator();
        while (iter.hasNext())
        {
            Object object = iter.next();
            if (object instanceof TreeFolder)
            {
                ((TreeFolder) object).childs_.setComparator(cmp);
            }
            else
            {
                System.err.println(object.getClass().getName());
            }
        }
        structureUpdate(root_);
    }

    public List<IProjectTreeNode> getExpandedNodes(IProjectTreeNode root)
    {
        IProjectTreeNode node;
        ArrayList<IProjectTreeNode> nodes;
        Enumeration<IProjectTreeNode> enumeration;

        if (root == null)
        {
            return getExpandedNodes(root_);
        }

        nodes = new ArrayList<IProjectTreeNode>();
        enumeration = root.children();
        while (enumeration.hasMoreElements())
        {
            node = enumeration.nextElement();
            if (ProjectTreePane.getInstance().getTree().isExpanded(
                    new TreePath(getPathToNode(node))))
            {
                nodes.add(node);
                nodes.addAll(getExpandedNodes(node));
            }
        }

        return nodes;
    }
}
