package viptool.impl.project.gui;

import java.util.Enumeration;

import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;

import viptool.project.IProjectTreeModel;
import viptool.project.IProjectTreeNode;

public class ProjectTreeNode implements IProjectTreeNode
{

    public static final String FOLDER    = "folder";
    public static final String PROJECT   = "project";
    public static final String WORKSPACE = "workspace";

    private IProjectTreeNode   parent_;
    private String             name_;

    private IProjectTreeModel  model_;

    private boolean            folder_;
    private boolean            project_;
    private boolean            workspace_;

    public ProjectTreeNode(String name, String type)
    {
        this(name);
        if (type.equals(FOLDER))
        {
            folder_ = true;
        }
        else if (type.equals(PROJECT))
        {
            project_ = true;
        }
        else if (type.equals(WORKSPACE))
        {
            workspace_ = true;
        }
    }

    public ProjectTreeNode(Object userobject, String type)
    {
        this(userobject);
        if (type.equals(FOLDER))
        {
            folder_ = true;
        }
        else if (type.equals(PROJECT))
        {
            project_ = true;
        }
        else if (type.equals(WORKSPACE))
        {
            workspace_ = true;
        }
    }

    public ProjectTreeNode(String name)
    {
        name_ = name;
    }

    public ProjectTreeNode(Object userobject)
    {
        name_ = userobject.toString();
    }

    public Enumeration children()
    {
        if (model_ != null)
        {
            return model_.getChildren(this);
        }
        return null;
    }

    public boolean getAllowsChildren()
    {
        if (isFolder() || isProject() || isWorkspace())
        {
            return true;
        }
        return false;
    }

    public TreeNode getChildAt(int childIndex)
    {
        if (model_ != null)
        {
            return (TreeNode) model_.getChild(this, childIndex);
        }
        return null;
    }

    public int getChildCount()
    {
        if (model_ != null)
        {
            return model_.getChildCount(this);
        }
        return 0;
    }

    public int getIndex(TreeNode node)
    {
        if (model_ != null)
        {
            return model_.getIndexOfChild(this, node);
        }
        return -1;
    }

    public TreeNode getParent()
    {
        return parent_;
    }

    public void insert(MutableTreeNode child, int index)
    {
        if (model_ != null)
        {
            child.setParent(this);
        }
    }

    public boolean isFolder()
    {
        return folder_;
    }

    public boolean isLeaf()
    {
        if (model_ != null)
        {
            return model_.isLeaf(this);
        }
        return true;
    }

    public boolean isProject()
    {
        return project_;
    }

    public boolean isWorkspace()
    {
        return workspace_;
    }

    public void remove(int index)
    {
        if (model_ != null)
        {
            Object oChild = model_.getChild(this, index);
            if (oChild instanceof MutableTreeNode)
            {
                ((MutableTreeNode) oChild).removeFromParent();
            }
        }
    }

    public void remove(MutableTreeNode node)
    {
        node.removeFromParent();
    }

    public void removeFromParent()
    {
        if (model_ != null && getParent() != null)
        {
            model_.removeNode(this);
        }
        parent_ = null;
    }

    public void setFolder(boolean isFolder)
    {
        folder_ = isFolder;
        if (isFolder)
        {
            project_ = false;
            workspace_ = false;
        }
    }

    public void setParent(MutableTreeNode parent)
    {
        if (parent instanceof IProjectTreeNode)
        {
            IProjectTreeNode projectparent = (IProjectTreeNode) parent;
            model_ = projectparent.getModel();
            if (model_ != null)
            {
                parent_ = projectparent;
                model_.addNode(this);
            }
        }
    }

    public void setProject(boolean isProject)
    {
        project_ = isProject;
        if (isProject)
        {
            folder_ = false;
            workspace_ = false;
        }
    }

    public void setUserObject(Object object)
    {
        name_ = object.toString();
        update();
    }

    public void setWorkspace(boolean isWorkspace)
    {
        workspace_ = isWorkspace;
        if (isWorkspace)
        {
            project_ = false;
            folder_ = false;
        }
    }

    public void update()
    {
        if (model_ != null)
        {
            model_.updateNode(this);
        }
    }

    public IProjectTreeModel getModel()
    {
        return model_;
    }

    public String toString()
    {
        return name_;
    }

    public void setModel(IProjectTreeModel model)
    {
        model_ = model;
    }
}
