package viptool.impl.project.gui;

import java.awt.BorderLayout;
import java.awt.Component;

import javax.swing.Icon;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;

import viptool.base.gui.INonCloseableChildWindow;
import viptool.base.gui.INonFocusableChildWindow;
import viptool.impl.base.MainFrame;
import viptool.impl.base.Workbench;
import viptool.impl.project.gui.treedialogs.LookAndFeelPanel;
import viptool.impl.util.resources.ResourceLoader;
import viptool.util.resources.ResourceConstants;

/**
 * This class is the panel for the projecttree
 */
public class ProjectTreePane extends JPanel implements
        INonCloseableChildWindow, INonFocusableChildWindow
{

    // tree
    private ProjectTree            tree_;
    // model for tree
    private ProjectTreeModel       model_;
    // the running instace of dndtreepane
    private static ProjectTreePane instance_;

    // name of the editor group
    public static final String     GROUP_NAME = ResourceConstants.GROUP_PROJECTS;

    // <><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><>
    /**
     * creates a new treepanel with the given name
     */
    private ProjectTreePane(String name)
    {
        // set name
        setName(name);
        // set BoderLayout
        setLayout(new BorderLayout());
        // create model
        createModel(name);
        // create JTree
        tree_ = new ProjectTree(model_);
        // add tree to panel
        add(new JScrollPane(tree_), BorderLayout.CENTER);
    }

    // <><><><><><><><><><><><><><><><><><>
    // methode to create the treemodel
    // <><><><><><><><><><><><><><><><><><>
    /**
     * creates a new treemodel
     */
    private void createModel(String name)
    {
        // create model for tree
        model_ = new ProjectTreeModel(name);
    }

    // <><><><><><><><><><><><><><><><><><>
    // static methodes to manage tree
    // <><><><><><><><><><><><><><><><><><>
    /**
     * creates a new tree with the given name
     */
    public static ProjectTreePane createTree(String name)
    {
        if (instance_ == null)
        {
            instance_ = new ProjectTreePane(name);
            return instance_;
        }
        else
        {
            instance_.close();
            instance_ = null;
            instance_ = new ProjectTreePane(name);
            showTree();
            return instance_;
        }
    }

    /**
     * shows the tree
     */
    public static void showTree()
    {
        MainFrame.getInstance().getExtendedPane().addChildWindow(instance_);
    }

    /**
     * returns the running tree or create a default-tree if there is no running
     * tree
     */
    public static synchronized ProjectTreePane getInstance()
    {
        if (instance_ == null)
        {
            String name = "default";

            if (Workbench.getInstance().getCurrentWorkspace() != null)
            {
                name = Workbench.getInstance().getCurrentWorkspace().toString();
            }

            instance_ = new ProjectTreePane(name);
        }
        return instance_;
    }

    // <><><><><><><><><><><><><><><><><><>
    // methodes for manage tree
    // <><><><><><><><><><><><><><><><><><>
    /**
     * close the running tree
     */
    public void close()
    {
        MainFrame.getInstance().getExtendedPane().removeChildWindow(this);
        instance_ = null;
    }

    /**
     * returns the model of the running tree
     */
    public ProjectTreeModel getTreeModel()
    {
        return model_;
    }

    /**
     * retruns the running tree
     */
    public ProjectTree getTree()
    {
        return tree_;
    }

    /**
     * reset the running tree (create empty model and set name to the given
     * string)
     */
    public void resetTree(String name)
    {
        setName(name);
        createModel(name);
        tree_.setModel(model_);
        tree_.revalidate();
        MainFrame.getInstance().getExtendedPane().updateTitle(this);
    }

    /**
     * set the look and feel for the tree
     * 
     */
    public void setLookAndFeel()
    {
        new LookAndFeelPanel(this, MainFrame.getInstance());
    }

    // <><><><><><><><><><><><><><><><><><>
    // implementation of Childwindow
    // <><><><><><><><><><><><><><><><><><>

    /**
     * returns always true, because the tree can always be closed
     */
    public boolean canClose(boolean force)
    {
        return true;
    }

    /**
     * returns this tree
     */
    public Component getComponent()
    {
        return this;
    }

    /**
     * returns the icon for this tree (ResourceConstants.ICON_WORKSPACE)
     */
    public Icon getIcon()
    {
        return ResourceLoader.getInstance().getIcon(
                ResourceConstants.ICON_WORKSPACE);
    }

    /**
     * returns the menubar for this tree (return null, because treemenubar is
     * defaultmenubar)
     */
    public JMenuBar getMenuBar()
    {
        return null;
    }

    /**
     * retruns the title for the childwindow
     */
    public String getTitle()
    {
        return "Workspace " + getName();
    }

    /**
     * retuns the toolbar for the tree (return null, because toolbar is
     * defaulttoolbar)
     */
    public JToolBar getToolBar()
    {
        return null;
    }

    public void windowActivated()
    {
    }
}
