package viptool.impl.project.gui;

import java.awt.Toolkit;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JComponent;
import javax.swing.JTree;
import javax.swing.TransferHandler;
import javax.swing.plaf.TreeUI;
import javax.swing.plaf.basic.BasicTreeUI;
import javax.swing.tree.TreePath;

/**
 * this class is needed to support drag and drop better
 * than the default-implementation. Most of the code is copied from the
 * BasicTreeUI-class and their parent classes.
 */
public class ProjectTreeUI extends BasicTreeUI{
	
	/**
	 * The tuned DefaultDragGestureRecognizer from BasicTreeUI
	 */
	static class DnDTreeDragGestureRecognizer 
			implements MouseListener, MouseMotionListener{

		private MouseEvent dndArmedEvent = null;
	    
	    private static int motionThreshold;
	    
	    private static boolean checkedMotionThreshold = false;

	    private static int getMotionThreshold() {
	        if (checkedMotionThreshold) {
	            return motionThreshold;
	        } else {
	            checkedMotionThreshold = true;
	            try {
	                motionThreshold = ((Integer)Toolkit.getDefaultToolkit().getDesktopProperty("DnD.gestureMotionThreshold")).intValue();
	            } catch (Exception e) {
	                motionThreshold = 5;
	            }
	        }
	        return motionThreshold;
	    }
	    
	    protected int mapDragOperationFromModifiers(MouseEvent e) {
	        int mods = e.getModifiersEx();
	        
	        if ((mods & InputEvent.BUTTON1_DOWN_MASK) != InputEvent.BUTTON1_DOWN_MASK) {
	            return TransferHandler.NONE;
	        }

	        //JComponent c = getComponent(e);
	        //TransferHandler th = c.getTransferHandler();
	        return -1;
	        //return SunDragSourceContextPeer.convertModifiersToDropAction(mods, th.getSourceActions(c)); 
	    }
	    
	    public void mouseClicked(MouseEvent e) {
	    }

	    /**
	     * THIS METHODE WAS CHANGED A LITTLE BIT TO ACCEPT DRAG OF UNSELECTED NODES
	     */
	    public void mousePressed(MouseEvent e) {
	    	dndArmedEvent = null;

	    	if((e.getModifiersEx() & MouseEvent.SHIFT_DOWN_MASK) == 0 
	    			&& (e.getModifiersEx() & MouseEvent.CTRL_DOWN_MASK) == 0)
		    	if (isDragPossible(e) && mapDragOperationFromModifiers(e) != TransferHandler.NONE) {
		    		dndArmedEvent = e;
		    	}
	    }
	    
	    public void mouseReleased(MouseEvent e) {
	        dndArmedEvent = null;
	    }
	    
	    public void mouseEntered(MouseEvent e) {
	    }
	    
	    public void mouseExited(MouseEvent e) {
	    }

	    public void mouseDragged(MouseEvent e) {
	    	if (dndArmedEvent != null) 
	    	{
	            e.consume();
	            
	            int action = mapDragOperationFromModifiers(e);
	            
	            if (action == TransferHandler.NONE) {
	                return;
	            }
	            
			    int dx = Math.abs(e.getX() - dndArmedEvent.getX());
			    int dy = Math.abs(e.getY() - dndArmedEvent.getY());
			    if ((dx > getMotionThreshold()) || (dy > getMotionThreshold())) 
			    {
			    	// start transfer... shouldn't be a click at this point
		            JComponent c = getComponent(e);
		            TransferHandler th = c.getTransferHandler();
		            th.exportAsDrag(c, dndArmedEvent, action);
		            dndArmedEvent = null;
			    }
			}
	    }
	    
	    public void mouseMoved(MouseEvent e) {
	    }
	    
	    /*private TransferHandler getTransferHandler(MouseEvent e) {
	        JComponent c = getComponent(e);
	        return c == null ? null : c.getTransferHandler();
	    }*/
	    
	    /**
	     * Determines if the following are true:
	     * <ul>
	     * <li>the press event is located over a selection
	     * <li>the dragEnabled property is true
	     * <li>A TranferHandler is installed
	     * </ul>
	     * <p>
	     * This is implemented to check for a TransferHandler.
	     * Subclasses should perform the remaining conditions.
	     */
	    protected JComponent getComponent(MouseEvent e) 
	    {
	    	Object src = e.getSource();
	    	if (src instanceof JComponent) 
	    	{
	    		JComponent c = (JComponent) src;
	    		return c;
	    	}
	    	return null;
	    }
		
		/**
		 * Determines if the following are true:
		 * <ul>
		 * <li>the press event is located over a selection
		 * <li>the dragEnabled property is true
		 * <li>A TranferHandler is installed
		 * </ul>
		 * <p>
		 * This is implemented to check for a TransferHandler.
		 * Subclasses should perform the remaining conditions.
		 * 
		 * THIS METHODE WAS CHANGED A LITTLE BIT TO ACCEPT DRAG OF UNSELECTED NODES
		 */
	    protected boolean isDragPossible(MouseEvent e) {
	        JComponent c = getComponent(e);
		    if (c != null && c.getTransferHandler() != null) 
		    {
		    	JTree tree = (JTree)this.getComponent(e);
		    	if(tree.getDragEnabled()) 
		    	{
		    		TreeUI ui = tree.getUI();
		    		TreePath path = ui.getClosestPathForLocation(tree, e.getX(), e.getY());
		    		if(path != null) 
		    		{
		    			if(tree.getSelectionPaths() == null)
		    			{
		    				tree.setSelectionPath(path);
		    			}
		    			else if(!tree.isPathSelected(path) && tree.getSelectionPaths().length == 1)
		    			{
		    				tree.setSelectionPath(path);
		    			}
		    			else
		    			{
		    				tree.addSelectionPath(path);
		    			}
		    			return true;
		    		}
		    	}
		    }
		    return false;
		}
	}
	
	private static DnDTreeDragGestureRecognizer recognizer_ = new DnDTreeDragGestureRecognizer();
	
	/**
	 * This methode is used to install the new DragGestureRecognizer
	 */
	protected void installListeners() 
	{
		super.installListeners();
		tree.addMouseMotionListener(recognizer_);
		tree.addMouseListener(recognizer_);
	}
}
