package viptool.impl.project.gui;

import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.event.InputEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JTree;
import javax.swing.TransferHandler;
import javax.swing.tree.TreePath;

import viptool.impl.project.ProjectGuiLinker;
import viptool.impl.project.action.ProjectTreeTransferable;



public class TreeDragAndDropSupport extends TransferHandler{
	
	private static TreeDragAndDropSupport instance_ = null;
	
	public static TreeDragAndDropSupport getInstance()
	{
		if(instance_ == null)
		{
			instance_ = new TreeDragAndDropSupport();
		}
		return instance_;
	}
	
	private TreePath[] paths_;
	
	private TreeDragAndDropSupport()
	{
		
	}
	
	/**
	 * tests if a drag can be imported
	 */
	public boolean canImport(JComponent comp, DataFlavor[] transferFlavors)
	{
		for(int i=0; i<transferFlavors.length; i++)
		{
			if(transferFlavors[i].equals(DataFlavor.javaFileListFlavor))
			{
				return true;
			}
		}
		return false;
	}
	
	/**
	 * export selected data as new drag
	 */
	public void exportAsDrag(JComponent comp, InputEvent e, int action) 
	{
		super.exportAsDrag(comp, e, action);
	}
		
	public void exportToClipboard(ProjectTree tree, Clipboard clip, int action)
	{
		boolean exportSuccess = false;
		Transferable t = null;

		int clipboardAction = getSourceActions(tree) & action;
		if (clipboardAction != TransferHandler.NONE) 
		{
			t = createTransferable(tree);
			if (t != null) 
			{
				clip.setContents(t, null);
				exportSuccess = true;
            }
        }
        if (exportSuccess) 
        {
        	exportDone(tree, t, clipboardAction);
        }
        else 
        {
        	exportDone(tree, null, TransferHandler.NONE);
        }
	}
	
	/**
	 * ends a drag-drop-operation, means it deletes the transfered elements
	 * if it was a move
	 */
	protected void exportDone(JComponent source, Transferable data, int action)
	{
		if(data instanceof ProjectTreeTransferable)
		{
			ProjectTreeTransferable transferable = (ProjectTreeTransferable)data;
			if(source instanceof ProjectTree && action == DnDConstants.ACTION_MOVE
					&& !transferable.wasText() && ProjectTreeTransferable.getInternal())
			{
				for(int i=0; i<paths_.length; i++)
				{
					if(paths_[i].getPath().length > 1)
					{
						Object node = paths_[i].getLastPathComponent();
						ProjectGuiLinker.getInstance().delete((ProjectTreeNode)node);
						
					}
				}
			}
		}
	}
	
	/**
	 * creates a new transferable
	 */
	public Transferable createTransferable(JComponent c)
	{
		if(c instanceof JTree)
		{
			TreePath[] paths = ((JTree)c).getSelectionPaths();
			
			//Backup paths for exportDone
			paths_ = paths;
			if(paths != null)
			{
				Vector files = new Vector();
				for(int i=0; i<paths.length; i++)
				{
					Object object = paths[i].getLastPathComponent();
					if(object instanceof ProjectTreeNode)
					{
						ProjectTreeNode node = (ProjectTreeNode)object;
						File file = ProjectGuiLinker.getInstance().getFile(node);
						files.add(file);
					}
				}
							
				String text = null;
				TreePath path = ((JTree)c).getSelectionPath();
				if(path != null)
				{
					Object object = path.getLastPathComponent();
					if(object instanceof ProjectTreeNode)
					{
						ProjectTreeNode node = (ProjectTreeNode)object;
						File file = ProjectGuiLinker.getInstance().getFile(node);
						if(file != null && !file.isDirectory())
						{
							
							try
							{
								InputStreamReader reader = new InputStreamReader(new FileInputStream(file));
								BufferedReader buffreader = new BufferedReader(reader);
								StringBuffer buffer = new StringBuffer();
								int ch;
								while((ch = buffreader.read()) >= 0)
								{
									buffer.append((char)ch);
								}
								buffreader.close();
								text = buffer.toString();
							}
							catch(Exception e)
							{
								System.err.println("TreeDragAndDropSupport: createTransferable:");
								e.printStackTrace();
							}
						}
					}
				}
				if(text == null)
				{
					text = "File couldn't be loaded!";
				}
				
				if(files.size()>0)
				{
					return new ProjectTreeTransferable(files, text);
				}
			}
		}
		return null;
	}
	
	/**
	 * returns the actions the tree supports
	 */
	public int getSourceActions(JComponent c)
	{
		return DnDConstants.ACTION_COPY_OR_MOVE;
	}
	
}
