package viptool.impl.project.io;

import java.io.IOException;
import java.util.regex.Pattern;

import javax.swing.Icon;

import viptool.base.IWorkbench;
import viptool.base.error.FileInterpreterProviderException;
import viptool.base.io.IFileInterpreter;
import viptool.base.io.IFileInterpreterProvider;
import viptool.base.wizard.IWizard;
import viptool.impl.project.wizard.NewProjectWizard;
import viptool.impl.util.resources.ResourceLoader;
import viptool.project.IAbstractFile;
import viptool.util.FileUtils;
import viptool.util.resources.ResourceConstants;

public class ProjectFileProvider implements IFileInterpreterProvider
{
    // pattern <?xml ...><workspace>
    private static final Pattern PATTERN = Pattern.compile(
                                                 ".*<\\?xml\\s*.*\\?>\\s*<project\\s*.*>"
                                                         + ".*",
                                                 Pattern.CASE_INSENSITIVE
                                                         | Pattern.DOTALL);

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the FileInterpreterProvider interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.base.io.FileInterpreterProvider */
    public boolean matchFormat(IAbstractFile file)
            throws FileInterpreterProviderException
    {
        try
        {
            return FileUtils.isFileOfType(file.getFile(), PATTERN);
        }
        catch(IOException exception)
        {
            throw new FileInterpreterProviderException(this,
                    "The file type cannot be determined!", exception);
        }
    }

    /* @see viptool.base.io.FileInterpreterProvider */
    public IFileInterpreter createFromFile(IAbstractFile file)
            throws FileInterpreterProviderException
    {
        return new ProjectFile(this, file);
    }

    /* @see viptool.base.io.FileInterpreterProvider */
    public IFileInterpreter createNewFile(IAbstractFile file)
            throws FileInterpreterProviderException
    {
        try
        {
            FileUtils.createFileWithContent(file.getFile(),
                    "<?xml version=\"1.0\" encoding=\"utf-8\"?>"
                            + "<project></project>");
        }
        catch(IOException exception)
        {
            throw new FileInterpreterProviderException(this,
                    "The project cannot be created!", exception);
        }

        return new ProjectFile(this, file);
    }

    /* @see viptool.base.io.FileInterpreterProvider */
    public String getFileType()
    {
        return "Viptool project";
    }

    /* @see viptool.base.io.FileInterpreterProvider */
    public String getDescription()
    {
        return "Viptool project that can contain petri nets an other files.";
    }

    /* @see viptool.base.io.FileInterpreterProvider */
    public Icon getIcon()
    {
        return ResourceLoader.getInstance().getIcon(
                ResourceConstants.ICON_PROJECT);
    }

    /* @see viptool.base.io.FileInterpreterProvider */
    public IWizard getNewFileWizard()
    {
        return new NewProjectWizard(this);
    }

    /* @see viptool.base.io.FileInterpreterProvider */
    public boolean hasNewFileWizard()
    {
        return true;
    }
    
    public boolean canCreateNewFile(IWorkbench workbench)
    {
        return (workbench.getCurrentWorkspace() != null);
    }
}
