package viptool.impl.project.io;

import java.io.IOException;
import java.util.regex.Pattern;

import javax.swing.Icon;

import viptool.base.IWorkbench;
import viptool.base.error.FileInterpreterProviderException;
import viptool.base.io.IFileInterpreter;
import viptool.base.io.IFileInterpreterProvider;
import viptool.base.wizard.IWizard;
import viptool.impl.project.wizard.NewWorkspaceWizard;
import viptool.impl.util.resources.ResourceLoader;
import viptool.project.IAbstractFile;
import viptool.util.FileUtils;
import viptool.util.resources.ResourceConstants;

/**
 * This class is the provider for workspaces. It can create new workspace files
 * and workspace objects.
 */
public class WorkspaceFileProvider implements IFileInterpreterProvider
{
    // pattern <?xml ...><workspace>
    private static final Pattern PATTERN = Pattern.compile(
                                                 ".*<\\?xml\\s*.*\\?>\\s*<workspace\\s*.*/?>"
                                                         + ".*",
                                                 Pattern.CASE_INSENSITIVE
                                                         | Pattern.DOTALL);

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the FileInterpreterProvider interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.base.io.FileInterpreterProvider */
    public boolean matchFormat(IAbstractFile file)
            throws FileInterpreterProviderException
    {
        try
        {
            return FileUtils.isFileOfType(file.getFile(), PATTERN);
        }
        catch(IOException exception)
        {
            throw new FileInterpreterProviderException(this,
                    "The file type cannot be determined!", exception);
        }
    }

    /* @see viptool.base.io.FileInterpreterProvider */
    public IFileInterpreter createFromFile(IAbstractFile file)
            throws FileInterpreterProviderException
    {
        return new WorkspaceFile(this, file);
    }

    /* @see viptool.base.io.FileInterpreterProvider */
    public IFileInterpreter createNewFile(IAbstractFile file)
            throws FileInterpreterProviderException
    {
        try
        {
            FileUtils.createFileWithContent(file.getFile(),
                    "<?xml version=\"1.0\" encoding=\"utf8\"?>"
                            + "<workspace></workspace>");
        }
        catch(IOException exception)
        {
            throw new FileInterpreterProviderException(this,
                    "The workspace cannot be created!", exception);
        }

        return new WorkspaceFile(this, file);
    }

    /* @see viptool.base.io.FileInterpreterProvider */
    public String getFileType()
    {
        return "viptool workspace";
    }

    /* @see viptool.base.io.FileInterpreterProvider */
    public String getDescription()
    {
        return "Viptool workspace that contains several projects.";
    }

    /* @see viptool.base.io.FileInterpreterProvider */
    public Icon getIcon()
    {
        return ResourceLoader.getInstance().getIcon(
                ResourceConstants.ICON_WORKSPACE);
    }

    /* @see viptool.base.io.FileInterpreterProvider */
    public IWizard getNewFileWizard()
    {
        return new NewWorkspaceWizard(this);
    }

    /* @see viptool.base.io.FileInterpreterProvider */
    public boolean hasNewFileWizard()
    {
        return true;
    }
    /* @see viptool.util.filetype.FileTypeProvider */
    public boolean canCreateNewFile(IWorkbench workbench)
    {
        return true;
    }
    
}
