package viptool.impl.project.wizard;

import java.awt.Image;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.JComponent;

import viptool.base.error.FileInterpreterProviderException;
import viptool.base.gui.component.PathBrowserPanel;
import viptool.base.gui.component.event.IPathBrowserPanelListener;
import viptool.base.gui.dialog.DialogConstants;
import viptool.base.gui.dialog.IStandardModalDialog;
import viptool.base.wizard.IWizardPage;
import viptool.base.wizard.WizardAdapter;
import viptool.impl.base.Workbench;
import viptool.impl.project.io.ProjectFileProvider;
import viptool.impl.util.resources.ResourceLoader;
import viptool.project.FileSystem;
import viptool.project.IProject;
import viptool.util.resources.ResourceConstants;

public class NewProjectWizard extends WizardAdapter
{
    // the wizard page
    private ProjectPage          projectPage_ = null;

    // the provider the to the project
    private ProjectFileProvider  provider_    = null;

    // the hosting standard dialog
    private IStandardModalDialog dialog_      = null;

    // a flag the wizard was finish correctly. Used in windowClosed to avoid
    // a call to cancel on windowClosed method.
    private boolean              finished_    = false;

    // <><><><><><><><><><><><><><><><><><><><><>
    // inner classes
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * The workspace page for creating a new workspace. We have only one page
     * for entering name and path of the workspace.
     */
    private class ProjectPage implements IWizardPage, IPathBrowserPanelListener
    {
        // the page content
        private PathBrowserPanel pathBrowser_;

        // <><><><><><><><><><><><><><><><><><><><><>
        // Constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        public ProjectPage()
        {
            pathBrowser_ = new PathBrowserPanel();
            pathBrowser_.addPathBrowserPanelListener(this);
        }

        public String getPath()
        {
            return pathBrowser_.getSelectedPath();
        }

        public String getName()
        {
            return pathBrowser_.getSelectedName();
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementation of the WizardPage interface
        // <><><><><><><><><><><><><><><><><><><><><>
        /* @see viptool.gui.wizards.WizardPage */
        public boolean canGoToPrevoius()
        {
            // we have only one page
            return false;
        }

        /* @see viptool.gui.wizards.WizardPage */
        public JComponent getContent()
        {
            return pathBrowser_;
        }

        /* @see viptool.gui.wizards.WizardPage */
        public Image getImage()
        {
            return ResourceLoader.getInstance().getImage(
                    ResourceConstants.ICON_PROJECT);
        }

        /* @see viptool.gui.wizards.WizardPage */
        public String getMainDescription()
        {
            return "Enter Project info...";
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementation of the IPathBrowserListener interface
        // <><><><><><><><><><><><><><><><><><><><><>

        /* @see viptool.base.gui.dialog.event.IPathBrowserPanelListener */
        public void focusGainedOnName()
        {
            dialog_.setDetailedDescription("Enter a new name for the "
                    + "project here");
        }

        /* @see viptool.base.gui.dialog.event.IPathBrowserPanelListener */
        public void focusGainedOnPath()
        {
            dialog_.setDetailedDescription("Select a path for your "
                    + "new project");
        }

        /* @see viptool.base.gui.dialog.event.IPathBrowserPanelListener */
        public void focusLost()
        {
            dialog_.setDetailedDescription("");
        }

        /* @see viptool.base.gui.dialog.event.IPathBrowserPanelListener */
        public void nameNotValid()
        {
            dialog_.setNotice("Please enter a valid name for the project!",
                    DialogConstants.TYPE_ERROR);
        }

        /* @see viptool.base.gui.dialog.event.IPathBrowserPanelListener */
        public void pathNotValid()
        {
            dialog_.setNotice("Please enter a valid path for the project!",
                    DialogConstants.TYPE_ERROR);
        }

        /* @see viptool.base.gui.dialog.event.IPathBrowserPanelListener */
        public void validated()
        {
            dialog_.removeNotice();
            dialog_.setButtonsEnabled(DialogConstants.BUTTON_FINISH, true);
        }
    }

    public NewProjectWizard(ProjectFileProvider provider)
    {
        provider_ = provider;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the Wizard interface
    // <><><><><><><><><><><><><><><><><><><><><>

    /* @see viptool.wizards.Wizard */
    public void cancel()
    {
        // nothing to do here, the user has canceled the operation
    }

    /* @see viptool.wizards.Wizard */
    public void finish()
    {
        IProject project;

        try
        {
            // create Directories
            (new File(projectPage_.getPath())).mkdirs();
            project = (IProject) provider_.createNewFile(FileSystem.getInstance().createAbstractFile(
                    projectPage_.getPath() + File.separator
                            + projectPage_.getName() + ".prj"));

            Workbench.getInstance().getCurrentWorkspace().addProject(project);
        }
        catch (FileInterpreterProviderException exception)
        {
            Workbench.getInstance().showExceptionInDialog(exception);
        }

        finished_ = true;
    }

    /* @see viptool.wizards.Wizard */
    public synchronized IWizardPage getFirstPage(IStandardModalDialog dialog)
    {
        if (projectPage_ == null)
        {
            projectPage_ = new ProjectPage();
        }

        dialog_ = dialog;
        return projectPage_;
    }

    /* @see viptool.wizards.Wizard */
    public IWizardPage getNextPage()
    {
        // we have only one page
        return null;
    }

    /* @see viptool.wizards.Wizard */
    public IWizardPage getPreviousPage()
    {
        // we have only one page
        return null;
    }

    /* @see viptool.wizards.Wizard */
    public String getTitle()
    {
        return "New Project";
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementation of the WindowListener interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see java.awt.event.WindowListener */
    public void windowActivated(WindowEvent event)
    {
        String path;

        path = Workbench.getInstance().getCurrentWorkspace().getAbstractFile().getFile().getParentFile().getAbsolutePath();

        ((PathBrowserPanel) projectPage_.getContent()).initialize(path, true);
    }

    /* @see java.awt.event.WindowListener */
    public void windowClosed(WindowEvent event)
    {
        // if finished was not called the user has cancelled the wizard
        if (!finished_)
        {
            cancel();
        }
    }
}
