package viptool.impl.text.editor;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.GridLayout;
import java.awt.dnd.DropTarget;

import javax.swing.ActionMap;
import javax.swing.Icon;
import javax.swing.InputMap;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.JToolBar;
import javax.swing.JViewport;
import javax.swing.KeyStroke;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import static viptool.Constants.*;
import viptool.base.editor.Editor;
import viptool.base.editor.IEditor;
import viptool.base.editor.IEditorProvider;
import viptool.base.error.EditorNotAcceptedException;
import viptool.base.error.EditorNotOpenedException;
import viptool.base.error.FileInterpreterException;
import viptool.base.gui.IChildWindow;
import viptool.impl.base.Workbench;
import viptool.impl.base.gui.menu.MenuBarManager;
import viptool.impl.base.gui.toolbar.ToolBarManager;
import viptool.impl.text.editor.action.TexteditorAction;
import viptool.impl.text.editor.action.TexteditorMouseListener;
import viptool.impl.text.editor.dnd.TextDropTargetListener;
import viptool.impl.text.editor.dnd.TextTransferhandler;
import viptool.impl.text.io.TextFile;
import viptool.impl.util.resources.ResourceConstants;

// TODO comment
// TODO reimplement
public class TextEditor extends Editor implements IChildWindow,
        DocumentListener, IEditor
{
    private JTextPane   textpane_;

    private TextFile    element_;

    private JToolBar    toolbar_;

    private JMenuBar    menubar_;

    private JScrollPane content_;

    /**
     * This methode creates a new texteditor to the given TextFile.
     * 
     * @param element
     *            TextFile which should be shown
     */
    public TextEditor(IEditorProvider provider, TextFile element)
    {
        super(Workbench.getInstance(), provider);

        JPanel panel = new JPanel();

        content_ = new JScrollPane();

        element_ = element;
        textpane_ = new JTextPane();
        textpane_.setCursor(new Cursor(Cursor.TEXT_CURSOR));
        // textpane_.setToolTipText("The Defaulteditor. Right-Click for menu.");

        textpane_.addMouseListener(new TexteditorMouseListener(this));

        toolbar_ = ToolBarManager.getInstance().createToolBar(
                ID_TOOLBAR_DEFAULT_OPENED, this);
        menubar_ = MenuBarManager.getInstance().createMenuBar(
                ID_MENU_DEFAULT_OPENED, this);

        // enable shortcuts
        InputMap imap = textpane_.getInputMap();
        imap.put(KeyStroke.getKeyStroke("ctrl X"), "cut");
        imap.put(KeyStroke.getKeyStroke("ctrl C"), "copy");
        imap.put(KeyStroke.getKeyStroke("ctrl V"), "paste");
        imap.put(KeyStroke.getKeyStroke("ctrl S"), "save");
        imap.put(KeyStroke.getKeyStroke("ctrl E"), "close");

        ActionMap map = textpane_.getActionMap();
        map.put("copy",
                new TexteditorAction(TexteditorAction.ACTION_COPY, this));
        map.put("cut", new TexteditorAction(TexteditorAction.ACTION_CUT, this));
        map.put("paste", new TexteditorAction(TexteditorAction.ACTION_PASTE,
                this));
        map.put("save",
                new TexteditorAction(TexteditorAction.ACTION_SAVE, this));
        map.put("close", new TexteditorAction(TexteditorAction.ACTION_CLOSE,
                this));

        // implementation of Drag and Drop
        // TODO: make drag working and insert drop at right position
        textpane_.setTransferHandler(new TextTransferhandler(this));
        textpane_.setDragEnabled(true);
        textpane_.setDropTarget(new DropTarget(textpane_,
                new TextDropTargetListener(this)));
        textpane_.setAutoscrolls(true);

        panel.setLayout(new GridLayout());
        panel.add(textpane_);

        content_.setViewport(new JViewport());
        content_.setViewportView(panel);
    }

    public String getText()
    {
        return textpane_.getText();
    }

    public void setText(String text)
    {
        textpane_.setText(text);
    }

    public JTextPane getTextPane()
    {
        return textpane_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // implementation of IEditor
    // <><><><><><><><><><><><><><><><><><><><><>

    public void save() throws EditorNotOpenedException
    {
        try
        {
            element_.save();
            setModified(false);
        }
        catch (FileInterpreterException exception)
        {
            Workbench.getInstance().showExceptionInDialog(exception);
        }

    }

    public Component getComponent()
    {
        return content_;
    }

    public Icon getIcon()
    {
        return element_.getProvider().getIcon();
    }

    public JMenuBar getMenuBar()
    {
        return menubar_;
    }

    public JToolBar getToolBar()
    {
        return toolbar_;
    }

    public String getToolTipText()
    {
        return "The file \"" + getTitle() + "\" is displayed as text here!";
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // implementation of DocumentListener
    // <><><><><><><><><><><><><><><><><><><><><>

    public void changedUpdate(DocumentEvent e)
    {
        setModified(!textpane_.getText().equals(element_.getText()));
    }

    public void insertUpdate(DocumentEvent e)
    {
        setModified(!textpane_.getText().equals(element_.getText()));
    }

    public void removeUpdate(DocumentEvent e)
    {
        setModified(!textpane_.getText().equals(element_.getText()));
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // implementation of IEditor
    // <><><><><><><><><><><><><><><><><><><><><>

    public void initialize()
    {
        try
        {
            textpane_.getStyledDocument().removeDocumentListener(this);
            super.open(element_);

            textpane_.setText(element_.getText());
            setModified(false);
            textpane_.getStyledDocument().addDocumentListener(this);
        }
        catch (EditorNotAcceptedException exception)
        {
            getWorkbench().showExceptionInDialog(exception);
        }
    }

    public void fileLocked()
    {
        textpane_.setBackground(new Color(251, 189, 189));
        textpane_.setEditable(false);
    }

    public void fileUnlocked()
    {
        textpane_.setBackground(Color.WHITE);
        textpane_.setEditable(true);
    }

}
