package viptool.impl.text.editor.action;

import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.KeyStroke;

import viptool.impl.text.editor.TextEditor;
import viptool.impl.text.editor.dnd.TextTransferable;
import viptool.impl.util.resources.ResourceLoader;
import viptool.util.resources.ResourceConstants;

public class TexteditorAction extends AbstractAction{

	//Type-Constants
	public static int ACTION_CLOSE = 1;
	public static int ACTION_SAVE = 2;
	public static int ACTION_CUT = 3;
	public static int ACTION_COPY = 4;
	public static int ACTION_PASTE = 5;
	
	private int type_;
	private TextEditor editor_;
	
	
	public TexteditorAction(int type, TextEditor editor)
	{
		setEnabled(true);
		type_ = type;
		editor_ = editor;
		generateAction(type);
		
	}
	
	public void generateAction(int type)
	{
		if(type == ACTION_CLOSE)
		{
			putValue(AbstractAction.SMALL_ICON, ResourceLoader.getInstance().getIcon(ResourceConstants.ICON_CLOSE_FILE));
			putValue(AbstractAction.ACCELERATOR_KEY, KeyStroke.getKeyStroke("ctrl E"));
			putValue(AbstractAction.MNEMONIC_KEY, new Integer(KeyEvent.VK_S));
			putValue(AbstractAction.SHORT_DESCRIPTION, "Close");
			putValue(AbstractAction.LONG_DESCRIPTION, "close the file without save");
			putValue(AbstractAction.NAME, "Close");
		}
		else if(type == ACTION_SAVE)
		{
			putValue(AbstractAction.SMALL_ICON, ResourceLoader.getInstance().getIcon(ResourceConstants.ICON_SAVE_FILE));
			putValue(AbstractAction.ACCELERATOR_KEY, KeyStroke.getKeyStroke("ctrl S"));
			putValue(AbstractAction.MNEMONIC_KEY, new Integer(KeyEvent.VK_S));
			putValue(AbstractAction.SHORT_DESCRIPTION, "Save");
			putValue(AbstractAction.LONG_DESCRIPTION, "save the file");
			putValue(AbstractAction.NAME, "Save");
		}
		else if(type == ACTION_COPY)
		{
			putValue(AbstractAction.SMALL_ICON, ResourceLoader.getInstance().getIcon(ResourceConstants.ICON_COPY_TEXT));
			putValue(AbstractAction.ACCELERATOR_KEY, KeyStroke.getKeyStroke("ctrl C"));
			putValue(AbstractAction.MNEMONIC_KEY, new Integer(KeyEvent.VK_C));
			putValue(AbstractAction.SHORT_DESCRIPTION, "Copy");
			putValue(AbstractAction.LONG_DESCRIPTION, "copy selected text");
			putValue(AbstractAction.NAME, "Copy");
		}
		else if(type == ACTION_CUT)
		{
			putValue(AbstractAction.SMALL_ICON, ResourceLoader.getInstance().getIcon(ResourceConstants.ICON_CUT_TEXT));
			putValue(AbstractAction.ACCELERATOR_KEY, KeyStroke.getKeyStroke("ctrl X"));
			putValue(AbstractAction.MNEMONIC_KEY, new Integer(KeyEvent.VK_X));
			putValue(AbstractAction.SHORT_DESCRIPTION, "Cut");
			putValue(AbstractAction.LONG_DESCRIPTION, "cut selected text");
			putValue(AbstractAction.NAME, "Cut");
		}
		else if(type == ACTION_PASTE)
		{
			putValue(AbstractAction.SMALL_ICON, ResourceLoader.getInstance().getIcon(ResourceConstants.ICON_PASTE_TEXT));
			putValue(AbstractAction.ACCELERATOR_KEY, KeyStroke.getKeyStroke("ctrl V"));
			putValue(AbstractAction.MNEMONIC_KEY, new Integer(KeyEvent.VK_V));
			putValue(AbstractAction.SHORT_DESCRIPTION, "Paste");
			putValue(AbstractAction.LONG_DESCRIPTION, "paste clipboard at cursor position");
			putValue(AbstractAction.NAME, "Paste");
		}
	}
	
	public void actionPerformed(ActionEvent e) {
		//IResource element = editor_.getElement();
		Clipboard sysclip = Toolkit.getDefaultToolkit().getSystemClipboard();
		
		if(type_ == ACTION_CLOSE)
		{
            // TODO: reimplement
			// element.close();
		}
		else if(type_ == ACTION_SAVE)
		{
            // TODO: reimplement
			// element.saveElement();
		}
		else if(type_ == ACTION_COPY)
		{
			String text = editor_.getTextPane().getSelectedText();
			sysclip.setContents(new TextTransferable(text), null);
		}
		else if(type_ == ACTION_CUT)
		{
			String text = editor_.getTextPane().getSelectedText();
			sysclip.setContents(new TextTransferable(text), null);
			editor_.getTextPane().replaceSelection("");
		}
		else if(type_ == ACTION_PASTE)
		{
			Transferable transfer = sysclip.getContents(editor_);
			if(transfer.isDataFlavorSupported(DataFlavor.stringFlavor))
			{
				try
				{
					String text = (String)transfer.getTransferData(DataFlavor.stringFlavor);
					editor_.getTextPane().getDocument().insertString(
							editor_.getTextPane().getCaretPosition(), 
							text, 
							null);
				}
				catch(Exception ex)
				{
					ex.printStackTrace();
				}
			}
		}
	}

}
