package viptool.impl.text.editor.dnd;

import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DnDConstants;
import java.awt.event.InputEvent;

import javax.swing.JComponent;
import javax.swing.TransferHandler;

import viptool.impl.text.editor.TextEditor;

public class TextTransferhandler extends TransferHandler
{

    private TextEditor textEditor_;

    public TextTransferhandler(TextEditor editor)
    {
        textEditor_ = editor;
    }

    /**
     * tests if a drag can be imported
     */
    public boolean canImport(JComponent comp, DataFlavor[] transferFlavors)
    {
        for (int i = 0; i < transferFlavors.length; i++)
        {
            if (transferFlavors[i].equals(DataFlavor.stringFlavor))
            {
                return true;
            }
        }
        return false;
    }

    /**
     * creates a new transferable
     */
    public Transferable createTransferable(JComponent c)
    {
        if (c == textEditor_.getComponent())
        {
            String text = textEditor_.getTextPane().getSelectedText();
            if (text != null && text.length() > 0)
            {
                return new TextTransferable(text);
            }
        }
        return null;
    }

    /**
     * ends a drag-drop-operation, means it deletes the transfered elements if
     * it was a move
     */
    protected void exportDone(JComponent source, Transferable data, int action)
    {
        if (action == DnDConstants.ACTION_MOVE)
        {
            if (source == textEditor_.getComponent())
            {
                textEditor_.getTextPane().replaceSelection("");
            }
        }
    }

    /**
     * returns the actions the tree supports
     */
    public int getSourceActions(JComponent c)
    {
        return DnDConstants.ACTION_COPY_OR_MOVE;
    }

    /**
     * export selected data as new drag
     */
    public void exportAsDrag(JComponent comp, InputEvent e, int action)
    {
        super.exportAsDrag(comp, e, action);
    }

    /**
     * exports the selected data to the systemclipboard
     */
    public void exportToClipboard(JComponent comp, Clipboard clip, int action)
    {
        if (comp == textEditor_.getComponent())
        {
            String text = textEditor_.getTextPane().getSelectedText();
            if (text != null && text.length() > 0)
            {
                clip.setContents(new TextTransferable(text), null);
                if (action == DnDConstants.ACTION_MOVE)
                {
                    textEditor_.getTextPane().replaceSelection("");
                }
            }
        }
    }

}
