package viptool.impl.text.io;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import viptool.base.editor.IEditor;
import viptool.base.error.FileInterpreterException;
import viptool.base.error.VipToolException;
import viptool.base.io.FileInterpreter;
import viptool.base.io.IFileInterpreterProvider;
import viptool.impl.base.Workbench;
import viptool.impl.logger.Logger;
import viptool.impl.text.editor.TextEditor;
import viptool.project.IAbstractFile;

/**
 * This class represents a file as text.
 */
public class TextFile extends FileInterpreter
{

    private IAbstractFile file_;
    private String        text_ = "";

    /**
     * Creates a new TextFile form the given file.
     * 
     * @param provider
     *            provider of the file
     * 
     * @param file
     *            file which should be represented
     */
    public TextFile(IFileInterpreterProvider provider, IAbstractFile file)
    {
        super(provider, file);
        file_ = file;
    }

    /**
     * This methode returns the text of the file, if the text was loaded. To
     * load the text call load()
     * 
     * @return text, the file contains
     */
    public String getText()
    {
        return text_;
    }

    /**
     * This methode loads the text from the file into the memory.
     */
    public void load() throws FileInterpreterException
    {
        File file = file_.getFile();

        text_ = "";

        try
        {
            // open innputstream...
            FileInputStream fis = new FileInputStream(file);
            BufferedInputStream bis = new BufferedInputStream(fis);
            InputStreamReader reader = new InputStreamReader(bis);

            // read the text
            while (reader.ready())
            {
                text_ = text_ + ((char) reader.read());
            }

            // close the stream
            reader.close();
            bis.close();
            fis.close();
        }
        catch (Exception e)
        {
            Logger.getInstance().log(
                    Logger.ERROR_LEVEL,
                    "TextFile: File " + file_.getFile().getAbsolutePath()
                            + " couldn't be loaded!");
            Workbench.getInstance().showExceptionInDialog(
                    new VipToolException("TextFile: File "
                            + file_.getFile().getAbsolutePath()
                            + " couldn't be loaded!"));
        }
    }

    /**
     * This methode writes the changes to the file, if an editor has made
     * changes.
     */
    public void save() throws FileInterpreterException
    {
        IEditor editor = Workbench.getInstance().findLockingEditor(file_);
        if (editor != null && editor instanceof TextEditor)
        {
            // get the text
            TextEditor texteditor = (TextEditor) editor;
            String text = texteditor.getText();
            // get the file
            File file = file_.getFile();
            try
            {
                // open outputstream
                FileOutputStream fos = new FileOutputStream(file);
                BufferedOutputStream bos = new BufferedOutputStream(fos);
                OutputStreamWriter writer = new OutputStreamWriter(bos);

                // write the text
                writer.write(text);

                // close the stream
                writer.flush();
                writer.close();
                bos.close();
                fos.close();

                // update the text
                text_ = text;
            }
            catch (Exception e)
            {
                Logger.getInstance().log(
                        Logger.ERROR_LEVEL,
                        "TextFile: File " + file_.getFile().getAbsolutePath()
                                + " couldn't be loaded!");
                Workbench.getInstance().showExceptionInDialog(
                        new VipToolException("TextFile: File "
                                + file_.getFile().getAbsolutePath()
                                + " couldn't be loaded!"));
            }
        }
    }
}
