package viptool.impl.util;

import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.Iterator;
import java.util.Vector;

import viptool.impl.project.gui.ComparatorJustAlphabet;

public class SortedVector extends Vector{

	private boolean autosort_ = true;
	private boolean sorted_ = true;
	private Comparator comparator_ = new ComparatorJustAlphabet();
	
	public boolean add(Object element)
	{
		if(autosort_)
		{
			if(sorted_ == false)
			{
				sortVector();
			}
			int pos = findIndexToInsert(element);
			super.insertElementAt(element, pos);
			return true;
		}
		else
		{
			return super.add(element);
		}
	}
	
    public void add(int index, Object element)
    {
    	if(autosort_)
    	{
    		add(element);
    	}
    	else
    	{	
    		sorted_ = false;
    		super.add(index, element);
    	}
    }
    
    public boolean addAll(Collection collection)
    {
    	if(autosort_)
    	{
    		Iterator iter = collection.iterator();
    		while(iter.hasNext())
    		{
    			add(iter.next());
    		}
    		return true;
    	}
    	else
    	{
    		sorted_ = false;
    		return super.addAll(collection);
    	}
    }
    
    public boolean addAll(int index, Collection collection)
    {
    	if(autosort_)
    	{
    		return addAll(collection);
    	}
    	else
    	{
    		sorted_ = false;
    		return super.addAll(index, collection);
    	}
    }
    
    public void addElement(Object element)
    {
    	if(autosort_)
    	{
    		add(element);
    	}
    	else
    	{
    		sorted_ = false;
    		super.addElement(element);
    	}
    }
    
    public void insertElementAt(Object element, int index)
    {
    	if(autosort_)
    	{
    		add(element);
    	}
    	else
    	{
    		sorted_ = false;
    		super.insertElementAt(element, index);
    	}
    }
    
    public void setComparator(Comparator comparator)
    {
    	comparator_ = comparator;
    	sortVector();
    }
    
    public void sortVector()
    {
    	Object[] array = super.toArray();
    	Arrays.sort(array, comparator_);
    	this.clear();
    	for(int i=0; i<array.length; i++)
    	{
    		super.add(array[i]);
    	}
    	sorted_ = true;
    }
    
    public void enableAutoSort(boolean enable)
    {
    	autosort_ = enable;
    }
    
    private int findIndexToInsert(Object element)
    {
    	if(size() == 0)
    	{
    		return 0;
    	}
    	
    	for(int i=0; i<size(); i++)
    	{
    		if(comparator_.compare(get(i), element) >= 0)
    		{
    			return i;
    		}
    	}
    	return size();
    }
}
