package viptool.impl.util.resources;

import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;

import viptool.base.error.ResourceLocateException;
import viptool.base.error.VipToolException;
import viptool.impl.base.Workbench;
import viptool.impl.extension.extensionloader.ExtensionLoader;
import viptool.impl.logger.Logger;
import viptool.util.resources.AbstractResourceLoader;

public class ResourceLoader extends AbstractResourceLoader
{
    private static ResourceLoader instance_ = null;

    private ResourceLoader()
    {
    }

    public static synchronized ResourceLoader getInstance()
    {
        if(instance_ == null)
        {
            instance_ = new ResourceLoader();
        }

        return instance_;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // 
    // <><><><><><><><><><><><><><><><><><><><><>

    public URI getResource(String resource)
    {
        URI resourceURI;

        Logger.getInstance().debug(
                "Searching in '" + resource + "' 'viptool/util/resources/'...");
        resourceURI = getResource("viptool/util/resources/", resource);
        if(resourceURI == null)
        {
            // try the own core path
            Logger.getInstance().debug(
                    "Searching '" + resource
                            + "' in 'viptool/impl/util/resources/'...");
            resourceURI = getResource("viptool/impl/util/resources/", resource);
        }
        if(resourceURI == null)
        {
            // try to find without base
            Logger.getInstance().debug(
                    "Searching '" + resource + "' in all paths...");
            resourceURI = getResource("", resource);
        }

        return resourceURI;
    }

    public URI getResource(String base, String resource)
    {
        URL resourceURL;

        resourceURL = ExtensionLoader.getInstance()
                .getResource(base + resource);

        try
        {
            if(resourceURL != null)
            {
                Logger.getInstance().debug(
                        "Resource '" + resourceURL + "' found.");
                return resourceURL.toURI();
            }

            return null;
        }
        catch (URISyntaxException exception)
        {
            Workbench.getInstance().showExceptionInDialog(
                    new VipToolException(exception));

            throw new ResourceLocateException(exception);
        }
    }
}
