package viptool.job;

/**
 * A progress monitor is used to indiciate the progress of a job. A job calls
 * methods from this interface to indicate a new state.
 */
public interface IProgressMonitor
{
    /**
     * Called to initialize the progress monitor. The parameter indicates how
     * many steps the job needs to complete. If the count of steps is unknown,
     * -1 should be given.<br>
     * This method must be called only once.
     * 
     * @param steps
     *            count of steps that are needed to complete the job
     */
    public void start(int steps);

    /**
     * Increments the progress monitor by one.
     */
    public void step();

    /**
     * Sets the position of the progress monitor to position. The position must
     * be greater or equal than zero and lower or equal to the step count given
     * to the start method.
     * 
     * @param position
     *            the position to which the progress should be set
     */
    public void setPosition(int position);

    /**
     * Increments the progress monitor by one and also sets a description of
     * what is currently being done by the job.
     * 
     * @param description
     *            description what the job actual does
     */
    public void step(String description);

    /**
     * Sets a description with information about what is currently being done by
     * the job
     * 
     * @param description
     *            description what the job actual does
     */
    public void setDescription(String description);

    /**
     * Ends up the progress. This increments the position to the count of steps
     * that were given to the start method. This also marks the progress as
     * finished, so the other methods are deactivated. <br>
     * This method must be called only once.
     */
    public void end();
}
