package viptool.job;

/**
 * This is the class for all jobs in viptool. Jobs are executed by the
 * IJobScheduler. Every job is running in is own thread. Jobs have two running
 * properties: priority and mode.<br>
 * The priority determines the priority of the thread. The mode is used by the
 * viptool: Only one foreground job can run at a time, but many background jobs
 * can ran simultaneous. It is also possible to run one foreground job while a
 * background job is running.<br>
 */
abstract public class Job implements IJob
  {
    private String name_;

    private String description_;

    private int    mode_;

    private int    priority_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructor
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Create a job object. This object has information about its name, a
     * description of the job, its mode and its priority. The mode must be one
     * of the MODE_... flags defined in the IJob interface. The priority must be
     * one of the PRIORITY_... flags defined in the IJob interface.
     * 
     * @param name
     *          the name of the job
     * 
     * @param description
     *          a description of the job
     * 
     * @param mode
     *          the running mode of the job
     * 
     * @param priority
     *          the priority for the job
     */
    public Job(String name, String description, int mode, int priority)
      {
        name_ = name;
        description_ = description;
        mode_ = mode;
        priority_ = priority;
      }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Implementing the IJob Interface
    // <><><><><><><><><><><><><><><><><><><><><>
    /* @see viptool.job.IJob */
    public String getName()
      {
        return name_;
      }

    /* @see viptool.job.IJob */
    public String getDescription()
      {
        return description_;
      }

    /* @see viptool.job.IJob */
    public int getMode()
      {
        return mode_;
      }

    /* @see viptool.job.IJob */
    public int getPriority()
      {
        return priority_;
      }
  }
