package viptool.job.event;

/**
 * The job listener is added to jobs so objects can listen to jobs state.
 * Listeners will be notified if the job was cancelled or is ready with its
 * work.
 */
public interface IJobListener
{
    /**
     * Called to notify the listener that the job was started and will begin to
     * work now. The event object contains additional information about the job.
     * 
     * @param jobEvent
     *            a job event with additional information
     */
    public void jobStarted(JobEvent jobEvent);

    /**
     * This method is called if the job is ready with its work and has finished.
     * The event object contains additional information about the job.
     * 
     * @param jobEvent
     *            a job event with additional information
     */
    public void jobFinished(JobEvent jobEvent);

    /**
     * This method is called if the job was cancelled by a call to the cancel()
     * method of the job.
     * 
     * @param jobEvent
     *            a job event with additional information
     */
    public void jobCancelled(JobEvent jobEvent);
}
