package viptool.job.event;

/**
 * The job scheduler listener is added to the job scheduler so objects can 
 * listen to jobs state. Listeners will be notified if the job was cancelled or 
 * is ready with its work.
 */
public interface IJobSchedulerListener 
{
    /**
     * Called to notify the listener that a job has been added to the list of 
     * waiting jobs. The given integer tells the listener wether the job has
     * been added to the foreground or background waiting list by using the
     * final mode variables of the class IJob.
     * 
     * @param mode
     *          has to be one of the final variables MODE_* defined in the class
     *          IJob to tell the listener which list has been modified 
     *          (foreground or background waiting jobs)
     */
    public void newJobWaiting(int mode);
  
    /**
     * Called to notify the listener that a job was started and will begin to
     * work now. The event object contains additional information about the job.
     * 
     * @param jobEvent
     *            a job event with additional information
     */
    public void jobStarted(JobEvent jobEvent);

    /**
     * This method is called if a job is ready with its work and has finished.
     * The event object contains additional information about the job.
     * 
     * @param jobEvent
     *            a job event with additional information
     */
    public void jobFinished(JobEvent jobEvent);

    /**
     * This method is called if a job was cancelled by a call to the cancel()
     * method of the job.
     * 
     * @param jobEvent
     *            a job event with additional information
     */
    public void jobCancelled(JobEvent jobEvent);
    
}
