package viptool.logger;

import java.io.File;
import java.util.HashSet;
import java.util.Iterator;

import viptool.logger.event.ILoggerChangedListener;

public abstract class AbstractLogger implements ILogger
{
    private HashSet              listeners_;
    
    public AbstractLogger()
    {
        listeners_ = new HashSet();
    }
    

    /**
     * Notifies the listener, that a LogObject was added to the logger.
     * 
     * @param logObject
     *            the LogObject, that was added to the logger
     */
    protected void fireLogObjectAddedEvent(LogObject logObject)
    {
        Iterator it = listeners_.iterator();

        while (it.hasNext())
        {

            ((ILoggerChangedListener) it.next()).logObjectAdded(logObject);
        }
    }
    
    public void addLoggerChangedListener(ILoggerChangedListener listener)
    {
        if(listener == null)
        {

            return;
        }

        listeners_.add(listener);
    }

    public void log(String msg)
    {
        log(ILogger.DEFAULT_LEVEL, msg);
    }
    
    public void log(String msg, boolean conditon)
    {
        if(conditon)
        {
            log(msg);
        }
    }
    
    public void debug(String msg)
    {
        log(ILogger.DEBUG_LEVEL, msg);
    }
    
    public void debug(String msg, boolean conditon)
    {
        if(conditon)
        {
            debug(msg);
        }
    }
    
    public void warning(String msg)
    {
        log(ILogger.WARRNING_LEVEL, msg);
    }
    
    public void warning(String msg, boolean conditon)
    {
        if(conditon)
        {
            debug(msg);
        }
    }
    
    public void info(String msg)
    {
        log(ILogger.INFO_LEVEL, msg);
    }
    
    public void info(String msg, boolean conditon)
    {
        if(conditon)
        {
            info(msg);
        }
    }

    public void error(String msg)
    {
        log(ILogger.ERROR_LEVEL, msg);
    }
    
    public void error(String msg, boolean conditon)
    {
        if(conditon)
        {
            error(msg);
        }
    }

    public void removeLoggerChangedListener(ILoggerChangedListener listener)
    {
        listeners_.remove(listener);
    }

    public void setFile(File file)
    {
        throw new UnsupportedOperationException();
    }
    
}
