package viptool.logger;

import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * This class is necessary for storing information of messages in you 
 * ConsoleLogger. All important information are centralized in one Object.
 */
public class LogObject {

	private Date date_;
	
	private int level_;
	
	private String msg_;
	
	// <><><><><><><><><><><><><><><><><><><><><>
	//  Constructors
	// <><><><><><><><><><><><><><><><><><><><><>
	
	/**
	 * Create a new LogObject with information, that are stored in the 
	 * ConsoleLogger. It includes a message with its date and its level of
	 * importance.
	 */
	public LogObject(int level, String msg)  {
		
		date_  = new Date();
		level_ = level;
		msg_   = msg;
	}
	
	// <><><><><><><><><><><><><><><><><><><><><>
	//  Sopporting Methods
	// <><><><><><><><><><><><><><><><><><><><><>

	/**
	 * Get the DateObject of the LogObject.
	 * 
	 * @return the DateObject
	 */
	public Date getDate()  {
		
		return date_;
	}
	
	/**
	 * Get the Level of the LogObject as an Integer.
	 * 
	 * @return the level of the LogObject
	 */
	public int getLevel()  {
		
		return level_;
	}
	
	/**
	 * Get the level of the LogObject as a String
	 * 
	 * @return the level of the LogObject
	 */
	private String getLevelString()  {
		
		String levelStr;
		
		if (level_ == ILogger.INFO_LEVEL)  {
			levelStr = "INFO";
		}
		else if (level_ == ILogger.WARRNING_LEVEL)  {
			levelStr = "WARNING";
		}
		else if (level_ == ILogger.ERROR_LEVEL)  {
			levelStr = "ERROR";
		}
        else if (level_ == ILogger.DEFAULT_LEVEL) {
            levelStr = "MESSAGE";
        }
        else if (level_ == ILogger.DEBUG_LEVEL) {
            levelStr = "DEBUG";
        }
		else {
			levelStr = "unknown level";
		}
		
		return levelStr;
	}
	
	/**
	 * Get the message of the LogObject.
	 * 
	 * @return the message of the LogObject
	 */
	public String getMessage()  {
		
		return msg_;
	}
	
	/**
	 * This converts the LogObject into a String. The user of this method has the
	 * possibility to define the format and the order of the output.
	 * Espcially the format of the date can be choosen.
	 * If the result of the methode should include the message, the input string
	 * must include {message}.
	 * If the result of the methode should include the level, the input string
	 * must include {level}.
	 * If the result of the methode should include the date, the input string
	 * must include {date:...}. Instead of the three points you have to write the
	 * format of the date. After ':' there MUST NOT be a space.
	 * 
	 * @param format
	 * 		the text, that should include the LogObject.
	 * 		This text must be in the correct format to get a output, that makes
	 * 		sense
	 * 		e.g.:  "At {date:EEE, MMM d, ''yy} you got the error {message} with
	 * 				  the level {level}"
	 * 		output:"At Wed, Jul 4, '06 you got the error NullPointerExeption
	 * 				  with the level ERROR."
	 * @return
	 * 		the LogObject converted in a String
	 */
	public String toString (String format)  {
		
		StringBuffer buffer = new StringBuffer(format);
		String levelStr = getLevelString();
		SimpleDateFormat simpleDateFormat;
		
		int messageIndex = buffer.indexOf("{message}");
		
		if (messageIndex != -1)  {
			
			buffer.replace(messageIndex, messageIndex + 9, msg_);
		}
		
		int levelIndex = buffer.indexOf("{level}");
		
		if (levelIndex != -1)  {
			
			buffer.replace(levelIndex, levelIndex + 7, levelStr);
		}
		
		int dateIndex = buffer.indexOf("{date");
		int lastDateIndex = buffer.indexOf("}", dateIndex);
		int dateFormatIndex = buffer.indexOf(":", dateIndex);
		
		String dateFormat = buffer.substring(dateFormatIndex + 1, lastDateIndex);
		
		simpleDateFormat = new SimpleDateFormat(dateFormat);
		
		String dateStr = simpleDateFormat.format(date_);
		if (dateIndex != -1)  {
			
			buffer.replace(dateIndex, lastDateIndex + 1, dateStr);
		}
		
		return buffer.toString();
	}
}
