package viptool.project;

import java.io.File;

import javax.swing.Icon;

public interface IProjectGuiLinker {
	
	public boolean isOpen(IProjectTreeNode node);
	
	public void makeVisible(IAbstractFile file);
	
		
	/**
	 * This methode returns the state of the element which is represented by the
	 * given node. It returns true if the element wasn't changed, false else.
	 * 
	 * @param node
	 * @return state of the node
	 */
	public boolean isValid(IProjectTreeNode node);
	
	/**
	 * This methode returns the icon for the given node.
	 * 
	 * @param node
	 * @return icon for the node
	 */
	public Icon getIconForNode(IProjectTreeNode node);
	
	/**
	 * This methode copies the diskfile which is represented by the given node
	 * to the given target or the parent of the given target if the target isn't
	 * a folder. The copied file will be registerd as new element by the
	 * project.
	 * 
	 * @param node
	 *            node which file shoud be copied
	 * @param target
	 *            target for the copy-operation
	 * @return true is the file was copied, false else
	 */
	public boolean copy(IProjectTreeNode node, IProjectTreeNode target);
	
	/**
	 * This methode creates a new folder with the given name as child of the
	 * given parent. If the parent isn't a folder, the new folder will be
	 * created as brother to the given parent.
	 * 
	 * @param parent
	 *            parent of the new folder
	 * @param name
	 *            name of the new folder
	 * @return true if the folder was created, false else
	 */
	public boolean createFolder(IProjectTreeNode parent, String name);
	
	/**
	 * This methode deletes the diskfile of the given node and remove the
	 * element form the project.
	 * 
	 * @param node
	 *            node which should be deleted
	 * @return true if the node was deleted, false else
	 */
	public boolean delete(IProjectTreeNode node);
	
	/**
	 * This methode returns the diskfile as java.io.File which is represented by
	 * the given node.
	 * 
	 * @param node
	 * @return file which is represented by the given node
	 */
	public File getFile(IProjectTreeNode node);
	
	/**
	 * This methode returns the abstract file which is represented by the given
	 * node.
	 * 
	 * @param node
	 * @return abstract file which is represented by the given node
	 */
	public IAbstractFile getAbstractFile(IProjectTreeNode node);
	
	/**
	 * This methode imports the given file, that means the file will be copied
	 * into through the parent given folder and the new file will be registerd
	 * as element of the project of the given parent.
	 * 
	 * @param file
	 *            diskfile which should be imported
	 * @param parent
	 *            parentfolder into which the file will be copied
	 * @return true if the file was imported, false else
	 */
	public boolean importFile(File file, IProjectTreeNode parent);
	
	/**
	 * This methode moves the given node to the new parent. This means the
	 * diskfile which is represented by the given node will be copied into the
	 * new folder and the old diskfile will be deleted.
	 * 
	 * @param node
	 *            node which should be moved
	 * @param target
	 *            new parent for the element
	 * @return true if move was successfull, false else
	 */
	public boolean move(IProjectTreeNode node, IProjectTreeNode target);
	
	/**
	 * The methode run runs the given node. This means if the node is a file it
	 * will be opened in the defaulteditor, if the node is a folder the folder
	 * will be updated.
	 * 
	 * @param node
	 *            node which should be opened/updated
	 */
	public void run(IProjectTreeNode node);
	
	/**
	 * This mehtode returns the absolut path of the projectfolder to which the
	 * given node belongs to.
	 * 
	 * @param node
	 *            node from which the project is searched
	 * @return path of the project-folder
	 */
	public String getProjectPath(IProjectTreeNode node);
	
	/**
	 * This methode adds the file which is represented by the given node to the
	 * blacklist of the project to which the given node belongs to.
	 * 
	 * @param node
	 */
	public void addToBlacklist(IProjectTreeNode node);
	
	/**
	 * This mehtode sends a close message to all editors which are registert at
	 * the abstractfile which is represented by this node. This means all open
	 * windows which show this file will be closed.
	 * 
	 * @param node
	 */
	public void close(IProjectTreeNode node);
	
	/**
	 * This methode closes the project to which the given node belongs. This
	 * means all open editors which show files from this project will be closed.
	 * 
	 * @param node
	 */
	public void closeProject(IProjectTreeNode node);
	
	
	/**
	 * This methode calls the editor who has the lock of the file which is
	 * represented by the given node that changes should be saved.
	 * 
	 * @param node
	 */
	public void save(IProjectTreeNode node);
	
	/**
	 * This methode saves all changes on all open files which belongs to the
	 * project the given node belongs to.
	 * 
	 * @param node
	 */
	public void saveProject(IProjectTreeNode node);
	
	
	/**
	 * This methode calls the Workspaceloader to save the current workspace.
	 * 
	 */
	public void saveWorkspace();
	
	/**
	 * This methode retruns the project to which the given node belongs to.
	 * 
	 * @param node
	 *            node form the project
	 * @return project to the node
	 */
	public IProject getProjectToNode(IProjectTreeNode node);

	/**
	 * This methode returns the Project of the selected element at the tree or
	 * null if no element is selected
	 * 
	 * @return IProject selected Project
	 */
	public IProject getSelectedProject();

	/**
	 * This methode returns the Folder of the selected element at the tree or
	 * null if no element is selected
	 * 
	 * @return IProjectFolder selected ProjectFolder
	 */
	public IAbstractFolder getSelectedFolder();

	/**
	 * This methode returns the selected element at the tree or null if no
	 * element is selected
	 * 
	 * @return IProject selected Project
	 */
	public IAbstractFile getSelectedElement();

	public boolean isProject(IAbstractFile file);
}
