package viptool.project;

import java.util.Enumeration;

import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeNode;


/**
 * This interface describes the functionality of our TreeNodes.
 * THIS INTERFACES IS USED JUST FOR VIPTOOL-INTERNAL COMMUNICATION. THE TREE
 * ALWAYS REPRESENTS THE PROJECT. MANIPULATE THIS
 */
public interface IProjectTreeNode extends MutableTreeNode, TreeNode{

	/**
	 * This methode returns an Enumeration over all childnodes of this node, or null
	 * it the node has no childs.
	 */
	public Enumeration children();
	
	/**
	 * Returns true if the node is a folder, else false.
	 */
	public boolean getAllowsChildren();
	
	/**
	 * Returns the child at position childIndex.
	 */
	public TreeNode getChildAt(int childIndex);
	
	/**
	 * Returns the number of childs of this node. If the child has no nodes
	 * it returns 0 even if the node didn't allows childs.
	 */
	public int getChildCount();
	
	/**
	 * Returns the index-number of the given node.
	 */
	public int getIndex(TreeNode node);
	
	/**
	 * Returns the parent of this node.
	 */
	public TreeNode getParent();
	
	/**
	 * Returns true if this node has no childs.
	 */
	public boolean isLeaf();
	
	/**
	 * Inserts the node child as child of this node. The index will be ignored, because
	 * the comparator gives the position.
	 */
	public void insert(MutableTreeNode child, int index);
	
	/**
	 * Removes the child at position index.
	 */
	public void remove(int index);
	
	/**
	 * Removes the child node.
	 */
	public void remove(MutableTreeNode node);
	
	/**
	 * Removes this node from its parent.
	 */
	public void removeFromParent();
	
	/**
	 * Adds this node to a new parent. This means it will be remove from
	 * its old parent if there is one and will be added as child to the node
	 * parent.
	 */
	public void setParent(MutableTreeNode parent);
	
	/**
	 * Set a userobject. The return-value of the toString()-Methode form this object
	 * will be used as name for this node.
	 */
	public void setUserObject(Object object);
	
	//----------------------further Methodes------------------------------------------
	
	/**
	 * Sets or removes the flag that this node represents a project. This is used to select
	 * the right icon for the node and defines if the node can have childs.
	 */
	public void setFolder(boolean isFolder);
		
	/**
	 * Sets or removes the flag that this node represents a project. This is used to select
	 * the right icon for the node and defines if the node can have childs.
	 */
	public void setProject(boolean isProject);
	
	/**
	 * Sets or removes the flag that this node represents a workspace. This is used to select
	 * the right icon for the node and defines if the node can have childs.
	 */
	public void setWorkspace(boolean isWorkspace);
	
	public boolean isFolder();
	
	public boolean isProject();
	
	public boolean isWorkspace();
	
	public void update();
	
	public IProjectTreeModel getModel();
}
