package viptool.project.event;

/**
 * A project listener is notified by the project if files were added to or
 * removed from the project.
 */
public interface IProjectListener
{
    /**
     * Called if a file was added to the listener. The given event object stores
     * all informations about the file.
     * 
     * @param event
     *            the event object with additional information
     */
    public void fileAdded(ProjectFileEvent event);

    /**
     * Called if a file was removed form the listener. The given event object
     * stores all information about the file.
     * 
     * @param event
     *            the event object with additional information
     */
    public void fileRemoved(ProjectFileEvent event);

    /**
     * Called if a file was changed. The file event contains several additional
     * informations. If the abstract folder is not null, the abstract folder
     * must be updated. If the abstract file is not null, the abstract file must
     * be updated. Otherwise the project was changed.
     * 
     * @param event
     *            the event object with additional information
     */
    public void fileChanged(ProjectFileEvent event);
}
