package viptool.project.event;

import java.util.EventObject;

import viptool.project.IAbstractFile;
import viptool.project.IAbstractFolder;
import viptool.project.IProject;

/**
 * This event is used by the project listener and is used to indicate a change
 * in the project structure, e.g. when a file is added or removed.
 */
public class ProjectFileEvent extends EventObject
{
    // the folder in which the change occurs
    private IAbstractFolder folder_;

    // the file that was added or removed
    private IAbstractFile   file_;

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Constructs a new project file event object and initializes it.
     * 
     * @param project
     *            the involved project
     * @param folder
     *            the folder of the added or removed file or null if the file is
     *            under no subfolder
     * @param file
     *            the file that was added or removed
     */
    public ProjectFileEvent(IProject project, IAbstractFolder folder,
            IAbstractFile file)
    {
        super(project);

        folder_ = folder;
        file_ = file;
    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // getter methods
    // <><><><><><><><><><><><><><><><><><><><><>
    /**
     * Returns the involved project.
     * 
     * @return the project
     */
    public IProject getProject()
    {
        return (IProject)getSource();
    }

    /**
     * Returns the involved folder or null if the file is directly in the
     * project folder.
     * 
     * @return the abstract folder or null if the project is the parent
     */
    public IAbstractFolder getAbstractFolder()
    {
        return folder_;
    }

    /**
     * Returns the file that was added or removed from the project.
     * 
     * @return the file that was added or removed
     */
    public IAbstractFile getAbstractFile()
    {
        return file_;
    }
}
