package viptool.project.wizard;

import viptool.base.IWorkbench;
import viptool.base.error.FileInterpreterProviderException;
import viptool.base.gui.component.FileNamePanel;
import viptool.base.gui.component.FolderTreePanel.FolderTreeSelection;
import viptool.base.gui.dialog.DialogConstants;
import viptool.base.gui.dialog.IStandardModalDialog;
import viptool.base.wizard.IWizardPage;
import viptool.project.FileSystem;
import viptool.project.IAbstractFolderProvider;

public class SelectFolderWizard extends NewFolderWizard
{
    // <><><><><><><><><><><><><><><><><><><><><>
    // inner classes
    // <><><><><><><><><><><><><><><><><><><><><>
    private class FolderPage extends NewFolderWizard.FolderPage
    {
        // <><><><><><><><><><><><><><><><><><><><><>
        // Constructors
        // <><><><><><><><><><><><><><><><><><><><><>
        public FolderPage()
        {
            panel_ = new FileNamePanel(workbench_);
            panel_.addFileNamePanelListener(this);
            panel_.disableEnterName();
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementation of the WizardPage interface
        // <><><><><><><><><><><><><><><><><><><><><>

        /* @see viptool.gui.wizards.WizardPage */
        public String getMainDescription()
        {
            return "Select a folder...";
        }

        // <><><><><><><><><><><><><><><><><><><><><>
        // Implementation of IFileNamePanelListener interface
        // <><><><><><><><><><><><><><><><><><><><><>

        /* @see viptool.base.gui.component.event.IFileNamePanelListener */
        public void focusGainedOnPathTree()
        {
            dialog_.setDetailedDescription("Please select a folder");
        }

        /* @see viptool.base.gui.component.event.IFileNamePanelListener */
        public void focusLost()
        {
            dialog_.setDetailedDescription(null);
        }

        /* @see viptool.base.gui.component.event.IFileNamePanelListener */
        public void nameNotValid()
        {
            dialog_.setNotice("You have to select a folder!",
                    DialogConstants.TYPE_ERROR);
            dialog_.setButtonsEnabled(DialogConstants.BUTTON_FINISH, false);
        }

        /* @see viptool.base.gui.component.event.IFileNamePanelListener */
        public void validated()
        {
            dialog_.removeNotice();
            dialog_.setButtonsEnabled(DialogConstants.BUTTON_FINISH, true);
        }

    }

    // <><><><><><><><><><><><><><><><><><><><><>
    // Constructors
    // <><><><><><><><><><><><><><><><><><><><><>
    public SelectFolderWizard(IWorkbench workbench,
            IAbstractFolderProvider provider)
    {
        super(workbench, provider);
    }

    /* @see viptool.wizards.Wizard */
    public void finish()
    {
        IAbstractFolderProvider provider;
        FolderTreeSelection selection = folderPage_.getSelectedPath();

        // create the folder first, if necessary
        try
        {
            if (selection.isVirtualFolder())
            {
                provider = workbench_.getAbstractFolderProvider();

                if (selection.getParent() == null)
                {
                    if (selection.getFolder() != null)
                    {
                        provider.createNewFile(FileSystem.getInstance().createAbstractFolder(
                                selection.getProject(),
                                selection.getFolder().getFullPath()));
                    }
                }
                else
                {
                    provider.createNewFile(FileSystem.getInstance().createAbstractFolder(
                            selection.getProject(), selection.getParent(),
                            selection.getFolder().getFullPath()));
                }
            }
        }
        catch (FileInterpreterProviderException exception)
        {
            workbench_.showExceptionInDialog(exception);
            return;
        }

        if (selection.getFolder() == null)
        {
            folder_ = selection.getProject().getAbstractFile().getPathFile().getAbsolutePath();
        }
        else
        {
            folder_ = selection.getFolder().getFullPath();
        }

        finished_ = true;
    }

    /* @see viptool.wizards.Wizard */
    public synchronized IWizardPage getFirstPage(IStandardModalDialog dialog)
    {
        if (folderPage_ == null)
        {
            folderPage_ = new FolderPage();
        }

        dialog_ = dialog;
        return folderPage_;
    }

    /* @see viptool.wizards.Wizard */
    public String getTitle()
    {
        return "Select Folder";
    }
}
