package viptool.util;

import java.io.File;

import javax.swing.Icon;

import viptool.base.IWorkbench;
import viptool.base.error.FileInterpreterProviderException;
import viptool.base.error.UnsupportedFileException;
import viptool.base.io.IFileInterpreter;
import viptool.base.io.IFileInterpreterProvider;
import viptool.base.wizard.IWizard;
import viptool.project.IAbstractFile;

public class ClassFileInterpreter implements IFileInterpreterProvider{

	private String superClass_ = "java.lang.Object";
	private boolean test_ = false;
	
	public boolean canCreateNewFile(IWorkbench workbench) {
		return false;
	}

	public IFileInterpreter createFromFile(IAbstractFile file) throws FileInterpreterProviderException {
		return null;
	}

	public IFileInterpreter createNewFile(IAbstractFile file) throws FileInterpreterProviderException {
		return null;
	}

	public String getDescription() {
		return "Java Class-File";
	}

	public String getFileType() {
		return ".class";
	}

	public Icon getIcon() {
		return null;
	}

	public IWizard getNewFileWizard() {
		return null;
	}

	public boolean hasNewFileWizard() {
		return false;
	}

	/**
	 * This method returns true if:
	 * 	- the file is a valid class file and testForSuperClass is set to false (default)
	 *  - the file is a valid class file and testForSuperClass is set to true and the Class
	 *    extends the given Superclass.
	 */
	public boolean matchFormat(IAbstractFile file) throws FileInterpreterProviderException {
		File f = file.getFile();
		try {
			if(f!=null && f.exists() && !f.isDirectory() && f.getName().endsWith(".class")){
				ClassFile cf = ClassFile.getClassFile(f);
				if(test_){
					if(!cf.getParentClass().equals(superClass_)){
						return false;
					}
				}
				return true;
			}else{
				return false;
			}
		} catch (UnsupportedFileException e) {
			return false;
		}
	}
	
	/**
	 * This method disables/enables the test for a given Superclass.
	 * 
	 * @param test
	 * 		true means test for Superclass
	 */
	public void testForSuperClass(boolean test){
		test_ = test;
	}
	
	/**
	 * This method set the searched Superclass. The default value is java.lang.Object.
	 * 
	 * @param name
	 * 		full qualified class name of superclass
	 */
	public void setSuperClass(String name){
		if(name != null){
			superClass_ = name;
		}else{
			superClass_ = "java.lang.Object";
		}
	}
}
