package viptool.util;

import java.awt.Font;


/**
 * Every class that wants to be able to rename its displayed objects must
 * implement this interface. It will be tested if the new name is valid and if
 * so it can be set.
 */
public interface Renameable
  {
    /**
     * this method should be implemented in a way that allows it to return true,
     * if a MultiLineRenamer is needed, or false if a SingleLineRenamer is
     * sufficient.
     * 
     * @return true, if a MultiLineRenamer is needed, false otherwise
     */
    public boolean isMultiLine();
    /**
     * this method tests whether the new name of the object is valid or invalid
     * 
     * @param name
     *          the new name, which should be tested if it is a valid one
     * 
     * @return true, if the new name is valid
     */
    public boolean isValidName(String name);

    /**
     * method should return the font that is used in the component that displays
     * the name to change. This font should be used in the renamer, so it has
     * to be get by this method.
     * 
     * @return the font used in the component that displays the name to change
     */
    public Font getFont();
    
    /**
     * this method returns the name of the object, so that it can be displayed
     * at the beginning of the rename process
     * 
     * @return a string, which should be equal to the name of the object
     */
    public String getName();

    /**
     * this method sets the new name to the object it is called by
     * 
     * @param name
     *          the name, which should be set
     */
    public void setName(String name);
  }
